@@ -23,9 +23,13 @@ import java.awt.Component;
 import java.awt.ComponentOrientation;
 import java.awt.Dimension;
 import java.awt.EventQueue;
+import java.awt.Frame;
 import java.awt.Image;
 import java.awt.Insets;
 import java.awt.Rectangle;
+import java.awt.Window;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
 import java.awt.event.ComponentAdapter;
 import java.awt.event.ComponentEvent;
 import java.awt.event.WindowAdapter;
@@ -34,7 +38,7 @@ import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.io.File;
 import java.lang.ref.WeakReference;
-import java.util.Arrays;
+import java.util.ArrayList;
 import java.util.List;
 import java.util.Locale;
 
@@ -42,6 +46,7 @@ import javax.swing.ImageIcon;
 import javax.swing.JComponent;
 import javax.swing.JFrame;
 import javax.swing.JRootPane;
+import javax.swing.Timer;
 
 import com.eteks.sweethome3d.model.CollectionEvent;
 import com.eteks.sweethome3d.model.CollectionListener;
@@ -97,22 +102,28 @@ public class HomeFramePane extends JRootPane implements View {
    * Builds and shows the frame that displays this pane.
    */
   public void displayView() {
-    JFrame homeFrame = new JFrame() {
+    final JFrame homeFrame = new JFrame() {
       {
         // Replace frame rootPane by home controller view
         setRootPane(HomeFramePane.this);
       }
     };
     // Update frame image and title 
-    Image [] frameImages = {new ImageIcon(HomeFramePane.class.getResource("resources/frameIcon.png")).getImage(),
-                            new ImageIcon(HomeFramePane.class.getResource("resources/frameIcon32x32.png")).getImage()};
+    List<Image> frameImages = new ArrayList<Image>(3);
+    frameImages.add(new ImageIcon(HomeFramePane.class.getResource("resources/frameIcon.png")).getImage());
+    frameImages.add(new ImageIcon(HomeFramePane.class.getResource("resources/frameIcon32x32.png")).getImage());
+    if (OperatingSystem.isMacOSXLeopardOrSuperior()) {
+      frameImages.add(new ImageIcon(HomeFramePane.class.getResource("resources/frameIcon128x128.png")).getImage());
+    }
     try {
       // Call Java 1.6 setIconImages by reflection
-      homeFrame.getClass().getMethod("setIconImages", List.class)
-          .invoke(homeFrame, Arrays.asList(frameImages));
+      homeFrame.getClass().getMethod("setIconImages", List.class).invoke(homeFrame, frameImages);
     } catch (Exception ex) {
       // Call setIconImage available in previous versions
-      homeFrame.setIconImage(frameImages [0]);
+      homeFrame.setIconImage(frameImages.get(0));
+    }
+    if (OperatingSystem.isMacOSXLionOrSuperior()) {
+      MacOSXConfiguration.installToolBar(this);
     }
     updateFrameTitle(homeFrame, this.home, this.application);
     // Change component orientation
@@ -134,8 +145,14 @@ public class HomeFramePane extends JRootPane implements View {
     // Add listeners to model and frame    
     addListeners(this.home, this.application, this.controller.getHomeController(), homeFrame);
     
-    // Show frame
     homeFrame.setVisible(true);
+    // Request the frame to go to front again because closing waiting dialog meanwhile  
+    // could put in front the already opened frame  
+    EventQueue.invokeLater(new Runnable() {
+        public void run() {
+          homeFrame.toFront();
+        }
+      });
   }
   
   /**
@@ -284,18 +301,49 @@ public class HomeFramePane extends JRootPane implements View {
         && screenHeight <= screenSize.height) {
       final Rectangle frameBounds = new Rectangle(x, y, width, height);
       if (maximized != null && maximized) {
-        // Display first the frame at its maximum size to keep splitters location 
-        Insets insets = frame.getInsets();
-        frame.setSize(screenSize.width + insets.left + insets.right, 
-            screenSize.height + insets.bottom);
-        EventQueue.invokeLater(new Runnable() {
+        if (OperatingSystem.isMacOSX() 
+            && OperatingSystem.isJavaVersionGreaterOrEqual("1.7")) {
+          // Display the frame at its maximum size because calling setExtendedState to maximize 
+          // the frame moves it to the bottom left at its minimum size  
+          Insets insets = frame.getInsets();
+          frame.setSize(screenSize.width + insets.left + insets.right, 
+              screenSize.height + insets.bottom);
+        } else if (OperatingSystem.isLinux()) {
+          EventQueue.invokeLater(new Runnable() {
             public void run() {
-              // Resize to home non maximized bounds
-              frame.setBounds(frameBounds);
-              // Finally maximize
+              // Under Linux, maximize frame once it's displayed
               frame.setExtendedState(JFrame.MAXIMIZED_BOTH);
             }
           });
+        } else {
+          frame.setExtendedState(JFrame.MAXIMIZED_BOTH);
+        }
+        // Add a listener that will set the normal size when the frame leaves the maximized state
+        frame.addWindowStateListener(new WindowAdapter() {
+            @Override
+            public void windowStateChanged(WindowEvent ev) {
+              if ((ev.getOldState() == JFrame.MAXIMIZED_BOTH 
+                    || (OperatingSystem.isMacOSX() 
+                        && OperatingSystem.isJavaVersionGreaterOrEqual("1.7")
+                        && ev.getOldState() == JFrame.NORMAL))
+                  && ev.getNewState() == JFrame.NORMAL) {
+                if (OperatingSystem.isMacOSXLionOrSuperior()) {
+                  // Set back frame size later once frame reduce animation is finished 
+                  new Timer(20, new ActionListener() {
+                      public void actionPerformed(ActionEvent ev) {
+                        if (frame.getHeight() < 40) {
+                          ((Timer)ev.getSource()).stop();
+                          frame.setBounds(frameBounds);
+                        }
+                      }
+                    }).start();
+                } else {
+                  frame.setBounds(frameBounds);
+                }
+                frame.removeWindowStateListener(this);
+              }
+            }
+          });
       } else {
         // Reuse home bounds
         frame.setBounds(frameBounds);
@@ -306,6 +354,35 @@ public class HomeFramePane extends JRootPane implements View {
       frame.pack();
       frame.setSize(Math.min(screenSize.width * 4 / 5, frame.getWidth()), 
               Math.min(screenSize.height * 4 / 5, frame.getHeight()));
+      if (OperatingSystem.isMacOSX() 
+          && OperatingSystem.isJavaVersionGreaterOrEqual("1.7")) {
+        // JFrame#setLocationByPlatform does nothing under Java 7
+        int minX = Integer.MAX_VALUE;
+        int minY = Integer.MAX_VALUE;
+        int maxX = Integer.MIN_VALUE;
+        int maxY = Integer.MIN_VALUE;
+        for (Frame applicationFrame : Frame.getFrames()) {
+          if (applicationFrame.isShowing() 
+              && applicationFrame.getBackground().getAlpha() != 0) {
+            minX = Math.min(minX, applicationFrame.getX());
+            minY = Math.min(minY, applicationFrame.getY());
+            maxX = Math.max(maxX, applicationFrame.getX());
+            maxY = Math.max(maxY, applicationFrame.getY());
+          }
+        }
+        
+        if (minX == Integer.MAX_VALUE || minX >= 23) {
+          x = 0;
+        } else {
+          x = maxX + 23;
+        }
+        if (minY == Integer.MAX_VALUE || minY >= 23) {
+          y = 0;
+        } else {
+          y = maxY + 23;
+        }
+        frame.setLocation(x, y);
+      }
     }
   }
 
@@ -360,6 +437,18 @@ public class HomeFramePane extends JRootPane implements View {
           }
         }
       }
+
+      if (!frame.isVisible() 
+          && OperatingSystem.isMacOSXLionOrSuperior()) {
+        try {
+          // Call Mac OS X specific FullScreenUtilities.setWindowCanFullScreen(homeFrame, true) by reflection 
+          Class.forName("com.apple.eawt.FullScreenUtilities").
+              getMethod("setWindowCanFullScreen", new Class<?> [] {Window.class, boolean.class}).
+              invoke(null, frame, true);
+        } catch (Exception ex) {
+          // Full screen mode is not supported
+        }
+      }
     } else {
       title += " - " + application.getName(); 
       if (home.isModified() || home.isRecovered()) {
