@@ -119,8 +119,8 @@ class MacOSXConfiguration {
     final JMenuBar defaultMenuBar = defaultHomeView.getJMenuBar();
     JFrame frame = null;
     try {
-      if (OperatingSystem.isJavaVersionGreaterOrEqual("1.7")) {
-        // Application#setDefaultMenuBar does nothing under Java 7
+      if (OperatingSystem.isJavaVersionBetween("1.7", "1.7.0_60")) {
+        // Application#setDefaultMenuBar does nothing under Java 7 < 1.7.0_60
         frame = createDummyFrameWithDefaultMenuBar(homeApplication, defaultHomeView, defaultMenuBar);
       } else if (UIManager.getLookAndFeel().getClass().getName().equals(UIManager.getSystemLookAndFeelClassName())) {
         macosxApplication.setDefaultMenuBar(defaultMenuBar);
@@ -301,7 +301,7 @@ class MacOSXConfiguration {
           MacOSXConfiguration.addWindowMenu(
               homeFrame, homeFrame.getJMenuBar(), homeApplication, false);
           
-          if (OperatingSystem.isJavaVersionGreaterOrEqual("1.7")) {
+          if (OperatingSystem.isJavaVersionBetween("1.7", "1.7.0_60")) {
             // Help system to understand it should display the main menu of one of the remaining windows when a window is closed
             homeFrame.addWindowListener(new WindowAdapter() {
                 @Override
@@ -353,6 +353,23 @@ class MacOSXConfiguration {
           addWindowMenu(frame, defaultMenuBar, homeApplication, true);
         }
       });
+    homeApplication.addHomesListener(new CollectionListener<Home>() {
+        public void collectionChanged(CollectionEvent<Home> ev) {
+          if (ev.getType() == CollectionEvent.Type.DELETE) {
+            EventQueue.invokeLater(new Runnable() {
+                public void run() {
+                  if (frame.isActive()) {
+                    List<Home> homes = homeApplication.getHomes();
+                    if (homes.size() >= 1) {
+                      // Request focus in a remaining home if the dummy frame is active 
+                      homeApplication.getHomeFrame(homes.get(homes.size() - 1)).requestFocus();
+                    }
+                  }
+                }
+              });
+          }
+        }
+      });
     return frame;
   }
   
