@@ -139,6 +139,10 @@ import com.eteks.sweethome3d.tools.ExtensionsClassLoader;
  *     the action that lets the user export the edited home to a SH3D file. 
  *     <br>By default, the value of this parameter is <code>false</code>.</li>
  *     
+ *     <li><code>enableExportToCSV</code> specifies whether this applet should enable
+ *     the action that lets the user export the furniture list of the edited home to a CSV file. 
+ *     <br>By default, the value of this parameter is <code>false</code>.</li>
+ *     
  *     <li><code>enableExportToSVG</code> specifies whether this applet should enable
  *     the action that lets the user export the plan of the edited home to a SVG file. 
  *     <br>By default, the value of this parameter is <code>false</code>.</li>
@@ -179,7 +183,8 @@ public class SweetHome3DApplet extends JApplet {
   public void init() {
     if (!isJava5OrSuperior()) {
       showText(getLocalizedString("requirementsMessage"));
-    } else if (!getDocumentBase().getHost().equals(getCodeBase().getHost())) {
+    } else if (getCodeBase() != null
+               && !getDocumentBase().getHost().equals(getCodeBase().getHost())) {
       showText(getLocalizedString("unauthorizedHostError"));
     } else {
       createAppletApplication();
@@ -272,8 +277,7 @@ public class SweetHome3DApplet extends JApplet {
       Class sweetHome3DAppletClass = SweetHome3DApplet.class;
       List java3DFiles = new ArrayList();
       if (!System.getProperty("os.name").startsWith("Mac OS X")
-          || System.getProperty("java.version").startsWith("1.5")
-          || System.getProperty("java.version").startsWith("1.6")) {
+          || System.getProperty("java.version").startsWith("1.5")) {
         java3DFiles.addAll(Arrays.asList(new String [] {
             "j3dcore.jar", // Main Java 3D jars
             "vecmath.jar",
@@ -295,9 +299,13 @@ public class SweetHome3DApplet extends JApplet {
             "macosx/java3d-1.6/libjogl_desktop.jnilib",
             "macosx/java3d-1.6/libnativewindow_awt.jnilib",
             "macosx/java3d-1.6/libnativewindow_macosx.jnilib"}));
-        // Disable JOGL library loader
-        System.setProperty("jogamp.gluegen.UseTempJarCache", "false");
-        System.setProperty("com.eteks.sweethome3d.j3d.useOffScreen3DView", "true");
+        try {
+          // Disable JOGL library loader
+          System.setProperty("jogamp.gluegen.UseTempJarCache", "false");
+          System.setProperty("com.eteks.sweethome3d.j3d.useOffScreen3DView", "true");
+        } catch (AccessControlException ex) {
+          // Unsigned applet
+        }
       }
       if ("64".equals(System.getProperty("sun.arch.data.model"))) {
         java3DFiles.add("linux/x64/libj3dcore-ogl.so"); // Linux 64 bits DLLs
