@@ -49,6 +49,7 @@ import com.eteks.sweethome3d.j3d.ModelManager;
 import com.eteks.sweethome3d.j3d.TextureManager;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.InterruptedRecorderException;
+import com.eteks.sweethome3d.model.LengthUnit;
 import com.eteks.sweethome3d.model.Library;
 import com.eteks.sweethome3d.model.RecorderException;
 import com.eteks.sweethome3d.model.UserPreferences;
@@ -119,6 +120,11 @@ public final class ViewerHelper {
         public boolean isNavigationPanelVisible() {
           return "true".equalsIgnoreCase(applet.getParameter(NAVIGATION_PANEL));
         }
+        
+        @Override
+        public LengthUnit getLengthUnit() {
+          return LengthUnit.CENTIMETER;
+        }
       };
     
     // Create a view factory able to instantiate only a 3D view and a threaded task view
