@@ -25,7 +25,6 @@ import java.io.IOException;
 import java.math.BigDecimal;
 import java.net.MalformedURLException;
 import java.net.URL;
-import java.net.URLClassLoader;
 import java.security.AccessControlException;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
@@ -85,12 +84,13 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
      */
     INFORMATION("information"),
     /**
-     * The key for the tags associated to a piece of furniture (optional). 
+     * The key for the tags or keywords associated to a piece of furniture (optional). 
      * Tags are separated by commas with possible heading or trailing spaces. 
      */
     TAGS("tags"),
     /**
-     * The key for the creation date of a piece of furniture at <code>yyyy-MM-dd</code> format (optional).
+     * The key for the creation or publication date of a piece of furniture at 
+     * <code>yyyy-MM-dd</code> format (optional).
      */
     CREATION_DATE("creationDate"),
     /**
@@ -154,6 +154,15 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
      * will be considered as a door or a window. 
      */
     DOOR_OR_WINDOW("doorOrWindow"),
+    /**
+     * The key for the shape of a door or window used to cut out walls when they intersect it (optional).
+     * This shape should be defined with the syntax of the d attribute of a 
+     * <a href="http://www.w3.org/TR/SVG/paths.html">SVG path element</a>
+     * and should fit in a square spreading from (0, 0) to (1, 1) which will be 
+     * scaled afterwards to the real size of the piece. 
+     * If not specified, then this shape will be automatically computed from the actual shape of the model.  
+     */
+    DOOR_OR_WINDOW_CUT_OUT_SHAPE("doorOrWindowCutOutShape"),
     /**
      * The key for the wall thickness in centimeters of a door or a window (optional).
      * By default, a door or a window has the same depth as the wall it belongs to.
@@ -372,7 +381,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
       for (URL pluginFurnitureCatalogUrl : pluginFurnitureCatalogUrls) {
         try {        
           ResourceBundle resource = ResourceBundle.getBundle(PLUGIN_FURNITURE_CATALOG_FAMILY, Locale.getDefault(), 
-              new URLClassLoader(new URL [] {pluginFurnitureCatalogUrl}));
+              new URLContentClassLoader(pluginFurnitureCatalogUrl));
           this.libraries.add(0, new DefaultLibrary(pluginFurnitureCatalogUrl.toExternalForm(), 
               UserPreferences.FURNITURE_LIBRARY_TYPE, resource));
           readFurniture(resource, pluginFurnitureCatalogUrl, furnitureResourcesUrlBase, identifiedFurniture);
@@ -397,7 +406,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
     return Collections.unmodifiableList(this.libraries);
   }
 
-  private static final Map<File,URL> pluginFurnitureCatalogUrlUpdates = new HashMap<File,URL>(); 
+  private static final Map<File,URL> pluginFurnitureCatalogUrlUpdates = new HashMap<File, URL>(); 
   
   /**
    * Reads plug-in furniture catalog from the <code>pluginFurnitureCatalogFile</code> file. 
@@ -405,24 +414,26 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
   private void readPluginFurnitureCatalog(File pluginFurnitureCatalogFile,
                                           List<String> identifiedFurniture) {
     try {
-      URL pluginFurnitureCatalogUrl = pluginFurnitureCatalogFile.toURI().toURL();
+      final URL pluginFurnitureCatalogUrl;
       long urlModificationDate = pluginFurnitureCatalogFile.lastModified();
-      URL urlUpdate = pluginFurnitureCatalogUrlUpdates.get(pluginFurnitureCatalogUrl.toString());
+      URL urlUpdate = pluginFurnitureCatalogUrlUpdates.get(pluginFurnitureCatalogFile);
       if (pluginFurnitureCatalogFile.canWrite()
           && (urlUpdate == null 
               || urlUpdate.openConnection().getLastModified() < urlModificationDate)) {
         // Copy updated resource URL content to a temporary file to ensure furniture added to home can safely 
         // reference any file of the catalog file even if its content is changed afterwards
-        TemporaryURLContent contentCopy = TemporaryURLContent.copyToTemporaryURLContent(new URLContent(pluginFurnitureCatalogUrl));
+        TemporaryURLContent contentCopy = TemporaryURLContent.copyToTemporaryURLContent(new URLContent(pluginFurnitureCatalogFile.toURI().toURL()));
         URL temporaryFurnitureCatalogUrl = contentCopy.getURL();
         pluginFurnitureCatalogUrlUpdates.put(pluginFurnitureCatalogFile, temporaryFurnitureCatalogUrl);
         pluginFurnitureCatalogUrl = temporaryFurnitureCatalogUrl;
       } else if (urlUpdate != null) {
         pluginFurnitureCatalogUrl = urlUpdate;
+      } else {
+        pluginFurnitureCatalogUrl = pluginFurnitureCatalogFile.toURI().toURL();
       }
       
-      ResourceBundle resourceBundle = ResourceBundle.getBundle(PLUGIN_FURNITURE_CATALOG_FAMILY, Locale.getDefault(), 
-          new URLClassLoader(new URL [] {pluginFurnitureCatalogUrl}));
+      final ClassLoader urlLoader = new URLContentClassLoader(pluginFurnitureCatalogUrl);
+      ResourceBundle resourceBundle = ResourceBundle.getBundle(PLUGIN_FURNITURE_CATALOG_FAMILY, Locale.getDefault(), urlLoader);
       this.libraries.add(0, new DefaultLibrary(pluginFurnitureCatalogFile.getCanonicalPath(), 
           UserPreferences.FURNITURE_LIBRARY_TYPE, resourceBundle));
       readFurniture(resourceBundle, pluginFurnitureCatalogUrl, null, identifiedFurniture);
@@ -602,6 +613,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
     String currency = getOptionalString(resource, PropertyKey.CURRENCY.getKey(index), null);
 
     if (doorOrWindow) {
+      String doorOrWindowCutOutShape = getOptionalString(resource, PropertyKey.DOOR_OR_WINDOW_CUT_OUT_SHAPE.getKey(index), null);     
       float wallThicknessPercentage = getOptionalFloat(
           resource, PropertyKey.DOOR_OR_WINDOW_WALL_THICKNESS.getKey(index), depth) / depth;
       float wallDistancePercentage = getOptionalFloat(
@@ -609,7 +621,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
       Sash [] sashes = getDoorOrWindowSashes(resource, index, width, depth);
       return new CatalogDoorOrWindow(id, name, description, information, tags, creationDate, grade, 
           icon, planIcon, model, width, depth, height, elevation, movable, 
-          wallThicknessPercentage, wallDistancePercentage, sashes, 
+          doorOrWindowCutOutShape, wallThicknessPercentage, wallDistancePercentage, sashes,
           modelRotation, creator, resizable, deformable, texturable, price, valueAddedTaxPercentage, currency);
     } else {
       LightSource [] lightSources = getLightSources(resource, index, width, depth, height);
