@@ -26,6 +26,7 @@ import java.io.InputStream;
 import java.io.InterruptedIOException;
 import java.io.ObjectOutputStream;
 import java.io.OutputStream;
+import java.net.URISyntaxException;
 import java.net.URL;
 import java.net.URLEncoder;
 import java.util.ArrayList;
@@ -154,26 +155,38 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
         URL zipUrl = urlContent.getJAREntryURL();
         String entryName = urlContent.getJAREntryName();
         int lastSlashIndex = entryName.lastIndexOf('/');
-        String entryDirectory = entryName.substring(0, lastSlashIndex + 1);
-        // Write in home stream each zipped stream entry that is stored in the same directory  
-        for (String zipEntryName : getZipUrlEntries(zipUrl)) {
-          if (zipEntryName.startsWith(entryDirectory)) {
-            Content siblingContent = new URLContent(new URL("jar:" + zipUrl + "!/" 
-                + URLEncoder.encode(zipEntryName, "UTF-8").replace("+", "%20")));
-            writeZipEntry(zipOut, entryNameOrDirectory + zipEntryName.substring(lastSlashIndex), siblingContent);
+        if (lastSlashIndex != -1) {
+          // Consider content is a multi part resource only if it's in a subdirectory
+          String entryDirectory = entryName.substring(0, lastSlashIndex + 1);
+          // Write in home stream each zipped stream entry that is stored in the same directory  
+          for (String zipEntryName : getZipUrlEntries(zipUrl)) {
+            if (zipEntryName.startsWith(entryDirectory)) {
+              Content siblingContent = new URLContent(new URL("jar:" + zipUrl + "!/" 
+                  + URLEncoder.encode(zipEntryName, "UTF-8").replace("+", "%20")));
+              writeZipEntry(zipOut, entryNameOrDirectory + zipEntryName.substring(lastSlashIndex), siblingContent);
+            }
           }
+        } else {
+          // Consider the content as not a multipart resource
+          writeZipEntry(zipOut, entryNameOrDirectory, urlContent);
         }
       } else {
         // This should be the case only when resource isn't in a JAR file during development
-        File contentFile = new File(urlContent.getURL().getFile());
-        File parentFile = new File(contentFile.getParent());
-        File [] siblingFiles = parentFile.listFiles();
-        // Write in home stream each file that is stored in the same directory  
-        for (File siblingFile : siblingFiles) {
-          if (!siblingFile.isDirectory()) {
-            writeZipEntry(zipOut, entryNameOrDirectory + "/" + siblingFile.getName(), 
-                new URLContent(siblingFile.toURI().toURL()));
+        try {
+          File contentFile = new File(urlContent.getURL().toURI());
+          File parentFile = new File(contentFile.getParent());
+          File [] siblingFiles = parentFile.listFiles();
+          // Write in home stream each file that is stored in the same directory  
+          for (File siblingFile : siblingFiles) {
+            if (!siblingFile.isDirectory()) {
+              writeZipEntry(zipOut, entryNameOrDirectory + "/" + siblingFile.getName(), 
+                  new URLContent(siblingFile.toURI().toURL()));
+            }
           }
+        } catch (URISyntaxException ex) {
+          IOException ex2 = new IOException();
+          ex2.initCause(ex);
+          throw ex2;
         }
       }
     } else {
@@ -316,7 +329,11 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
               ResourceURLContent resourceUrlContent = (ResourceURLContent)urlContent;
               if (resourceUrlContent.isMultiPartResource()) {
                 // If content is a resource coming from a JAR file, retrieve its file name
-                subEntryName = entryName.substring(entryName.lastIndexOf('/'));
+                int lastSlashIndex = entryName.lastIndexOf('/');
+                if (lastSlashIndex != -1) {
+                  // Consider content is a multi part resource only if it's in a subdirectory
+                  subEntryName = entryName.substring(lastSlashIndex);
+                }
               }
             } else {
               // Retrieve entry name in zipped stream
@@ -327,7 +344,13 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
             // If content is a resource coming from a directory (this should be the case 
             // only when resource isn't in a JAR file during development), retrieve its file name
             if (resourceUrlContent.isMultiPartResource()) {
-              subEntryName = "/" + new File(resourceUrlContent.getURL().getFile()).getName();
+              try {
+                subEntryName = "/" + new File(resourceUrlContent.getURL().toURI()).getName();
+              } catch (URISyntaxException ex) {
+                IOException ex2 = new IOException();
+                ex2.initCause(ex);
+                throw ex2;
+              }
             }
           }
         } 
