@@ -24,7 +24,6 @@ import java.io.FileFilter;
 import java.io.IOException;
 import java.net.MalformedURLException;
 import java.net.URL;
-import java.net.URLClassLoader;
 import java.security.AccessControlException;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -194,7 +193,7 @@ public class DefaultTexturesCatalog extends TexturesCatalog {
       for (URL pluginTexturesCatalogUrl : pluginTexturesCatalogUrls) {
         try {
           ResourceBundle resource = ResourceBundle.getBundle(PLUGIN_TEXTURES_CATALOG_FAMILY, Locale.getDefault(),
-                  new URLClassLoader(new URL [] {pluginTexturesCatalogUrl}));
+              new URLContentClassLoader(pluginTexturesCatalogUrl));
           this.libraries.add(0, new DefaultLibrary(pluginTexturesCatalogUrl.toExternalForm(), 
               UserPreferences.TEXTURES_LIBRARY_TYPE, resource));
           readTextures(resource, pluginTexturesCatalogUrl, texturesResourcesUrlBase, identifiedTextures);
@@ -227,25 +226,26 @@ public class DefaultTexturesCatalog extends TexturesCatalog {
   private void readPluginTexturesCatalog(File pluginTexturesCatalogFile,
                                          List<String> identifiedTextures) {
     try {
-      URL pluginTexturesCatalogUrl = pluginTexturesCatalogFile.toURI().toURL();
+      final URL pluginTexturesCatalogUrl;;
       long urlModificationDate = pluginTexturesCatalogFile.lastModified();
       URL urlUpdate = pluginTexturesCatalogUrlUpdates.get(pluginTexturesCatalogFile);
-      boolean modifiableUrl = pluginTexturesCatalogFile.canWrite();
-      if (modifiableUrl
+      if (pluginTexturesCatalogFile.canWrite()
           && (urlUpdate == null 
               || urlUpdate.openConnection().getLastModified() < urlModificationDate)) {
         // Copy updated resource URL content to a temporary file to ensure textures used in home can safely 
         // reference any file of the catalog file even if its content is changed afterwards
-        TemporaryURLContent contentCopy = TemporaryURLContent.copyToTemporaryURLContent(new URLContent(pluginTexturesCatalogUrl));
+        TemporaryURLContent contentCopy = TemporaryURLContent.copyToTemporaryURLContent(new URLContent(pluginTexturesCatalogFile.toURI().toURL()));
         URL temporaryTexturesCatalogUrl = contentCopy.getURL();
         pluginTexturesCatalogUrlUpdates.put(pluginTexturesCatalogFile, temporaryTexturesCatalogUrl);
         pluginTexturesCatalogUrl = temporaryTexturesCatalogUrl;
       } else if (urlUpdate != null) {
         pluginTexturesCatalogUrl = urlUpdate;
+      } else {
+        pluginTexturesCatalogUrl = pluginTexturesCatalogFile.toURI().toURL();
       }
       
-      ResourceBundle resourceBundle = ResourceBundle.getBundle(PLUGIN_TEXTURES_CATALOG_FAMILY, Locale.getDefault(),
-          new URLClassLoader(new URL [] {pluginTexturesCatalogUrl}));      
+      final ClassLoader urlLoader = new URLContentClassLoader(pluginTexturesCatalogUrl);
+      ResourceBundle resourceBundle = ResourceBundle.getBundle(PLUGIN_TEXTURES_CATALOG_FAMILY, Locale.getDefault(), urlLoader);      
       this.libraries.add(0, new DefaultLibrary(pluginTexturesCatalogFile.getCanonicalPath(), 
           UserPreferences.TEXTURES_LIBRARY_TYPE, resourceBundle));
       readTextures(resourceBundle, pluginTexturesCatalogUrl, null, identifiedTextures);
