@@ -20,22 +20,19 @@
 package com.eteks.sweethome3d.io;
 
 import java.io.File;
-import java.io.IOException;
-import java.io.ObjectInputStream;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 import java.util.Locale;
 
-import com.eteks.sweethome3d.model.Content;
 import com.eteks.sweethome3d.model.FurnitureCatalog;
 import com.eteks.sweethome3d.model.LengthUnit;
+import com.eteks.sweethome3d.model.Library;
 import com.eteks.sweethome3d.model.PatternsCatalog;
 import com.eteks.sweethome3d.model.RecorderException;
 import com.eteks.sweethome3d.model.TextureImage;
 import com.eteks.sweethome3d.model.TexturesCatalog;
 import com.eteks.sweethome3d.model.UserPreferences;
-import com.eteks.sweethome3d.tools.ResourceURLContent;
 
 /**
  * Default user preferences.
@@ -72,55 +69,65 @@ public class DefaultUserPreferences extends UserPreferences {
     // Build default patterns catalog
     List<TextureImage> patterns = new ArrayList<TextureImage>();
     patterns.add(new DefaultPatternTexture("foreground"));
+    patterns.add(new DefaultPatternTexture("reversedHatchUp"));
+    patterns.add(new DefaultPatternTexture("reversedHatchDown"));
+    patterns.add(new DefaultPatternTexture("reversedCrossHatch"));
+    patterns.add(new DefaultPatternTexture("background"));
     patterns.add(new DefaultPatternTexture("hatchUp"));
     patterns.add(new DefaultPatternTexture("hatchDown"));
     patterns.add(new DefaultPatternTexture("crossHatch"));
-    patterns.add(new DefaultPatternTexture("background"));
     PatternsCatalog patternsCatalog = new PatternsCatalog(patterns);
     setPatternsCatalog(patternsCatalog);
     // Read other preferences from resource bundle
     setFurnitureCatalogViewedInTree(Boolean.parseBoolean(
         localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "furnitureCatalogViewedInTree")));
-    setNavigationPanelVisible(Boolean.parseBoolean(localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "navigationPanelVisible")));    
+    setNavigationPanelVisible(Boolean.parseBoolean(localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "navigationPanelVisible")));  
+    setAerialViewCenteredOnSelectionEnabled(Boolean.parseBoolean(getOptionalLocalizedString(localizedPreferences, "aerialViewCenteredOnSelectionEnabled", "false")));
     setUnit(LengthUnit.valueOf(localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "unit").toUpperCase(Locale.ENGLISH)));
     setRulersVisible(Boolean.parseBoolean(localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "rulersVisible")));
     setGridVisible(Boolean.parseBoolean(localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "gridVisible")));
-    setFurnitureViewedFromTop(Boolean.parseBoolean(localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "furnitureViewedFromTop")));
-    setFloorColoredOrTextured(Boolean.parseBoolean(localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "roomFloorColoredOrTextured")));
+    // Allow furnitureViewedFromTop and roomFloorColoredOrTextured to be different according to the running OS
+    String osName = System.getProperty("os.name");
+    setFurnitureViewedFromTop(Boolean.parseBoolean(getOptionalLocalizedString(localizedPreferences, "furnitureViewedFromTop." + osName, 
+        localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "furnitureViewedFromTop"))));
+    setFloorColoredOrTextured(Boolean.parseBoolean(getOptionalLocalizedString(localizedPreferences, "roomFloorColoredOrTextured." + osName,
+        localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "roomFloorColoredOrTextured"))));
     setWallPattern(patternsCatalog.getPattern(localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "wallPattern")));
+    String newWallPattern = localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "wallPattern");
+    if (newWallPattern != null) {
+      setNewWallPattern(patternsCatalog.getPattern(newWallPattern));
+    }
     setNewWallThickness(Float.parseFloat(localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "newWallThickness")));
     setNewWallHeight(Float.parseFloat(localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "newHomeWallHeight")));
-    try {
-      setNewFloorThickness(Float.parseFloat(localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "newFloorThickness")));
-    } catch (IllegalArgumentException ex) {
-      setNewFloorThickness(12);
-    }
-    try {
-      setAutoSaveDelayForRecovery(Integer.parseInt(localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "autoSaveDelayForRecovery")));
-    } catch (IllegalArgumentException ex) {
-      // Disable auto save
-      setAutoSaveDelayForRecovery(0);
-    }
+    setNewFloorThickness(Float.parseFloat(getOptionalLocalizedString(localizedPreferences, "newFloorThickness", "12")));
+    setCheckUpdatesEnabled(Boolean.parseBoolean(getOptionalLocalizedString(localizedPreferences, "checkUpdatesEnabled", "false")));
+    setAutoSaveDelayForRecovery(Integer.parseInt(getOptionalLocalizedString(localizedPreferences, "autoSaveDelayForRecovery", "0")));
     setRecentHomes(new ArrayList<String>());
-    try {
-      setCurrency(localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "currency"));
-    } catch (IllegalArgumentException ex) {
-      // Don't use currency and prices in program
-    }
+    setRecentColors(new ArrayList<Integer>());
+    setCurrency(getOptionalLocalizedString(localizedPreferences, "currency", null)); 
     for (String property : new String [] {"LevelName", "HomePieceOfFurnitureName", "RoomName", "LabelText"}) {
-      try {
-        String [] autoCompletionStrings = localizedPreferences.getLocalizedString(DefaultUserPreferences.class, "autoCompletionStrings#" + property).trim().split(",");
+      String autoCompletionStringsList = getOptionalLocalizedString(localizedPreferences, "autoCompletionStrings#" + property, null);
+      if (autoCompletionStringsList != null) {
+        String [] autoCompletionStrings = autoCompletionStringsList.trim().split(",");
         if (autoCompletionStrings.length > 0) {
           for (int i = 0; i < autoCompletionStrings.length; i++) {
             autoCompletionStrings [i] = autoCompletionStrings [i].trim();
           }
           setAutoCompletionStrings(property, Arrays.asList(autoCompletionStrings));
         }
-      } catch (IllegalArgumentException ex) {
-        // No default auto completion strings
       }
     }
   }
+  
+  private String getOptionalLocalizedString(UserPreferences localizedPreferences, 
+                                            String   resourceKey,
+                                            String   defaultValue) {
+    try {
+      return localizedPreferences.getLocalizedString(DefaultUserPreferences.class, resourceKey);
+    } catch (IllegalArgumentException ex) {
+      return defaultValue;
+    }
+  }
 
   /**
    * Throws an exception because default user preferences can't be written 
@@ -128,7 +135,7 @@ public class DefaultUserPreferences extends UserPreferences {
    */
   @Override
   public void write() throws RecorderException {
-    throw new RecorderException("Default user preferences can't be written");
+    throw new UnsupportedOperationException("Default user preferences can't be written");
   }
 
   /**
@@ -136,7 +143,7 @@ public class DefaultUserPreferences extends UserPreferences {
    */
   @Override
   public boolean languageLibraryExists(String name) throws RecorderException {
-    throw new RecorderException("Default user preferences can't manage language libraries");
+    throw new UnsupportedOperationException("Default user preferences can't manage language libraries");
   }
 
   /**
@@ -144,15 +151,15 @@ public class DefaultUserPreferences extends UserPreferences {
    */
   @Override
   public void addLanguageLibrary(String name) throws RecorderException {
-    throw new RecorderException("Default user preferences can't manage language libraries");
+    throw new UnsupportedOperationException("Default user preferences can't manage language libraries");
   }
   
   /**
-   * Throws an exception because default user preferences can't manage furniture libraries.
+   * Throws an exception because default user preferences can't manage additional furniture libraries.
    */
   @Override
   public boolean furnitureLibraryExists(String name) throws RecorderException {
-    throw new RecorderException("Default user preferences can't manage furniture libraries");
+    throw new UnsupportedOperationException("Default user preferences can't manage furniture libraries");
   }
 
   /**
@@ -160,7 +167,7 @@ public class DefaultUserPreferences extends UserPreferences {
    */
   @Override
   public void addFurnitureLibrary(String name) throws RecorderException {
-    throw new RecorderException("Default user preferences can't manage furniture libraries");
+    throw new UnsupportedOperationException("Default user preferences can't manage furniture libraries");
   }
   
   /**
@@ -168,7 +175,7 @@ public class DefaultUserPreferences extends UserPreferences {
    */
   @Override
   public boolean texturesLibraryExists(String name) throws RecorderException {
-    throw new RecorderException("Default user preferences can't manage textures libraries");
+    throw new UnsupportedOperationException("Default user preferences can't manage textures libraries");
   }
 
   /**
@@ -176,73 +183,15 @@ public class DefaultUserPreferences extends UserPreferences {
    */
   @Override
   public void addTexturesLibrary(String name) throws RecorderException {
-    throw new RecorderException("Default user preferences can't manage textures libraries");
-  }
-}
-
-/**
- * A pattern built from resources.
- * @since 3.3
- */
-class DefaultPatternTexture implements TextureImage {
-  private static final long serialVersionUID = 1L;
-
-  private final String      name;
-  private transient Content image;
-  
-  public DefaultPatternTexture(String name) {
-    this.name = name;
-    this.image = new ResourceURLContent(DefaultPatternTexture.class, "resources/patterns/" + this.name + ".png");
-  }
-  
-  /**
-   * Initializes transient fields and reads pattern from <code>in</code> stream with default reading method.
-   */
-  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
-    in.defaultReadObject();
-    try {
-      this.image = new ResourceURLContent(DefaultPatternTexture.class, "resources/patterns/" + this.name + ".png");
-    } catch (IllegalArgumentException ex) {
-      this.image = new ResourceURLContent(DefaultPatternTexture.class, "resources/patterns/foreground.png");
-    }
-  }
-
-  public String getName() {
-    return this.name;
-  }
-
-  public Content getImage() {
-    return this.image;
-  }
-  
-  public float getWidth() {
-    return 10;
-  }
-  
-  public float getHeight() {
-    return 10;
+    throw new UnsupportedOperationException("Default user preferences can't manage textures libraries");
   }
 
   /**
-   * Returns <code>true</code> if the object in parameter is equal to this pattern.
+   * Throws an exception because default user preferences don't support libraries.
+   * @since 4.0
    */
   @Override
-  public boolean equals(Object obj) {
-    if (obj == this) {
-      return true;
-    } else if (obj instanceof DefaultPatternTexture) {
-      DefaultPatternTexture pattern = (DefaultPatternTexture)obj;
-      return pattern.name.equals(this.name);
-    } else {
-      return false;
-    }
-  }
-  
-  /**
-   * Returns a hash code for this pattern.
-   */
-  @Override
-  public int hashCode() {
-    return this.name.hashCode();
+  public List<Library> getLibraries() {
+    throw new UnsupportedOperationException();
   }
 }
