@@ -43,7 +43,9 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Map.Entry;
+import java.util.MissingResourceException;
 import java.util.Properties;
+import java.util.ResourceBundle;
 import java.util.Set;
 import java.util.TreeSet;
 import java.util.concurrent.Executor;
@@ -61,9 +63,11 @@ import com.eteks.sweethome3d.model.Content;
 import com.eteks.sweethome3d.model.FurnitureCatalog;
 import com.eteks.sweethome3d.model.FurnitureCategory;
 import com.eteks.sweethome3d.model.LengthUnit;
+import com.eteks.sweethome3d.model.Library;
 import com.eteks.sweethome3d.model.PatternsCatalog;
 import com.eteks.sweethome3d.model.RecorderException;
 import com.eteks.sweethome3d.model.Sash;
+import com.eteks.sweethome3d.model.TextureImage;
 import com.eteks.sweethome3d.model.TexturesCatalog;
 import com.eteks.sweethome3d.model.TexturesCategory;
 import com.eteks.sweethome3d.model.UserPreferences;
@@ -78,54 +82,60 @@ import com.eteks.sweethome3d.tools.URLContent;
  * @author Emmanuel Puybaret
  */
 public class FileUserPreferences extends UserPreferences {
-  private static final String LANGUAGE                              = "language";
-  private static final String UNIT                                  = "unit";
-  private static final String FURNITURE_CATALOG_VIEWED_IN_TREE      = "furnitureCatalogViewedInTree";
-  private static final String NAVIGATION_PANEL_VISIBLE              = "navigationPanelVisible";
-  private static final String MAGNETISM_ENABLED                     = "magnetismEnabled";
-  private static final String RULERS_VISIBLE                        = "rulersVisible";
-  private static final String GRID_VISIBLE                          = "gridVisible";
-  private static final String FURNITURE_VIEWED_FROM_TOP             = "furnitureViewedFromTop";
-  private static final String ROOM_FLOOR_COLORED_OR_TEXTURED        = "roomFloorColoredOrTextured";
-  private static final String WALL_PATTERN                          = "wallPattern";
-  private static final String NEW_WALL_HEIGHT                       = "newHomeWallHeight";
-  private static final String NEW_WALL_THICKNESS                    = "newWallThickness";
-  private static final String NEW_FLOOR_THICKNESS                   = "newFloorThickness";
-  private static final String AUTO_SAVE_DELAY_FOR_RECOVERY          = "autoSaveDelayForRecovery";
-  private static final String AUTO_COMPLETION_PROPERTY              = "autoCompletionProperty#";
-  private static final String AUTO_COMPLETION_STRINGS               = "autoCompletionStrings#";
-  private static final String RECENT_HOMES                          = "recentHomes#";
-  private static final String IGNORED_ACTION_TIP                    = "ignoredActionTip#";  
+  private static final String LANGUAGE                                  = "language";
+  private static final String UNIT                                      = "unit";
+  private static final String EXTENSIBLE_UNIT                           = "extensibleUnit";
+  private static final String FURNITURE_CATALOG_VIEWED_IN_TREE          = "furnitureCatalogViewedInTree";
+  private static final String NAVIGATION_PANEL_VISIBLE                  = "navigationPanelVisible";
+  private static final String AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED = "aerialViewCenteredOnSelectionEnabled";
+  private static final String MAGNETISM_ENABLED                         = "magnetismEnabled";
+  private static final String RULERS_VISIBLE                            = "rulersVisible";
+  private static final String GRID_VISIBLE                              = "gridVisible";
+  private static final String FURNITURE_VIEWED_FROM_TOP                 = "furnitureViewedFromTop";
+  private static final String ROOM_FLOOR_COLORED_OR_TEXTURED            = "roomFloorColoredOrTextured";
+  private static final String WALL_PATTERN                              = "wallPattern";
+  private static final String NEW_WALL_PATTERN                          = "newWallPattern";
+  private static final String NEW_WALL_HEIGHT                           = "newHomeWallHeight";
+  private static final String NEW_WALL_THICKNESS                        = "newWallThickness";
+  private static final String NEW_FLOOR_THICKNESS                       = "newFloorThickness";
+  private static final String CHECK_UPDATES_ENABLED                     = "checkUpdatesEnabled";
+  private static final String UPDATES_MINIMUM_DATE                      = "updatesMinimumDate";
+  private static final String AUTO_SAVE_DELAY_FOR_RECOVERY              = "autoSaveDelayForRecovery";
+  private static final String AUTO_COMPLETION_PROPERTY                  = "autoCompletionProperty#";
+  private static final String AUTO_COMPLETION_STRINGS                   = "autoCompletionStrings#";
+  private static final String RECENT_COLORS                             = "recentColors";
+  private static final String RECENT_HOMES                              = "recentHomes#";
+  private static final String IGNORED_ACTION_TIP                        = "ignoredActionTip#";  
 
-  private static final String FURNITURE_NAME                        = "furnitureName#";
-  private static final String FURNITURE_CATEGORY                    = "furnitureCategory#";
-  private static final String FURNITURE_ICON                        = "furnitureIcon#";
-  private static final String FURNITURE_MODEL                       = "furnitureModel#";
-  private static final String FURNITURE_WIDTH                       = "furnitureWidth#";
-  private static final String FURNITURE_DEPTH                       = "furnitureDepth#";
-  private static final String FURNITURE_HEIGHT                      = "furnitureHeight#";
-  private static final String FURNITURE_MOVABLE                     = "furnitureMovable#";
-  private static final String FURNITURE_DOOR_OR_WINDOW              = "furnitureDoorOrWindow#";
-  private static final String FURNITURE_ELEVATION                   = "furnitureElevation#";
-  private static final String FURNITURE_COLOR                       = "furnitureColor#";
-  private static final String FURNITURE_MODEL_ROTATION              = "furnitureModelRotation#";
-  private static final String FURNITURE_STAIRCASE_CUT_OUT_SHAPE           = "furnitureStaircaseCutOutShape#"; 
-  private static final String FURNITURE_BACK_FACE_SHOWN             = "furnitureBackFaceShown#";
-  private static final String FURNITURE_ICON_YAW                    = "furnitureIconYaw#";
-  private static final String FURNITURE_PROPORTIONAL                = "furnitureProportional#";
+  private static final String FURNITURE_NAME                            = "furnitureName#";
+  private static final String FURNITURE_CATEGORY                        = "furnitureCategory#";
+  private static final String FURNITURE_ICON                            = "furnitureIcon#";
+  private static final String FURNITURE_MODEL                           = "furnitureModel#";
+  private static final String FURNITURE_WIDTH                           = "furnitureWidth#";
+  private static final String FURNITURE_DEPTH                           = "furnitureDepth#";
+  private static final String FURNITURE_HEIGHT                          = "furnitureHeight#";
+  private static final String FURNITURE_MOVABLE                         = "furnitureMovable#";
+  private static final String FURNITURE_DOOR_OR_WINDOW                  = "furnitureDoorOrWindow#";
+  private static final String FURNITURE_ELEVATION                       = "furnitureElevation#";
+  private static final String FURNITURE_COLOR                           = "furnitureColor#";
+  private static final String FURNITURE_MODEL_ROTATION                  = "furnitureModelRotation#";
+  private static final String FURNITURE_STAIRCASE_CUT_OUT_SHAPE         = "furnitureStaircaseCutOutShape#"; 
+  private static final String FURNITURE_BACK_FACE_SHOWN                 = "furnitureBackFaceShown#";
+  private static final String FURNITURE_ICON_YAW                        = "furnitureIconYaw#";
+  private static final String FURNITURE_PROPORTIONAL                    = "furnitureProportional#";
 
-  private static final String TEXTURE_NAME                          = "textureName#";
-  private static final String TEXTURE_CATEGORY                      = "textureCategory#";
-  private static final String TEXTURE_IMAGE                         = "textureImage#";
-  private static final String TEXTURE_WIDTH                         = "textureWidth#";
-  private static final String TEXTURE_HEIGHT                        = "textureHeight#";
+  private static final String TEXTURE_NAME                              = "textureName#";
+  private static final String TEXTURE_CATEGORY                          = "textureCategory#";
+  private static final String TEXTURE_IMAGE                             = "textureImage#";
+  private static final String TEXTURE_WIDTH                             = "textureWidth#";
+  private static final String TEXTURE_HEIGHT                            = "textureHeight#";
 
-  private static final String FURNITURE_CONTENT_PREFIX              = "Furniture-3-";
-  private static final String TEXTURE_CONTENT_PREFIX                = "Texture-3-";
+  private static final String FURNITURE_CONTENT_PREFIX                  = "Furniture-3-";
+  private static final String TEXTURE_CONTENT_PREFIX                    = "Texture-3-";
 
-  private static final String LANGUAGE_LIBRARIES_PLUGIN_SUB_FOLDER  = "languages";
-  private static final String FURNITURE_LIBRARIES_PLUGIN_SUB_FOLDER = "furniture";
-  private static final String TEXTURES_LIBRARIES_PLUGIN_SUB_FOLDER  = "textures";
+  private static final String LANGUAGE_LIBRARIES_PLUGIN_SUB_FOLDER      = "languages";
+  private static final String FURNITURE_LIBRARIES_PLUGIN_SUB_FOLDER     = "furniture";
+  private static final String TEXTURES_LIBRARIES_PLUGIN_SUB_FOLDER      = "textures";
 
   private static final Content DUMMY_CONTENT;
   
@@ -136,6 +146,9 @@ public class FileUserPreferences extends UserPreferences {
   private Preferences                preferences;
   private Executor                   catalogsLoader;
   private Executor                   updater;
+  private List<Library>              libraries;
+  
+  public static final String PLUGIN_LANGUAGE_LIBRARY_FAMILY = "PluginLanguageLibrary";
   
   static {
     Content dummyURLContent = null;
@@ -183,6 +196,7 @@ public class FileUserPreferences extends UserPreferences {
   public FileUserPreferences(File preferencesFolder,
                              File [] applicationFolders,
                              Executor updater) {
+    this.libraries = new ArrayList<Library>();
     this.preferencesFolder = preferencesFolder;
     this.applicationFolders = applicationFolders;
     Executor defaultExecutor = new Executor() {
@@ -238,12 +252,25 @@ public class FileUserPreferences extends UserPreferences {
     setPatternsCatalog(patternsCatalog);
 
     // Read other preferences 
-    setUnit(LengthUnit.valueOf(preferences.get(UNIT, 
-        defaultPreferences.getLengthUnit().name())));
+    LengthUnit defaultLengthUnit = defaultPreferences.getLengthUnit();
+    try {
+      // EXTENSIBLE_UNIT was added in version 4.0 to store new additional length unit 
+      // to avoid breaking program if an older version of FileUserPreferences reads new preferences
+      String extensibleUnit = preferences.get(EXTENSIBLE_UNIT, null);
+      if (extensibleUnit != null) {
+        setUnit(LengthUnit.valueOf(extensibleUnit));
+      } else {
+        setUnit(LengthUnit.valueOf(preferences.get(UNIT, defaultLengthUnit.name())));
+      }
+    } catch (IllegalArgumentException ex) {
+      setUnit(defaultLengthUnit);
+    }
     setFurnitureCatalogViewedInTree(preferences.getBoolean(FURNITURE_CATALOG_VIEWED_IN_TREE, 
         defaultPreferences.isFurnitureCatalogViewedInTree()));
     setNavigationPanelVisible(preferences.getBoolean(NAVIGATION_PANEL_VISIBLE, 
         defaultPreferences.isNavigationPanelVisible()));
+    setAerialViewCenteredOnSelectionEnabled(preferences.getBoolean(AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED, 
+        defaultPreferences.isAerialViewCenteredOnSelectionEnabled()));
     setMagnetismEnabled(preferences.getBoolean(MAGNETISM_ENABLED, true));
     setRulersVisible(preferences.getBoolean(RULERS_VISIBLE, 
         defaultPreferences.isRulersVisible()));
@@ -260,15 +287,37 @@ public class FileUserPreferences extends UserPreferences {
       // Ensure wall pattern always exists even if new patterns are added in future versions
       setWallPattern(defaultPreferences.getWallPattern());
     }
+    try {
+      if (defaultPreferences.getNewWallPattern() != null) {
+        setNewWallPattern(patternsCatalog.getPattern(preferences.get(NEW_WALL_PATTERN, 
+            defaultPreferences.getNewWallPattern().getName())));
+      }
+    } catch (IllegalArgumentException ex) {
+      // Keep new wall pattern unchanged
+    }
     setNewWallThickness(preferences.getFloat(NEW_WALL_THICKNESS, 
         defaultPreferences.getNewWallThickness()));
     setNewWallHeight(preferences.getFloat(NEW_WALL_HEIGHT,
         defaultPreferences.getNewWallHeight()));    
     setNewFloorThickness(preferences.getFloat(NEW_FLOOR_THICKNESS, 
         defaultPreferences.getNewFloorThickness()));
+    setCheckUpdatesEnabled(preferences.getBoolean(CHECK_UPDATES_ENABLED,
+        defaultPreferences.isCheckUpdatesEnabled()));
+    if (preferences.get(UPDATES_MINIMUM_DATE, null) != null) {
+      setUpdatesMinimumDate(preferences.getLong(UPDATES_MINIMUM_DATE, 0));
+    }
     setAutoSaveDelayForRecovery(preferences.getInt(AUTO_SAVE_DELAY_FOR_RECOVERY,
         defaultPreferences.getAutoSaveDelayForRecovery()));
     setCurrency(defaultPreferences.getCurrency());    
+    // Read recent colors list
+    String [] recentColors = preferences.get(RECENT_COLORS, "").split(",");
+    List<Integer> recentColorsList = new ArrayList<Integer>(recentColors.length);
+    for (String color : recentColors) {
+      if (color.length() > 0) {
+        recentColorsList.add(Integer.decode(color) | 0xFF000000);
+      }
+    }
+    setRecentColors(recentColorsList);
     // Read recent homes list
     List<String> recentHomes = new ArrayList<String>();
     for (int i = 1; i <= getRecentHomesMaxCount(); i++) {
@@ -323,6 +372,7 @@ public class FileUserPreferences extends UserPreferences {
    * Updates the default supported languages with languages available in plugin folder. 
    */
   private void updateSupportedLanguages() {
+    removeLibraries(LANGUAGE_LIBRARY_TYPE);
     List<ClassLoader> resourceClassLoaders = new ArrayList<ClassLoader>();
     String [] defaultSupportedLanguages = getDefaultSupportedLanguages();
     Set<String> supportedLanguages = new TreeSet<String>(Arrays.asList(defaultSupportedLanguages));
@@ -340,14 +390,25 @@ public class FileUserPreferences extends UserPreferences {
         if (pluginLanguageLibraryFiles != null) {
           // Treat language files in reverse order so file named with a date or a version 
           // will be taken into account from most recent to least recent
-          Arrays.sort(pluginLanguageLibraryFiles, Collections.reverseOrder());
+          Arrays.sort(pluginLanguageLibraryFiles, Collections.reverseOrder(OperatingSystem.getFileVersionComparator()));
           for (File pluginLanguageLibraryFile : pluginLanguageLibraryFiles) {
             try {
               Set<String> languages = getLanguages(pluginLanguageLibraryFile);
               if (!languages.isEmpty()) {
                 supportedLanguages.addAll(languages);
                 URL pluginFurnitureCatalogUrl = pluginLanguageLibraryFile.toURI().toURL();
-                resourceClassLoaders.add(new URLClassLoader(new URL [] {pluginFurnitureCatalogUrl}));
+                URLClassLoader classLoader = new URLClassLoader(new URL [] {pluginFurnitureCatalogUrl});
+                resourceClassLoaders.add(classLoader);
+
+                DefaultLibrary languageLibrary;
+                try {
+                  languageLibrary = new DefaultLibrary(pluginLanguageLibraryFile.getCanonicalPath(), LANGUAGE_LIBRARY_TYPE,
+                      ResourceBundle.getBundle(PLUGIN_LANGUAGE_LIBRARY_FAMILY, Locale.getDefault(), classLoader));
+                } catch (MissingResourceException ex) {
+                  languageLibrary = new DefaultLibrary(pluginLanguageLibraryFile.getCanonicalPath(), LANGUAGE_LIBRARY_TYPE, 
+                      null, getLanguageLibraryDefaultName(languages), null, getDefaultVersion(pluginLanguageLibraryFile), null, null);
+                }
+                libraries.add(0, languageLibrary);
               }
             } catch (IOException ex) {
               // Ignore malformed files
@@ -399,6 +460,52 @@ public class FileUserPreferences extends UserPreferences {
     }
   }
   
+  /**
+   * Returns a text in English describing the given languages.
+   */
+  private String getLanguageLibraryDefaultName(Set<String> languages) {
+    String description = "";
+    for (String language : languages) {
+      if (description.length() > 0) {
+        description += ", ";
+      }
+      int underscoreIndex = language.indexOf('_');
+      Locale locale = underscoreIndex < 0 
+          ? new Locale(language) 
+          : new Locale(language.substring(0, underscoreIndex), language.substring(underscoreIndex + 1));
+      description += locale.getDisplayLanguage(Locale.ENGLISH);
+      if (underscoreIndex >= 0) {
+        description += " (" + locale.getDisplayCountry(Locale.ENGLISH) + ")";
+      }
+    }
+    if (languages.size() > 1) {
+      description += " languages support";
+    } else {
+      description += " language support";
+    }
+    return description;
+  }
+
+  /**
+   * Returns a version number from the given file name or <code>null</code>.
+   */
+  private String getDefaultVersion(File pluginLanguageLibraryFile) {
+    String fileName = pluginLanguageLibraryFile.getName();
+    // Search version number between last hyphen and last point
+    int hyphenIndex = fileName.lastIndexOf('-');
+    if (hyphenIndex > 0) {
+      int pointIndex = fileName.lastIndexOf('.');
+      if (pointIndex < 0) {
+        pointIndex = fileName.length();
+      }
+      String version = fileName.substring(hyphenIndex + 1, pointIndex);
+      if (version.matches("[\\d\\.]+")) {
+        return version;
+      }
+    }
+    return null;
+  }
+
   /**
    * Returns the default class loader of user preferences and the class loaders that
    * give access to resources in language libraries plugin folder. 
@@ -413,20 +520,24 @@ public class FileUserPreferences extends UserPreferences {
    */
   private void updateFurnitureDefaultCatalog(Executor furnitureCatalogLoader, 
                                              final Executor updater) {
-    // Delete default pieces of current furniture catalog          
     final FurnitureCatalog furnitureCatalog = getFurnitureCatalog();
-    for (FurnitureCategory category : furnitureCatalog.getCategories()) {
-      for (CatalogPieceOfFurniture piece : category.getFurniture()) {
-        if (!piece.isModifiable()) {
-          furnitureCatalog.delete(piece);
-        }
-      }
-    }
-    // Read default furniture catalog
     furnitureCatalogLoader.execute(new Runnable() {
         public void run() {
-          // Fill default furniture catalog 
-          DefaultFurnitureCatalog defaultFurnitureCatalog = new DefaultFurnitureCatalog(
+          updater.execute(new Runnable() {
+              public void run() {
+                // Delete default furniture of current furniture catalog          
+                for (FurnitureCategory category : furnitureCatalog.getCategories()) {
+                  for (CatalogPieceOfFurniture piece : category.getFurniture()) {
+                    if (!piece.isModifiable()) {
+                      furnitureCatalog.delete(piece);
+                    }
+                  }
+                }
+              }
+            });
+          
+          // Read default furniture catalog
+          final DefaultFurnitureCatalog defaultFurnitureCatalog = new DefaultFurnitureCatalog(
               FileUserPreferences.this, getFurnitureLibrariesPluginFolders());
           for (final FurnitureCategory category : defaultFurnitureCatalog.getCategories()) {
             for (final CatalogPieceOfFurniture piece : category.getFurniture()) {
@@ -437,28 +548,51 @@ public class FileUserPreferences extends UserPreferences {
                 });
             }
           }
+          updater.execute(new Runnable() {
+              public void run() {
+                removeLibraries(FURNITURE_LIBRARY_TYPE);
+                libraries.addAll(defaultFurnitureCatalog.getLibraries());
+              }
+            });
         }
       });
   }
 
+  /**
+   * Removes from the list of libraries the ones of the given type.
+   */
+  private void removeLibraries(String libraryType) {
+    for (Iterator<Library> it = this.libraries.iterator(); it.hasNext(); ) {
+      Library library = it.next();
+      if (library.getType() == libraryType) {
+        it.remove();
+      }
+    }
+  }
+
   /**
    * Reloads textures default catalog.
    */
   private void updateTexturesDefaultCatalog(Executor texturesCatalogLoader, 
                                             final Executor updater) {
-    // Delete default textures of current textures catalog          
     final TexturesCatalog texturesCatalog = getTexturesCatalog();
-    for (TexturesCategory category : texturesCatalog.getCategories()) {
-      for (CatalogTexture texture : category.getTextures()) {
-        if (!texture.isModifiable()) {
-          texturesCatalog.delete(texture);
-        }
-      }
-    }
-    // Read default textures catalog
     texturesCatalogLoader.execute(new Runnable() {
         public void run() {
-          TexturesCatalog defaultTexturesCatalog = new DefaultTexturesCatalog(
+          updater.execute(new Runnable() {
+              public void run() {
+                // Delete default textures of current textures catalog          
+                for (TexturesCategory category : texturesCatalog.getCategories()) {
+                  for (CatalogTexture texture : category.getTextures()) {
+                    if (!texture.isModifiable()) {
+                      texturesCatalog.delete(texture);
+                    }
+                  }
+                }
+              }
+            });
+            
+          // Read default textures catalog
+          final DefaultTexturesCatalog defaultTexturesCatalog = new DefaultTexturesCatalog(
               FileUserPreferences.this, getTexturesLibrariesPluginFolders());
           for (final TexturesCategory category : defaultTexturesCatalog.getCategories()) {
             for (final CatalogTexture texture : category.getTextures()) {
@@ -469,6 +603,12 @@ public class FileUserPreferences extends UserPreferences {
                 });
             }
           }
+          updater.execute(new Runnable() {
+              public void run() {
+                removeLibraries(TEXTURES_LIBRARY_TYPE);
+                libraries.addAll(defaultTexturesCatalog.getLibraries());
+              }
+            });
         }
       });
   }
@@ -610,18 +750,28 @@ public class FileUserPreferences extends UserPreferences {
 
     // Write other preferences 
     preferences.put(LANGUAGE, getLanguage());
-    preferences.put(UNIT, getLengthUnit().name());   
+    preferences.put(EXTENSIBLE_UNIT, getLengthUnit().name());   
     preferences.putBoolean(FURNITURE_CATALOG_VIEWED_IN_TREE, isFurnitureCatalogViewedInTree());
     preferences.putBoolean(NAVIGATION_PANEL_VISIBLE, isNavigationPanelVisible());
     preferences.putBoolean(MAGNETISM_ENABLED, isMagnetismEnabled());
+    preferences.putBoolean(AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED, isAerialViewCenteredOnSelectionEnabled());
     preferences.putBoolean(RULERS_VISIBLE, isRulersVisible());
     preferences.putBoolean(GRID_VISIBLE, isGridVisible());
     preferences.putBoolean(FURNITURE_VIEWED_FROM_TOP, isFurnitureViewedFromTop());
     preferences.putBoolean(ROOM_FLOOR_COLORED_OR_TEXTURED, isRoomFloorColoredOrTextured());
     preferences.put(WALL_PATTERN, getWallPattern().getName());
+    TextureImage newWallPattern = getNewWallPattern();
+    if (newWallPattern != null) {
+      preferences.put(NEW_WALL_PATTERN, newWallPattern.getName());
+    }
     preferences.putFloat(NEW_WALL_THICKNESS, getNewWallThickness());   
     preferences.putFloat(NEW_WALL_HEIGHT, getNewWallHeight());
     preferences.putFloat(NEW_FLOOR_THICKNESS, getNewFloorThickness());   
+    preferences.putBoolean(CHECK_UPDATES_ENABLED, isCheckUpdatesEnabled());
+    Long updatesMinimumDate = getUpdatesMinimumDate();
+    if (updatesMinimumDate != null) {
+      preferences.putLong(UPDATES_MINIMUM_DATE, updatesMinimumDate);
+    }
     preferences.putInt(AUTO_SAVE_DELAY_FOR_RECOVERY, getAutoSaveDelayForRecovery());
     // Write recent homes list
     int i = 1;
@@ -632,6 +782,16 @@ public class FileUserPreferences extends UserPreferences {
     for ( ; i <= getRecentHomesMaxCount(); i++) {
       preferences.remove(RECENT_HOMES + i);
     }
+    // Write recent colors
+    StringBuilder recentColors = new StringBuilder();
+    Iterator<Integer> itColor = getRecentColors().iterator();
+    for (int j = 0; j < 100 && itColor.hasNext(); j++) {
+      if (j > 0) {
+        recentColors.append(",");
+      } 
+      recentColors.append(String.format("#%6X", itColor.next() & 0xFFFFFF).replace(' ', '0'));
+    }
+    preferences.put(RECENT_COLORS, recentColors.toString());
     // Write ignored action tips
     i = 1;
     for (Iterator<Map.Entry<String, Boolean>> it = this.ignoredActionTips.entrySet().iterator();
@@ -1057,110 +1217,111 @@ public class FileUserPreferences extends UserPreferences {
   }
   
   /**
-   * Returns <code>true</code> if the given language library exists in the first 
-   * language libraries folder.
+   * Returns <code>true</code> if the language library at the given location exists 
+   * in the first language libraries folder.
+   * @param location the file path of the resource to check
    */
-  public boolean languageLibraryExists(String name) throws RecorderException {
+  public boolean languageLibraryExists(String location) throws RecorderException {
     File [] languageLibrariesPluginFolders = getLanguageLibrariesPluginFolders();
     if (languageLibrariesPluginFolders == null
         || languageLibrariesPluginFolders.length == 0) {
       throw new RecorderException("Can't access to language libraries plugin folder");
     } else {
-      String libraryFileName = new File(name).getName();
+      String libraryFileName = new File(location).getName();
       return new File(languageLibrariesPluginFolders [0], libraryFileName).exists();
     }
   }
   
   /**
-   * Adds <code>languageLibraryName</code> to the first language libraries folder
+   * Adds <code>languageLibraryPath</code> to the first language libraries folder
    * to make the language library it contains available to supported languages.
    */
-  public void addLanguageLibrary(String languageLibraryName) throws RecorderException {
+  public void addLanguageLibrary(String languageLibraryPath) throws RecorderException {
     try {
       File [] languageLibrariesPluginFolders = getLanguageLibrariesPluginFolders();
       if (languageLibrariesPluginFolders == null
           || languageLibrariesPluginFolders.length == 0) {
         throw new RecorderException("Can't access to language libraries plugin folder");
       }
-      File languageLibraryFile = new File(languageLibraryName);
-      copyToLibraryFolder(languageLibraryFile, languageLibrariesPluginFolders [0]);
+      copyToLibraryFolder(new File(languageLibraryPath), languageLibrariesPluginFolders [0]);
       updateSupportedLanguages();
     } catch (IOException ex) {
       throw new RecorderException(
-          "Can't write " + languageLibraryName +  " in language libraries plugin folder", ex);
+          "Can't write " + languageLibraryPath +  " in language libraries plugin folder", ex);
     }
   }
 
   /**
-   * Returns <code>true</code> if the given furniture library file exists in the first 
-   * furniture libraries folder.
-   * @param name the name of the resource to check
+   * Returns <code>true</code> if the furniture library at the given <code>location</code> exists 
+   * in the first furniture libraries folder.
+   * @param location the file path of the resource to check
    */
   @Override
-  public boolean furnitureLibraryExists(String name) throws RecorderException {
+  public boolean furnitureLibraryExists(String location) throws RecorderException {
     File [] furnitureLibrariesPluginFolders = getFurnitureLibrariesPluginFolders();
     if (furnitureLibrariesPluginFolders == null
         || furnitureLibrariesPluginFolders.length == 0) {
       throw new RecorderException("Can't access to furniture libraries plugin folder");
     } else {
-      String libraryFileName = new File(name).getName();
+      String libraryFileName = new File(location).getName();
       return new File(furnitureLibrariesPluginFolders [0], libraryFileName).exists();
     }
   }
 
   /**
-   * Adds the file <code>furnitureLibraryName</code> to the first furniture libraries folder 
+   * Adds the file <code>furnitureLibraryPath</code> to the first furniture libraries folder 
    * to make the furniture library available to catalog.
    */
   @Override
-  public void addFurnitureLibrary(String furnitureLibraryName) throws RecorderException {
+  public void addFurnitureLibrary(String furnitureLibraryPath) throws RecorderException {
     try {
       File [] furnitureLibrariesPluginFolders = getFurnitureLibrariesPluginFolders();
       if (furnitureLibrariesPluginFolders == null
           || furnitureLibrariesPluginFolders.length == 0) {
         throw new RecorderException("Can't access to furniture libraries plugin folder");
       }
-      copyToLibraryFolder(new File(furnitureLibraryName), furnitureLibrariesPluginFolders [0]);
+      copyToLibraryFolder(new File(furnitureLibraryPath), furnitureLibrariesPluginFolders [0]);
       updateFurnitureDefaultCatalog(this.catalogsLoader, this.updater);
     } catch (IOException ex) {
       throw new RecorderException(
-          "Can't write " + furnitureLibraryName +  " in furniture libraries plugin folder", ex);
+          "Can't write " + furnitureLibraryPath +  " in furniture libraries plugin folder", ex);
     }
   }
 
   /**
-   * Returns <code>true</code> if the given textures library file exists in the first textures libraries folder.
-   * @param name the name of the resource to check
+   * Returns <code>true</code> if the textures library at the given <code>location</code> exists 
+   * in the first textures libraries folder.
+   * @param location the file path of the resource to check
    */
   @Override
-  public boolean texturesLibraryExists(String name) throws RecorderException {
+  public boolean texturesLibraryExists(String location) throws RecorderException {
     File [] texturesLibrariesPluginFolders = getTexturesLibrariesPluginFolders();
     if (texturesLibrariesPluginFolders == null
         || texturesLibrariesPluginFolders.length == 0) {
       throw new RecorderException("Can't access to textures libraries plugin folder");
     } else {
-      String libraryFileName = new File(name).getName();
-      return new File(texturesLibrariesPluginFolders [0], libraryFileName).exists();
+      String libraryLocation = new File(location).getName();
+      return new File(texturesLibrariesPluginFolders [0], libraryLocation).exists();
     }
   }
 
   /**
-   * Adds the file <code>texturesLibraryName</code> to the first textures libraries folder 
+   * Adds the file <code>texturesLibraryPath</code> to the first textures libraries folder 
    * to make the textures library available to catalog.
    */
   @Override
-  public void addTexturesLibrary(String texturesLibraryName) throws RecorderException {
+  public void addTexturesLibrary(String texturesLibraryPath) throws RecorderException {
     try {
       File [] texturesLibrariesPluginFolders = getTexturesLibrariesPluginFolders();
       if (texturesLibrariesPluginFolders == null
           || texturesLibrariesPluginFolders.length == 0) {
         throw new RecorderException("Can't access to textures libraries plugin folder");
       }
-      copyToLibraryFolder(new File(texturesLibraryName), texturesLibrariesPluginFolders [0]);
+      copyToLibraryFolder(new File(texturesLibraryPath), texturesLibrariesPluginFolders [0]);
       updateTexturesDefaultCatalog(this.catalogsLoader, this.updater);
     } catch (IOException ex) {
       throw new RecorderException(
-          "Can't write " + texturesLibraryName +  " in textures libraries plugin folder", ex);
+          "Can't write " + texturesLibraryPath +  " in textures libraries plugin folder", ex);
     }
   }
 
@@ -1196,6 +1357,56 @@ public class FileUserPreferences extends UserPreferences {
     }
   }
   
+  /**
+   * Returns the libraries available in user preferences.
+   * @since 4.0
+   */
+  @Override
+  public List<Library> getLibraries() {
+    return Collections.unmodifiableList(new ArrayList<Library>(this.libraries));
+  }
+
+  /**
+   * Deletes the given <code>libraries</code> and updates user preferences.
+   * @since 4.0
+   */
+  public void deleteLibraries(List<Library> libraries) throws RecorderException {
+    boolean updateFurnitureCatalog = false;
+    boolean updateTexturesCatalog  = false;
+    boolean updateSupportedLanguages = false;
+    for (Library library : libraries) {
+      if (!new File(library.getLocation()).delete()) {
+        throw new RecorderException("Couldn't delete file " + library.getLocation());
+      } else {
+        if (FURNITURE_LIBRARY_TYPE.equals(library.getType())) {
+          updateFurnitureCatalog = true;
+        } else if (TEXTURES_LIBRARY_TYPE.equals(library.getType())) {
+          updateTexturesCatalog = true;
+        }  else if (LANGUAGE_LIBRARY_TYPE.equals(library.getType())) {
+          updateSupportedLanguages = true;
+        }
+      }
+    }
+
+    if (updateFurnitureCatalog) {
+      updateFurnitureDefaultCatalog(this.catalogsLoader, this.updater);
+    } 
+    if (updateTexturesCatalog) {
+      updateTexturesDefaultCatalog(this.catalogsLoader, this.updater);
+    }  
+    if (updateSupportedLanguages) {
+      updateSupportedLanguages();
+    }
+  }
+
+  /**
+   * Returns <code>true</code> if the given file <code>library</code> can be deleted. 
+   * @since 4.0
+   */
+  public boolean isLibraryDeletable(Library library) {
+    return new File(library.getLocation()).canWrite();    
+  }
+  
   /**
    * Preferences based on the <code>preferences.xml</code> file
    * stored in a preferences folder.  
