@@ -191,28 +191,11 @@ public class Component3DManager {
       // Ensure unused canvases are freed
       System.gc();
       // Create a Java 3D canvas  
-      return new Canvas3D(configuration, offscreen) {
-          @Override
-          public void preRender() {
-            if (renderingObserver != null) {
-              renderingObserver.canvas3DPreRendered(this);
-            }
-          }
-          
-          @Override
-          public void postRender() {
-            if (renderingObserver != null) {
-              renderingObserver.canvas3DPostRendered(this);
-            }
-          }
-          
-          @Override
-          public void postSwap() {
-            if (renderingObserver != null) {
-              renderingObserver.canvas3DSwapped(this);
-            }
-          }
-        };
+      if (renderingObserver != null) {
+        return new ObservedCanvas3D(configuration, offscreen, renderingObserver);
+      } else {
+        return new Canvas3D(configuration, offscreen);
+      }
     } catch (IllegalArgumentException ex) {
       IllegalRenderingStateException ex2 = new IllegalRenderingStateException("Can't create Canvas 3D");
       ex2.initCause(ex);
@@ -353,7 +336,7 @@ public class Component3DManager {
       return renderingErrorListener;
     }
   }
-  
+
   /**
    * An observer that receives notifications during the different steps 
    * of the loop rendering a canvas 3D.
@@ -374,4 +357,33 @@ public class Component3DManager {
      */
     public void canvas3DSwapped(Canvas3D canvas3D); 
   }
+
+  /**
+   * A canvas 3D observed during its rendering.
+   */
+  private static class ObservedCanvas3D extends Canvas3D {
+    private final RenderingObserver renderingObserver;
+
+    private ObservedCanvas3D(GraphicsConfiguration graphicsConfiguration, 
+                             boolean offScreen,
+                             RenderingObserver renderingObserver) {
+      super(graphicsConfiguration, offScreen);
+      this.renderingObserver = renderingObserver;
+    }
+
+    @Override
+    public void preRender() {
+      this.renderingObserver.canvas3DPreRendered(this);
+    }
+
+    @Override
+    public void postRender() {
+      this.renderingObserver.canvas3DPostRendered(this);
+    }
+
+    @Override
+    public void postSwap() {
+      this.renderingObserver.canvas3DSwapped(this);
+    }
+  }
 }
