@@ -29,6 +29,7 @@ import java.io.InputStream;
 import java.io.Reader;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.net.URLConnection;
 import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.HashMap;
@@ -89,6 +90,18 @@ import com.sun.j3d.utils.image.TextureLoader;
  * @author apptaro (bug fixes)
  */
 public class DAELoader extends LoaderBase implements Loader {
+  private Boolean useCaches;
+
+  /**
+   * Sets whether this loader should try or avoid accessing to URLs with cache.
+   * @param useCaches <code>Boolean.TRUE</code>, <code>Boolean.FALSE</code>, or 
+   *    <code>null</code> then caches will be used according to the value 
+   *    returned by {@link URLConnection#getDefaultUseCaches()}.
+   */
+  public void setUseCaches(Boolean useCaches) {
+    this.useCaches = Boolean.valueOf(useCaches);
+  }
+  
   /**
    * Returns the scene described in the given DAE file.
    */
@@ -116,13 +129,24 @@ public class DAELoader extends LoaderBase implements Loader {
     } 
     InputStream in;
     try {
-      in = url.openStream();
+      in = openStream(url);
     } catch (IOException ex) {
       throw new FileNotFoundException("Can't read " + url);
     }
     return load(new BufferedInputStream(in), baseUrl);
   }
 
+  /**
+   * Returns an input stream ready to read data from the given URL.
+   */
+  private InputStream openStream(URL url) throws IOException {
+    URLConnection connection = url.openConnection();
+    if (this.useCaches != null) {
+      connection.setUseCaches(this.useCaches.booleanValue());
+    }
+    return connection.getInputStream();
+  }
+
   /**
    * Returns the scene described in the given DAE file stream.
    */
@@ -173,7 +197,7 @@ public class DAELoader extends LoaderBase implements Loader {
   /**
    * SAX handler for DAE Collada stream.
    */
-  private static class DAEHandler extends DefaultHandler {
+  private class DAEHandler extends DefaultHandler {
     private final SceneBase scene;
     private final URL       baseUrl;
     private final Stack<Group>   parentGroups = new Stack<Group>();
@@ -610,9 +634,11 @@ public class DAELoader extends LoaderBase implements Loader {
      */
     private void handleImageElementsEnd(String name) throws SAXException {
       if ("init_from".equals(name)) {
+        InputStream in = null;
         try {
-          URL textureImageUrl = new URL(baseUrl, getCharacters());
-          BufferedImage textureImage = ImageIO.read(textureImageUrl);
+          URL textureImageUrl = new URL(this.baseUrl, getCharacters().replace("%", "%25").replace("#", "%23"));
+          in = openStream(textureImageUrl);
+          BufferedImage textureImage = ImageIO.read(in);
           if (textureImage != null) {
             TextureLoader textureLoader = new TextureLoader(textureImage);
             Texture texture = textureLoader.getTexture();
@@ -630,6 +656,14 @@ public class DAELoader extends LoaderBase implements Loader {
           } else {
             throw ex;
           }
+        } finally {
+          if (in != null) {
+            try {
+              in.close();
+            } catch (IOException ex) {
+              throw new SAXException("Issue while closing stream", ex);
+            }
+          }
         }
       } else if ("data".equals(name)) {
         throw new SAXException("<data> not supported");
