@@ -26,6 +26,7 @@ import java.awt.geom.Area;
 import java.awt.geom.GeneralPath;
 import java.awt.geom.PathIterator;
 import java.awt.geom.Rectangle2D;
+import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
@@ -34,14 +35,21 @@ import java.io.PrintStream;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Modifier;
+import java.net.JarURLConnection;
+import java.net.URISyntaxException;
 import java.net.URL;
+import java.net.URLConnection;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Comparator;
 import java.util.Enumeration;
 import java.util.HashMap;
+import java.util.HashSet;
+import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
+import java.util.TreeSet;
 import java.util.WeakHashMap;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
@@ -87,8 +95,13 @@ import org.apache.batik.parser.AWTPathProducer;
 import org.apache.batik.parser.ParseException;
 import org.apache.batik.parser.PathParser;
 
+import com.eteks.sweethome3d.model.CatalogTexture;
 import com.eteks.sweethome3d.model.Content;
+import com.eteks.sweethome3d.model.HomeMaterial;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
+import com.eteks.sweethome3d.model.HomeTexture;
+import com.eteks.sweethome3d.model.Room;
+import com.eteks.sweethome3d.tools.OperatingSystem;
 import com.eteks.sweethome3d.tools.TemporaryURLContent;
 import com.eteks.sweethome3d.tools.URLContent;
 import com.microcrowd.loader.java3d.max3ds.Loader3DS;
@@ -429,11 +442,7 @@ public class ModelManager {
     Transform3D modelTransform;
     if (modelRotation != null) {
       // Get model bounding box size with model rotation
-      Matrix3f modelRotationMatrix = new Matrix3f(modelRotation [0][0], modelRotation [0][1], modelRotation [0][2],
-          modelRotation [1][0], modelRotation [1][1], modelRotation [1][2],
-          modelRotation [2][0], modelRotation [2][1], modelRotation [2][2]);
-      modelTransform = new Transform3D();
-      modelTransform.setRotation(modelRotationMatrix);
+      modelTransform = getRotationTransformation(modelRotation);
       modelTransform.mul(translation);
       BoundingBox rotatedModelBounds = getBounds(node, modelTransform);
       rotatedModelBounds.getLower(lower);
@@ -451,7 +460,47 @@ public class ModelManager {
     scaleOneTransform.mul(modelTransform);
     return scaleOneTransform;
   }
+
+  /**
+   * Returns a transformation matching the given rotation.
+   */
+  Transform3D getRotationTransformation(float [][] modelRotation) {
+    Matrix3f modelRotationMatrix = new Matrix3f(modelRotation [0][0], modelRotation [0][1], modelRotation [0][2],
+        modelRotation [1][0], modelRotation [1][1], modelRotation [1][2],
+        modelRotation [2][0], modelRotation [2][1], modelRotation [2][2]);
+    Transform3D modelTransform = new Transform3D();
+    modelTransform.setRotation(modelRotationMatrix);
+    return modelTransform;
+  }
   
+  /**
+   * Returns a transformation able to place in the scene the normalized model 
+   * of the given <code>piece</code>.
+   */
+  Transform3D getPieceOFFurnitureNormalizedModelTransformation(HomePieceOfFurniture piece) {
+    // Set piece size
+    Transform3D scale = new Transform3D();
+    float pieceWidth = piece.getWidth();
+    // If piece model is mirrored, inverse its width
+    if (piece.isModelMirrored()) {
+      pieceWidth *= -1;
+    }
+    scale.setScale(new Vector3d(pieceWidth, piece.getHeight(), piece.getDepth()));
+    // Change its angle around y axis
+    Transform3D orientation = new Transform3D();
+    orientation.rotY(-piece.getAngle());
+    orientation.mul(scale);
+    // Translate it to its location
+    Transform3D pieceTransform = new Transform3D();
+    float z = piece.getElevation() + piece.getHeight() / 2;
+    if (piece.getLevel() != null) {
+      z += piece.getLevel().getElevation();
+    }
+    pieceTransform.setTranslation(new Vector3f(piece.getX(), z, piece.getY()));      
+    pieceTransform.mul(orientation);
+    return pieceTransform;
+  }
+
   /**
    * Reads asynchronously a 3D node from <code>content</code> with supported loaders
    * and notifies the loaded model to the given <code>modelObserver</code> once available. 
@@ -669,14 +718,52 @@ public class ModelManager {
     }
     
     Exception lastException = null;
+    Boolean useCaches = shouldUseCaches(urlContent);
     for (Loader loader : loaders) {
+      boolean loadSynchronously = false;
+      try {
+        // Call setUseCaches(Boolean) by reflection
+        loader.getClass().getMethod("setUseCaches", Boolean.class).invoke(loader, useCaches);
+      } catch (NoSuchMethodException ex) {
+        // If the method setUseCaches doesn't exist, set default cache use if different 
+        // from the required one and load models synchronously
+        URLConnection connection = urlContent.getURL().openConnection();
+        loadSynchronously = connection.getDefaultUseCaches() != useCaches;        
+      } catch (InvocationTargetException ex) {
+        if (ex instanceof Exception) {
+          lastException = (Exception)ex.getTargetException();
+          continue;
+        } else {
+          ex.printStackTrace();
+        }
+      } catch (Exception ex) {
+        ex.printStackTrace();
+      }
+      
       try {     
         // Ask loader to ignore lights, fogs...
         loader.setFlags(loader.getFlags() 
             & ~(Loader.LOAD_LIGHT_NODES | Loader.LOAD_FOG_NODES 
                 | Loader.LOAD_BACKGROUND_NODES | Loader.LOAD_VIEW_GROUPS));
         // Return the first scene that can be loaded from model URL content
-        Scene scene = loader.load(urlContent.getURL());
+        Scene scene;
+        if (loadSynchronously) {
+          synchronized (this.modelsLoader) {
+            URLConnection connection = urlContent.getURL().openConnection();
+            try {
+              connection.setDefaultUseCaches(useCaches);
+              scene = loader.load(urlContent.getURL());
+            } finally {
+              if (connection.getDefaultUseCaches() == useCaches) {
+                // Restore the default global value only when it didn't change yet,
+                // in case an other thread not synchronized on the same lock changed it
+                connection.setDefaultUseCaches(!useCaches);
+              }
+            }
+          }
+        } else {
+          scene = loader.load(urlContent.getURL());
+        }
 
         BranchGroup modelNode = scene.getSceneGroup();
         // If model doesn't have any child, consider the file as wrong
@@ -685,11 +772,10 @@ public class ModelManager {
         }
         
         // Update transparency of scene window panes shapes
-        updateShapeNamesAndWindowPanesTransparency(scene);
-        
+        updateShapeNamesAndWindowPanesTransparency(scene);        
         // Turn off lights because some loaders don't take into account the ~LOAD_LIGHT_NODES flag
-        turnOffLightsShareAndModulateTextures(modelNode);
-
+        turnOffLightsShareAndModulateTextures(modelNode);        
+        checkAppearancesName(modelNode);
         return modelNode;
       } catch (IllegalArgumentException ex) {
         lastException = ex;
@@ -727,6 +813,32 @@ public class ModelManager {
     } 
   }  
   
+  /**
+   * Returns <code>true</code> if reading from the given content should be done using caches.
+   */
+  private boolean shouldUseCaches(URLContent urlContent) throws IOException {
+    URLConnection connection = urlContent.getURL().openConnection();
+    if (OperatingSystem.isWindows() 
+        && (connection instanceof JarURLConnection)) {
+      JarURLConnection urlConnection = (JarURLConnection)connection;
+      URL jarFileUrl = urlConnection.getJarFileURL();
+      if (jarFileUrl.getProtocol().equalsIgnoreCase("file")) {
+        try {
+          if (new File(jarFileUrl.toURI()).canWrite()) {
+            // Refuse to use caches to be able to delete the writable files accessed with jar protocol under Windows, 
+            // as suggested in http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6962459 
+            return false;
+          }
+        } catch (URISyntaxException ex) {
+          IOException ex2 = new IOException();
+          ex2.initCause(ex);
+          throw ex2;
+        }
+      }
+    }
+    return connection.getDefaultUseCaches();
+  }
+  
   /**
    * Updates the name of scene shapes and transparency window panes shapes.
    */
@@ -810,6 +922,170 @@ public class ModelManager {
     } 
   }
 
+  /** 
+   * Ensures that all the appearance of the children shapes of the 
+   * given <code>node</code> have a name.
+   */
+  public void checkAppearancesName(Node node) {
+    // Search appearances used by node shapes keeping their enumeration order 
+    Set<Appearance> appearances = new LinkedHashSet<Appearance>(); 
+    searchMaterials(node, appearances);
+    int i = 0;
+    for (Appearance appearance : appearances) {
+      try {
+        if (appearance.getName() == null) {
+          appearance.setName("Texture_" + ++i);
+        }
+      } catch (NoSuchMethodError ex) {
+        // Don't support HomeMaterial with Java 3D < 1.4 where appearance name was added
+        break;
+      }
+    }
+  }
+
+  /** 
+   * Returns the materials used by the children shapes of the given <code>node</code>.
+   */
+  public HomeMaterial [] getMaterials(Node node) {
+    // Search appearances used by node shapes 
+    Set<Appearance> appearances = new HashSet<Appearance>(); 
+    searchMaterials(node, appearances);
+    Set<HomeMaterial> materials = new TreeSet<HomeMaterial>(new Comparator<HomeMaterial>() {
+        public int compare(HomeMaterial m1, HomeMaterial m2) {
+          String name1 = m1.getName();
+          String name2 = m2.getName();
+          if (name1 != null) {
+            if (name2 != null) {
+              return name1.compareTo(name2);
+            } else {
+              return 1;
+            }
+          } else if (name2 != null) {
+            return -1;
+          } else {
+            return 0;
+          }
+        }
+      });
+    for (Appearance appearance : appearances) {
+      Integer color = null;
+      Float   shininess = null;
+      Material material = appearance.getMaterial();
+      if (material != null) {
+        Color3f diffuseColor = new Color3f();
+        material.getDiffuseColor(diffuseColor);
+        color = ((int)(diffuseColor.x * 255) << 16) 
+            | ((int)(diffuseColor.y * 255) << 8)
+            | (int)(diffuseColor.z * 255); 
+        shininess = material.getShininess() / 128;          
+      }
+      Texture appearanceTexture = appearance.getTexture();
+      HomeTexture texture = null;
+      if (appearanceTexture != null) {
+        URL textureImageUrl = (URL)appearanceTexture.getUserData();
+        if (textureImageUrl != null) {
+          Content textureImage = new URLContent(textureImageUrl);
+          texture = new HomeTexture(new CatalogTexture(null, textureImage, -1, -1));
+        }
+      }
+      try {
+        materials.add(new HomeMaterial(appearance.getName(), color, texture, shininess));
+      } catch (NoSuchMethodError ex) {
+        // Don't support HomeMaterial with Java 3D < 1.4 where getName was added
+        return new HomeMaterial [0];
+      }
+    }
+    return materials.toArray(new HomeMaterial [materials.size()]);
+  }
+
+  private void searchMaterials(Node node, Set<Appearance> appearances) {
+    if (node instanceof Group) {
+      // Enumerate children
+      Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
+      while (enumeration.hasMoreElements()) {
+        searchMaterials((Node)enumeration.nextElement(), appearances);
+      }
+    } else if (node instanceof Link) {
+      searchMaterials(((Link)node).getSharedGroup(), appearances);
+    } else if (node instanceof Shape3D) {
+      Appearance appearance = ((Shape3D)node).getAppearance();
+      if (appearance != null) {
+        appearances.add(appearance);
+      }
+    }
+  }
+
+  /**
+   * Returns the AWT shape matching the given cut out shape if not <code>null</code> 
+   * or the 2D area of the 3D shapes children of the <code>node</code> 
+   * projected on its front side. The returned area is normalized in a 1 unit square
+   * centered at the origin.
+   */
+  Area getFrontArea(String cutOutShape, Node node) {
+    Area frontArea;
+    if (cutOutShape != null) {
+      frontArea = new Area(parseShape(cutOutShape));
+      frontArea.transform(AffineTransform.getScaleInstance(1, -1));
+      frontArea.transform(AffineTransform.getTranslateInstance(-.5, .5));
+    } else {
+      int vertexCount = getVertexCount(node);
+      if (vertexCount < 1000000) {
+        Area frontAreaWithHoles = new Area();
+        computeBottomOrFrontArea(node, frontAreaWithHoles, new Transform3D(), false, false);
+        // Remove holes and duplicated points
+        frontArea = new Area();
+        List<float []> currentPathPoints = new ArrayList<float[]>();
+        float [] previousRoomPoint = null;
+        for (PathIterator it = frontAreaWithHoles.getPathIterator(null, 1); !it.isDone(); it.next()) {
+          float [] areaPoint = new float[2];
+          switch (it.currentSegment(areaPoint)) {
+            case PathIterator.SEG_MOVETO :
+            case PathIterator.SEG_LINETO : 
+              if (previousRoomPoint == null
+                  || areaPoint [0] != previousRoomPoint [0] 
+                  || areaPoint [1] != previousRoomPoint [1]) {
+                currentPathPoints.add(areaPoint);
+              }
+              previousRoomPoint = areaPoint;
+              break;
+            case PathIterator.SEG_CLOSE :
+              if (currentPathPoints.get(0) [0] == previousRoomPoint [0] 
+                  && currentPathPoints.get(0) [1] == previousRoomPoint [1]) {
+                currentPathPoints.remove(currentPathPoints.size() - 1);
+              }
+              if (currentPathPoints.size() > 2) {
+                float [][] pathPoints = 
+                    currentPathPoints.toArray(new float [currentPathPoints.size()][]);
+                Room subRoom = new Room(pathPoints);
+                if (subRoom.getArea() > 0) {
+                  if (!subRoom.isClockwise()) {
+                    // Ignore clockwise points that match holes
+                    GeneralPath currentPath = new GeneralPath();
+                    currentPath.moveTo(pathPoints [0][0], pathPoints [0][1]);
+                    for (int i = 1; i < pathPoints.length; i++) {
+                      currentPath.lineTo(pathPoints [i][0], pathPoints [i][1]);
+                    }
+                    currentPath.closePath();
+                    frontArea.add(new Area(currentPath));
+                  }
+                }
+              }
+              currentPathPoints.clear();
+              previousRoomPoint = null;
+              break;
+          }
+        }
+        Rectangle2D bounds = frontAreaWithHoles.getBounds2D();
+        frontArea.transform(AffineTransform.getTranslateInstance(-bounds.getCenterX(), -bounds.getCenterY()));
+        frontArea.transform(AffineTransform.getScaleInstance(1 / bounds.getWidth(), 1 / bounds.getHeight()));        
+      } else {
+        frontArea = new Area(new Rectangle2D.Float(-.5f, -.5f, 1, 1));
+      }    
+    }
+    return frontArea;
+  }
+  
+  
   /**
    * Returns the 2D area of the 3D shapes children of the given <code>node</code> 
    * projected on the floor (plan y = 0). 
@@ -819,7 +1095,7 @@ public class ModelManager {
     int vertexCount = getVertexCount(node);
     if (vertexCount < 10000) {
       modelAreaOnFloor = new Area();
-      computeAreaOnFloor(node, modelAreaOnFloor, new Transform3D());
+      computeBottomOrFrontArea(node, modelAreaOnFloor, new Transform3D(), true, true);
     } else {
       List<float []> vertices = new ArrayList<float[]>(vertexCount); 
       computeVerticesOnFloor(node, vertices, new Transform3D());
@@ -867,9 +1143,13 @@ public class ModelManager {
   }
   
   /**
-   * Computes the vertices coordinates projected on floor of the 3D shapes children of <code>node</code>.
+   * Computes the 2D area on floor or on front side of the 3D shapes children of <code>node</code>.
    */
-  private void computeVerticesOnFloor(Node node, List<float []> vertices, Transform3D parentTransformations) {
+  private void computeBottomOrFrontArea(Node node, 
+                                        Area nodeArea, 
+                                        Transform3D parentTransformations,
+                                        boolean ignoreTransparentShapes, 
+                                        boolean bottom) {
     if (node instanceof Group) {
       if (node instanceof TransformGroup) {
         parentTransformations = new Transform3D(parentTransformations);
@@ -880,10 +1160,10 @@ public class ModelManager {
       // Compute all children
       Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
       while (enumeration.hasMoreElements()) {
-        computeVerticesOnFloor((Node)enumeration.nextElement(), vertices, parentTransformations);
+        computeBottomOrFrontArea((Node)enumeration.nextElement(), nodeArea, parentTransformations, ignoreTransparentShapes, bottom);
       }
     } else if (node instanceof Link) {
-      computeVerticesOnFloor(((Link)node).getSharedGroup(), vertices, parentTransformations);
+      computeBottomOrFrontArea(((Link)node).getSharedGroup(), nodeArea, parentTransformations, ignoreTransparentShapes, bottom);
     } else if (node instanceof Shape3D) {
       Shape3D shape = (Shape3D)node;
       Appearance appearance = shape.getAppearance();
@@ -893,96 +1173,25 @@ public class ModelManager {
           ? appearance.getTransparencyAttributes() : null;
       if ((renderingAttributes == null
             || renderingAttributes.getVisible())
-          && (transparencyAttributes == null
+          && (!ignoreTransparentShapes
+              || transparencyAttributes == null
               || transparencyAttributes.getTransparency() < 1)) {
         // Compute shape geometries area
         for (int i = 0, n = shape.numGeometries(); i < n; i++) {
-          Geometry geometry = shape.getGeometry(i);
-          if (geometry instanceof GeometryArray) {
-            GeometryArray geometryArray = (GeometryArray)geometry;      
-
-            int vertexCount = geometryArray.getVertexCount();
-            Point3f vertex = new Point3f();
-            if ((geometryArray.getVertexFormat() & GeometryArray.BY_REFERENCE) != 0) {
-              if ((geometryArray.getVertexFormat() & GeometryArray.INTERLEAVED) != 0) {
-                float [] vertexData = geometryArray.getInterleavedVertices();
-                int vertexSize = vertexData.length / vertexCount;
-                // Store vertices coordinates 
-                for (int index = 0, j = vertexSize - 3; index < vertexCount; j += vertexSize, index++) {
-                  vertex.x = vertexData [j];
-                  vertex.y = vertexData [j + 1];
-                  vertex.z = vertexData [j + 2];
-                  parentTransformations.transform(vertex);
-                  vertices.add(new float [] {vertex.x, vertex.z});
-                }
-              } else {
-                // Store vertices coordinates
-                float [] vertexCoordinates = geometryArray.getCoordRefFloat();
-                for (int index = 0, j = 0; index < vertexCount; j += 3, index++) {
-                  vertex.x = vertexCoordinates [j];
-                  vertex.y = vertexCoordinates [j + 1];
-                  vertex.z = vertexCoordinates [j + 2];
-                  parentTransformations.transform(vertex);
-                  vertices.add(new float [] {vertex.x, vertex.z});
-                }
-              }
-            } else {
-              // Store vertices coordinates
-              for (int index = 0, j = 0; index < vertexCount; j++, index++) {
-                geometryArray.getCoordinate(j, vertex);
-                parentTransformations.transform(vertex);
-                vertices.add(new float [] {vertex.x, vertex.z});
-              }
-            }
-          }
+          computeBottomOrFrontGeometryArea(shape.getGeometry(i), nodeArea, parentTransformations, bottom);
         }
       }
     }    
   }
   
   /**
-   * Computes the 2D area on floor of the 3D shapes children of <code>node</code>.
+   * Computes the bottom area of a 3D geometry if <code>bottom</code> is <code>true</code>, 
+   * and the front area if not.
    */
-  private void computeAreaOnFloor(Node node, Area nodeArea, Transform3D parentTransformations) {
-    if (node instanceof Group) {
-      if (node instanceof TransformGroup) {
-        parentTransformations = new Transform3D(parentTransformations);
-        Transform3D transform = new Transform3D();
-        ((TransformGroup)node).getTransform(transform);
-        parentTransformations.mul(transform);
-      }
-      // Compute all children
-      Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
-      while (enumeration.hasMoreElements()) {
-        computeAreaOnFloor((Node)enumeration.nextElement(), nodeArea, parentTransformations);
-      }
-    } else if (node instanceof Link) {
-      computeAreaOnFloor(((Link)node).getSharedGroup(), nodeArea, parentTransformations);
-    } else if (node instanceof Shape3D) {
-      Shape3D shape = (Shape3D)node;
-      Appearance appearance = shape.getAppearance();
-      RenderingAttributes renderingAttributes = appearance != null 
-          ? appearance.getRenderingAttributes() : null;
-      TransparencyAttributes transparencyAttributes = appearance != null 
-          ? appearance.getTransparencyAttributes() : null;
-      if ((renderingAttributes == null
-            || renderingAttributes.getVisible())
-          && (transparencyAttributes == null
-              || transparencyAttributes.getTransparency() < 1)) {
-        // Compute shape geometries area
-        for (int i = 0, n = shape.numGeometries(); i < n; i++) {
-          computeGeometryAreaOnFloor(shape.getGeometry(i), parentTransformations, nodeArea);
-        }
-      }
-    }    
-  }
-  
-  /**
-   * Computes the area on floor of a 3D geometry.
-   */
-  private void computeGeometryAreaOnFloor(Geometry geometry, 
-                                          Transform3D parentTransformations, 
-                                          Area nodeArea) {
+  private void computeBottomOrFrontGeometryArea(Geometry geometry, 
+                                                Area nodeArea, 
+                                                Transform3D parentTransformations,
+                                                boolean bottom) {
     if (geometry instanceof GeometryArray) {
       GeometryArray geometryArray = (GeometryArray)geometry;      
 
@@ -1000,7 +1209,11 @@ public class ModelManager {
             vertex.z = vertexData [i + 2];
             parentTransformations.transform(vertex);
             vertices [index++] = vertex.x;
-            vertices [index++] = vertex.z;
+            if (bottom) {
+              vertices [index++] = vertex.z;
+            } else {
+              vertices [index++] = vertex.y;
+            }
           }
         } else {
           // Store vertices coordinates
@@ -1011,7 +1224,11 @@ public class ModelManager {
             vertex.z = vertexCoordinates [i + 2];
             parentTransformations.transform(vertex);
             vertices [index++] = vertex.x;
-            vertices [index++] = vertex.z;
+            if (bottom) {
+              vertices [index++] = vertex.z;
+            } else {
+              vertices [index++] = vertex.y;
+            }
           }
         }
       } else {
@@ -1020,7 +1237,11 @@ public class ModelManager {
           geometryArray.getCoordinate(i, vertex);
           parentTransformations.transform(vertex);
           vertices [index++] = vertex.x;
-          vertices [index++] = vertex.z;
+          if (bottom) {
+            vertices [index++] = vertex.z;
+          } else {
+            vertices [index++] = vertex.y;
+          }
         }
       }
 
@@ -1208,6 +1429,80 @@ public class ModelManager {
     }
   }
 
+  /**
+   * Computes the vertices coordinates projected on floor of the 3D shapes children of <code>node</code>.
+   */
+  private void computeVerticesOnFloor(Node node, List<float []> vertices, Transform3D parentTransformations) {
+    if (node instanceof Group) {
+      if (node instanceof TransformGroup) {
+        parentTransformations = new Transform3D(parentTransformations);
+        Transform3D transform = new Transform3D();
+        ((TransformGroup)node).getTransform(transform);
+        parentTransformations.mul(transform);
+      }
+      // Compute all children
+      Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
+      while (enumeration.hasMoreElements()) {
+        computeVerticesOnFloor((Node)enumeration.nextElement(), vertices, parentTransformations);
+      }
+    } else if (node instanceof Link) {
+      computeVerticesOnFloor(((Link)node).getSharedGroup(), vertices, parentTransformations);
+    } else if (node instanceof Shape3D) {
+      Shape3D shape = (Shape3D)node;
+      Appearance appearance = shape.getAppearance();
+      RenderingAttributes renderingAttributes = appearance != null 
+          ? appearance.getRenderingAttributes() : null;
+      TransparencyAttributes transparencyAttributes = appearance != null 
+          ? appearance.getTransparencyAttributes() : null;
+      if ((renderingAttributes == null
+            || renderingAttributes.getVisible())
+          && (transparencyAttributes == null
+              || transparencyAttributes.getTransparency() < 1)) {
+        // Compute shape geometries area
+        for (int i = 0, n = shape.numGeometries(); i < n; i++) {
+          Geometry geometry = shape.getGeometry(i);
+          if (geometry instanceof GeometryArray) {
+            GeometryArray geometryArray = (GeometryArray)geometry;      
+
+            int vertexCount = geometryArray.getVertexCount();
+            Point3f vertex = new Point3f();
+            if ((geometryArray.getVertexFormat() & GeometryArray.BY_REFERENCE) != 0) {
+              if ((geometryArray.getVertexFormat() & GeometryArray.INTERLEAVED) != 0) {
+                float [] vertexData = geometryArray.getInterleavedVertices();
+                int vertexSize = vertexData.length / vertexCount;
+                // Store vertices coordinates 
+                for (int index = 0, j = vertexSize - 3; index < vertexCount; j += vertexSize, index++) {
+                  vertex.x = vertexData [j];
+                  vertex.y = vertexData [j + 1];
+                  vertex.z = vertexData [j + 2];
+                  parentTransformations.transform(vertex);
+                  vertices.add(new float [] {vertex.x, vertex.z});
+                }
+              } else {
+                // Store vertices coordinates
+                float [] vertexCoordinates = geometryArray.getCoordRefFloat();
+                for (int index = 0, j = 0; index < vertexCount; j += 3, index++) {
+                  vertex.x = vertexCoordinates [j];
+                  vertex.y = vertexCoordinates [j + 1];
+                  vertex.z = vertexCoordinates [j + 2];
+                  parentTransformations.transform(vertex);
+                  vertices.add(new float [] {vertex.x, vertex.z});
+                }
+              }
+            } else {
+              // Store vertices coordinates
+              for (int index = 0, j = 0; index < vertexCount; j++, index++) {
+                geometryArray.getCoordinate(j, vertex);
+                parentTransformations.transform(vertex);
+                vertices.add(new float [] {vertex.x, vertex.z});
+              }
+            }
+          }
+        }
+      }
+    }    
+  }
+  
   /**
    * Returns the convex polygon that surrounds the given <code>vertices</code>.
    * From Andrew's monotone chain 2D convex hull algorithm described at
