@@ -37,6 +37,7 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.ResourceBundle;
 import java.util.Set;
 import java.util.UUID;
 
@@ -61,6 +62,7 @@ import javax.media.j3d.LineStripArray;
 import javax.media.j3d.Link;
 import javax.media.j3d.Material;
 import javax.media.j3d.Node;
+import javax.media.j3d.PolygonAttributes;
 import javax.media.j3d.QuadArray;
 import javax.media.j3d.RenderingAttributes;
 import javax.media.j3d.Shape3D;
@@ -125,7 +127,7 @@ public class PhotoRenderer {
   
   private final SunflowAPI sunflow;
   private boolean useSunSky;
-  private boolean useAmbientOcclusion;
+  private boolean useSunskyLight;
   private String sunSkyLightName;
   private String sunLightName;
   private final Map<TransparentTextureKey, String> textureImagesCache = new HashMap<TransparentTextureKey, String>();
@@ -149,7 +151,7 @@ public class PhotoRenderer {
           public Object createObject3D(Home home, Selectable item, boolean waitForLoading) {
             if (item instanceof Room) {
               // Never display ceiling with top camera
-              return new Room3D((Room)item, home, home.getCamera() == home.getTopCamera(), true, waitForLoading);
+              return new Room3D((Room)item, home, !(home.getCamera() instanceof ObserverCamera), true, waitForLoading);
             } else {
               return super.createObject3D(home, item, waitForLoading);
             }  
@@ -172,10 +174,16 @@ public class PhotoRenderer {
     this.quality = quality;
     this.sunflow = new SunflowAPI();
     
-    this.useAmbientOcclusion = home.getCamera() instanceof ObserverCamera;
-    // SunFlow produce too much white spots when silk shader is used with sun sky
+    this.useSunskyLight = !(home.getCamera() instanceof ObserverCamera);
+    // SunFlow produce too much white spots when silk shader is used with sun sky light
     // so use this shader only when observer is used 
-    boolean silk = this.useAmbientOcclusion && quality == Quality.HIGH;
+    boolean silk = !this.useSunskyLight && quality == Quality.HIGH;
+    String shininessShader = getRenderingParameterValue("shininessShader");
+    if ("glossy".equals(shininessShader)) {
+      silk = false;
+    } else if ("silk".equals(shininessShader)) {
+      silk = true;
+    }  
     
     // Export to SunFlow the Java 3D shapes and appearance of the ground, the walls, the furniture and the rooms
     HomeEnvironment homeEnvironment = home.getEnvironment();
@@ -201,7 +209,7 @@ public class PhotoRenderer {
     homeEnvironment.setSubpartSizeUnderLight(subpartSize);
 
     HomeTexture skyTexture = homeEnvironment.getSkyTexture();
-    this.useSunSky = skyTexture == null || !(home.getCamera() instanceof ObserverCamera);
+    this.useSunSky = skyTexture == null || this.useSunskyLight;
     if (!this.useSunSky) {
       // If observer camera is used with a sky texture, 
       // create an image base light from sky texture  
@@ -319,11 +327,20 @@ public class PhotoRenderer {
       }
     }
 
-    this.sunflow.parameter("depths.diffuse", 1);
+    this.sunflow.parameter("depths.diffuse", Integer.parseInt(getRenderingParameterValue("diffusedBounces")));
     this.sunflow.parameter("depths.reflection", 4);
     this.sunflow.parameter("depths.refraction", 16);
     this.sunflow.options(SunflowAPI.DEFAULT_OPTIONS);
-    
+
+    Integer causticsEmit = new Integer(getRenderingParameterValue("causticsPhotons"));
+    if (causticsEmit > 0) {
+      this.sunflow.parameter("caustics.emit", causticsEmit);
+      this.sunflow.parameter("caustics", "kd");
+      this.sunflow.parameter("caustics.gather", 64);
+      this.sunflow.parameter("caustics.radius", 0.5f);
+      this.sunflow.options(SunflowAPI.DEFAULT_OPTIONS);
+    }
+
     // Use a spiral computing
     this.sunflow.parameter("bucket.size", 32);
     this.sunflow.parameter("bucket.order", "spiral");
@@ -346,6 +363,8 @@ public class PhotoRenderer {
     if (this.sunLightName != null) {
       this.sunflow.remove(this.sunLightName);
     }
+    // Possible values: default, path
+    String globalIllumination = getRenderingParameterValue("globalIllumination");
     float [] sunDirection = getSunDirection(this.compass, Camera.convertTimeToTimeZone(camera.getTime(), this.compass.getTimeZone()));
     // Update Sun direction during daytime
     if (sunDirection [1] > -0.075f) {
@@ -355,7 +374,7 @@ public class PhotoRenderer {
             new Vector3((float)Math.sin(compass.getNorthDirection()), 0, (float)Math.cos(compass.getNorthDirection())));
         this.sunflow.parameter("sundir", new Vector3(sunDirection [0], sunDirection [1], sunDirection [2]));
         this.sunflow.parameter("turbidity", 6f);
-        this.sunflow.parameter("samples", this.useAmbientOcclusion ? 0 : 12); 
+        this.sunflow.parameter("samples", this.useSunskyLight ? 12 : 0); 
         this.sunSkyLightName = UUID.randomUUID().toString();
         this.sunflow.light(this.sunSkyLightName, "sunsky");
       }
@@ -372,7 +391,7 @@ public class PhotoRenderer {
       float [] sunColor = sunSkyLight.getSunColor().getRGB();
       
       // Simulate additional Sun with a faraway sphere light of a color depending of the hour of the day
-      int sunPower = this.useAmbientOcclusion ? 40 : 10; 
+      int sunPower = this.useSunskyLight ? 10 : 40; 
       this.sunflow.parameter("radiance", null,
           (this.homeLightColor >> 16) * sunPower * (float)Math.sqrt(sunColor [0]), 
           ((this.homeLightColor >> 8) & 0xFF) * sunPower * (float)Math.sqrt(sunColor [1]), 
@@ -383,7 +402,8 @@ public class PhotoRenderer {
       this.sunLightName = UUID.randomUUID().toString();
       this.sunflow.light(this.sunLightName, "sphere");
 
-      if (this.useAmbientOcclusion) {
+      if (!this.useSunskyLight
+          && "default".equals(globalIllumination)) {
         this.sunflow.parameter("gi.engine", "ambocc");
         this.sunflow.parameter("gi.ambocc.bright", null, new float [] {1, 1, 1});
         // Use complementary color
@@ -395,7 +415,13 @@ public class PhotoRenderer {
         this.sunflow.options(SunflowAPI.DEFAULT_OPTIONS);
       }
     }
-
+    
+    if ("path".equals(globalIllumination)) {
+      this.sunflow.parameter("gi.engine", "path");
+      this.sunflow.parameter("gi.path.samples", 64);
+      this.sunflow.options(SunflowAPI.DEFAULT_OPTIONS);
+    }
+    
     // Update camera lens 
     final String CAMERA_NAME = "camera";    
     switch (camera.getLens()) {
@@ -406,8 +432,8 @@ public class PhotoRenderer {
         this.sunflow.camera(CAMERA_NAME, "fisheye");
         break;
       case NORMAL:
-        this.sunflow.parameter("focus.distance", 250f);
-        this.sunflow.parameter("lens.radius", 1f);
+        this.sunflow.parameter("focus.distance", new Float(getRenderingParameterValue("normalLens.focusDistance")));
+        this.sunflow.parameter("lens.radius", new Float(getRenderingParameterValue("normalLens.radius")));
         this.sunflow.camera(CAMERA_NAME, "thinlens");
         break;
       case PINHOLE:
@@ -451,16 +477,11 @@ public class PhotoRenderer {
     this.sunflow.parameter("resolutionX", image.getWidth());
     this.sunflow.parameter("resolutionY", image.getHeight());
     
-    if (this.quality == Quality.HIGH) {
-      this.sunflow.parameter("filter", "blackman-harris"); // box, gaussian, blackman-harris, sinc, mitchell or triangle
-      // The bigger aa.max is, the cleanest rendering you get
-      this.sunflow.parameter("aa.min", 1);
-      this.sunflow.parameter("aa.max",  2);
-    } else {
-      this.sunflow.parameter("filter", "box");
-      this.sunflow.parameter("aa.min", 0);
-      this.sunflow.parameter("aa.max", 0); 
-    }
+    int antiAliasingMin = Integer.parseInt(getRenderingParameterValue("antiAliasing.min"));
+    int antiAliasingMax = Integer.parseInt(getRenderingParameterValue("antiAliasing.max"));
+    this.sunflow.parameter("filter", antiAliasingMax > 0  ? "blackman-harris"  : "box");
+    this.sunflow.parameter("aa.min", antiAliasingMin);
+    this.sunflow.parameter("aa.max", antiAliasingMax); 
     this.sunflow.parameter("sampler", "bucket"); // ipr, fast or bucket 
 
     // Render image with default camera
@@ -481,6 +502,25 @@ public class PhotoRenderer {
     }
   }
 
+  /**
+   * Returns the value of the given rendering parameter.
+   */
+  private String getRenderingParameterValue(String parameterName) {
+    // Try to retrieve overridden parameter value from System property
+    // (for example: System property com.eteks.sweethome3d.j3d.PhotoRenderer.lowQuality.antiAliasing.min)
+    String prefixedParameter = this.quality.name().toLowerCase() + "Quality." + parameterName;
+    String baseName = PhotoRenderer.class.getName();
+    String value = System.getProperty(baseName + '.' + prefixedParameter);
+    if (value != null) {
+      return value;
+    } else {
+      // Return default value stored in properties resource file
+      // (for example: property lowQuality.antiAliasing.min 
+      //  in com/eteks/sweethome3d/j3d/PhotoRenderer.properties file)
+      return ResourceBundle.getBundle(baseName).getString(prefixedParameter);
+    }
+  }
+
   /**
    * Returns all the light children of the given <code>furniture</code>.  
    */
@@ -555,7 +595,14 @@ public class PhotoRenderer {
   
         String appearanceName = null;
         TexCoordGeneration texCoordGeneration = null;
+        int cullFace = PolygonAttributes.CULL_BACK;
+        boolean backFaceNormalFlip = false;
         if (appearance != null) {
+          PolygonAttributes polygonAttributes = appearance.getPolygonAttributes();
+          if (polygonAttributes != null) {
+            cullFace = polygonAttributes.getCullFace();
+            backFaceNormalFlip = polygonAttributes.getBackFaceNormalFlip();
+          }
           texCoordGeneration = appearance.getTexCoordGeneration();
           appearanceName = "shader" + uuid;
           boolean mirror = shapeName != null
@@ -568,7 +615,7 @@ public class PhotoRenderer {
           String objectNameBase = "object" + uuid + "-" + i;
           // Always ignore normals on walls
           String [] objectsName = exportNodeGeometry(shape.getGeometry(i), parentTransformations, texCoordGeneration, 
-              objectNameBase);
+              cullFace, backFaceNormalFlip, objectNameBase);
           if (objectsName != null) {
             for (String objectName : objectsName) {
               if (appearanceName != null) {
@@ -588,6 +635,8 @@ public class PhotoRenderer {
   private String [] exportNodeGeometry(Geometry geometry, 
                                        Transform3D parentTransformations, 
                                        TexCoordGeneration texCoordGeneration, 
+                                       int cullFace, 
+                                       boolean backFaceNormalFlip,
                                        String objectNameBase) {
     if (geometry instanceof GeometryArray) {
       GeometryArray geometryArray = (GeometryArray)geometry;
@@ -683,7 +732,7 @@ public class PhotoRenderer {
               for (int index = 0, i = vertexSize - 6, n = geometryArray.getVertexCount(); 
                    index < n; index++, i += vertexSize) {
                 Vector3f normal = new Vector3f(vertexData [i], vertexData [i + 1], vertexData [i + 2]);
-                exportNormal(parentTransformations, normal, index, normals);
+                exportNormal(parentTransformations, normal, index, normals, backFaceNormalFlip);
               }
             }
             // Export texture coordinates
@@ -715,7 +764,7 @@ public class PhotoRenderer {
               float [] normalCoordinates = geometryArray.getNormalRefFloat();
               for (int index = 0, i = 0, n = geometryArray.getVertexCount(); index < n; index++, i += 3) {
                 Vector3f normal = new Vector3f(normalCoordinates [i], normalCoordinates [i + 1], normalCoordinates [i + 2]);
-                exportNormal(parentTransformations, normal, index, normals);
+                exportNormal(parentTransformations, normal, index, normals, backFaceNormalFlip);
               }
             }
             // Export texture coordinates
@@ -747,7 +796,7 @@ public class PhotoRenderer {
             for (int index = 0, n = geometryArray.getVertexCount(); index < n; index++) {
               Vector3f normal = new Vector3f();
               geometryArray.getNormal(index, normal);
-              exportNormal(parentTransformations, normal, index, normals);
+              exportNormal(parentTransformations, normal, index, normals, backFaceNormalFlip);
             }
           }
           // Export texture coordinates
@@ -789,15 +838,15 @@ public class PhotoRenderer {
               IndexedTriangleArray triangleArray = (IndexedTriangleArray)geometryArray;
               for (int i = 0, n = triangleArray.getIndexCount(), triangleIndex = 0; i < n; i += 3) {
                 triangleIndex = exportIndexedTriangle(triangleArray, i, i + 1, i + 2, 
-                    verticesIndices, normalsIndices, uvsIndices, triangleIndex, vertices, exportedTriangles);
+                    verticesIndices, normalsIndices, uvsIndices, triangleIndex, vertices, exportedTriangles, cullFace);
               }
             } else if (geometryArray instanceof IndexedQuadArray) {
               IndexedQuadArray quadArray = (IndexedQuadArray)geometryArray;
               for (int i = 0, n = quadArray.getIndexCount(), triangleIndex = 0; i < n; i += 4) {
                 triangleIndex = exportIndexedTriangle(quadArray, i, i + 1, i + 2, 
-                    verticesIndices, normalsIndices, uvsIndices, triangleIndex, vertices, exportedTriangles);
+                    verticesIndices, normalsIndices, uvsIndices, triangleIndex, vertices, exportedTriangles, cullFace);
                 triangleIndex = exportIndexedTriangle(quadArray, i, i + 2, i + 3, 
-                    verticesIndices, normalsIndices, uvsIndices, triangleIndex, vertices, exportedTriangles);
+                    verticesIndices, normalsIndices, uvsIndices, triangleIndex, vertices, exportedTriangles, cullFace);
               }
             } else if (geometryArray instanceof IndexedLineStripArray) {
               IndexedLineStripArray lineStripArray = (IndexedLineStripArray)geometryArray;
@@ -815,10 +864,10 @@ public class PhotoRenderer {
                      i < n; i++, j++) {
                   if (j % 2 == 0) {
                     triangleIndex = exportIndexedTriangle(triangleStripArray, i, i + 1, i + 2, 
-                        verticesIndices, normalsIndices, uvsIndices, triangleIndex, vertices, exportedTriangles);
+                        verticesIndices, normalsIndices, uvsIndices, triangleIndex, vertices, exportedTriangles, cullFace);
                   } else { // Vertices of odd triangles are in reverse order               
                     triangleIndex = exportIndexedTriangle(triangleStripArray, i, i + 2, i + 1, 
-                        verticesIndices, normalsIndices, uvsIndices, triangleIndex, vertices, exportedTriangles);
+                        verticesIndices, normalsIndices, uvsIndices, triangleIndex, vertices, exportedTriangles, cullFace);
                   }
                 }
                 initialIndex += stripVertexCount [strip];
@@ -829,7 +878,7 @@ public class PhotoRenderer {
                 for (int i = initialIndex, n = initialIndex + stripVertexCount [strip] - 2; 
                      i < n; i++) {
                   triangleIndex = exportIndexedTriangle(triangleFanArray, initialIndex, i + 1, i + 2, 
-                      verticesIndices, normalsIndices, uvsIndices, triangleIndex, vertices, exportedTriangles);
+                      verticesIndices, normalsIndices, uvsIndices, triangleIndex, vertices, exportedTriangles, cullFace);
                 }
                 initialIndex += stripVertexCount [strip];
               }
@@ -883,15 +932,15 @@ public class PhotoRenderer {
               TriangleArray triangleArray = (TriangleArray)geometryArray;
               for (int i = 0, n = triangleArray.getVertexCount(), triangleIndex = 0; i < n; i += 3) {
                 triangleIndex = exportTriangle(triangleArray, i, i + 1, i + 2, 
-                    verticesIndices, triangleIndex, vertices, exportedTriangles);
+                    verticesIndices, triangleIndex, vertices, exportedTriangles, cullFace);
               }
             } else if (geometryArray instanceof QuadArray) {
               QuadArray quadArray = (QuadArray)geometryArray;
               for (int i = 0, n = quadArray.getVertexCount(), triangleIndex = 0; i < n; i += 4) {
                 triangleIndex = exportTriangle(quadArray, i, i + 1, i + 2, 
-                    verticesIndices, triangleIndex, vertices, exportedTriangles);
+                    verticesIndices, triangleIndex, vertices, exportedTriangles, cullFace);
                 triangleIndex = exportTriangle(quadArray, i + 2, i + 3, i, 
-                    verticesIndices, triangleIndex, vertices, exportedTriangles);
+                    verticesIndices, triangleIndex, vertices, exportedTriangles, cullFace);
               }
             } else if (geometryArray instanceof LineStripArray) {
               LineStripArray lineStripArray = (LineStripArray)geometryArray;
@@ -909,10 +958,10 @@ public class PhotoRenderer {
                      i < n; i++, j++) {
                   if (j % 2 == 0) {
                     triangleIndex = exportTriangle(triangleStripArray, i, i + 1, i + 2, 
-                        verticesIndices, triangleIndex, vertices, exportedTriangles);
+                        verticesIndices, triangleIndex, vertices, exportedTriangles, cullFace);
                   } else { // Vertices of odd triangles are in reverse order               
                     triangleIndex = exportTriangle(triangleStripArray, i, i + 2, i + 1, 
-                        verticesIndices, triangleIndex, vertices, exportedTriangles);
+                        verticesIndices, triangleIndex, vertices, exportedTriangles, cullFace);
                   }
                 }
                 initialIndex += stripVertexCount [strip];
@@ -923,7 +972,7 @@ public class PhotoRenderer {
                 for (int i = initialIndex, n = initialIndex + stripVertexCount [strip] - 2; 
                      i < n; i++) {
                   triangleIndex = exportTriangle(triangleFanArray, initialIndex, i + 1, i + 2, verticesIndices, 
-                      triangleIndex, vertices, exportedTriangles);
+                      triangleIndex, vertices, exportedTriangles, cullFace);
                 }
                 initialIndex += stripVertexCount [strip];
               }
@@ -1050,7 +1099,11 @@ public class PhotoRenderer {
    */
   private void exportNormal(Transform3D transformationToParent,
                             Vector3f normal, int index,
-                            float [] normals) {
+                            float [] normals,
+                            boolean backFaceNormalFlip) {
+    if (backFaceNormalFlip) {
+      normal.negate();
+    }
     transformationToParent.transform(normal);
     int i = index * 3;
     normals [i++] = normal.x;
@@ -1087,7 +1140,15 @@ public class PhotoRenderer {
                                     int [] verticesIndices, int [] normalsIndices, int [] textureCoordinatesIndices, 
                                     int index, 
                                     float [] vertices, 
-                                    Set<Triangle> exportedTriangles) {
+                                    Set<Triangle> exportedTriangles,
+                                    int cullFace) {
+    if (cullFace == PolygonAttributes.CULL_FRONT) {
+      // Reverse vertex order
+      int tmp = vertexIndex1;
+      vertexIndex1 = vertexIndex3;
+      vertexIndex3 = tmp;
+    }
+    
     int coordinateIndex1 = geometryArray.getCoordinateIndex(vertexIndex1);
     int coordinateIndex2 = geometryArray.getCoordinateIndex(vertexIndex2);
     int coordinateIndex3 = geometryArray.getCoordinateIndex(vertexIndex3);
@@ -1129,7 +1190,15 @@ public class PhotoRenderer {
                              int vertexIndex1, int vertexIndex2, int vertexIndex3,
                              int [] verticesIndices, int index, 
                              float [] vertices, 
-                             Set<Triangle> exportedTriangles) {
+                             Set<Triangle> exportedTriangles,
+                             int cullFace) {
+    if (cullFace == PolygonAttributes.CULL_FRONT) {
+      // Reverse vertex order
+      int tmp = vertexIndex1;
+      vertexIndex1 = vertexIndex3;
+      vertexIndex3 = tmp;
+    }
+    
     Triangle exportedTriangle = new Triangle(vertices, vertexIndex1, vertexIndex2, vertexIndex3);
     if (!exportedTriangles.contains(exportedTriangle)) {
       exportedTriangles.add(exportedTriangle);
