@@ -203,6 +203,15 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
     return null;
   }
 
+  /**
+   * Returns <code>null</code>.
+   * @since 4.2
+   */
+  @Override
+  public String getInformation() {
+    return null;
+  }
+  
   /**
    * Returns <code>true</code> if all furniture of this group are movable.
    */
@@ -321,7 +330,7 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
   public float [][] getModelRotation() {
     return IDENTITY;
   }
-  
+
   /**
    * Returns <code>null</code>.
    * @since 3.5
@@ -331,6 +340,15 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
     return null;
   }
   
+  /**
+   * Returns <code>null</code>.
+   * @since 4.2
+   */
+  @Override
+  public String getCreator() {
+    return null;
+  }
+  
   /**
    * Returns the price of the furniture of this group with a price.
    */
@@ -733,7 +751,12 @@ public class HomeFurnitureGroup extends HomePieceOfFurniture {
     // Deep clone furniture managed by this group
     clone.furniture = new ArrayList<HomePieceOfFurniture>(this.furniture.size());
     for (HomePieceOfFurniture piece : this.furniture) {
-      clone.furniture.add(piece.clone());
+      HomePieceOfFurniture pieceClone = piece.clone();
+      clone.furniture.add(pieceClone);
+      if (piece instanceof HomeDoorOrWindow
+          && ((HomeDoorOrWindow)piece).isBoundToWall()) {
+        ((HomeDoorOrWindow)pieceClone).setBoundToWall(true);
+      }
     }
     clone.furniture = Collections.unmodifiableList(clone.furniture);
     if (this.furnitureDefaultColors != null)  {
