@@ -29,10 +29,12 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashMap;
+import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.MissingResourceException;
+import java.util.Properties;
 import java.util.PropertyPermission;
 import java.util.PropertyResourceBundle;
 import java.util.ResourceBundle;
@@ -47,10 +49,15 @@ public abstract class UserPreferences {
    * to user preferences will be notified under a property name equal to the string value of one these properties.
    */
   public enum Property {LANGUAGE, SUPPORTED_LANGUAGES, UNIT, MAGNETISM_ENABLED, RULERS_VISIBLE, GRID_VISIBLE, 
-                        FURNITURE_VIEWED_FROM_TOP, ROOM_FLOOR_COLORED_OR_TEXTURED, WALL_PATTERN,    
+                        FURNITURE_VIEWED_FROM_TOP, ROOM_FLOOR_COLORED_OR_TEXTURED, WALL_PATTERN, NEW_WALL_PATTERN,    
                         NEW_WALL_HEIGHT, NEW_WALL_THICKNESS, NEW_FLOOR_THICKNESS, RECENT_HOMES, IGNORED_ACTION_TIP,
-                        FURNITURE_CATALOG_VIEWED_IN_TREE, NAVIGATION_PANEL_VISIBLE, 
-                        AUTO_SAVE_DELAY_FOR_RECOVERY, AUTO_COMPLETION_STRINGS}
+                        FURNITURE_CATALOG_VIEWED_IN_TREE, NAVIGATION_PANEL_VISIBLE, AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED, 
+                        CHECK_UPDATES_ENABLED, UPDATES_MINIMUM_DATE, AUTO_SAVE_DELAY_FOR_RECOVERY, AUTO_COMPLETION_STRINGS,
+                        RECENT_COLORS}
+  
+  public static final String FURNITURE_LIBRARY_TYPE = "Furniture library"; 
+  public static final String TEXTURES_LIBRARY_TYPE  = "Textures library"; 
+  public static final String LANGUAGE_LIBRARY_TYPE  = "Language library"; 
   
   private static final String [] DEFAULT_SUPPORTED_LANGUAGES; 
   private static final List<ClassLoader> DEFAULT_CLASS_LOADER = 
@@ -60,8 +67,21 @@ public abstract class UserPreferences {
   private static final TextStyle DEFAULT_ROOM_TEXT_STYLE = new TextStyle(24f);
 
   static {
-    ResourceBundle resource = ResourceBundle.getBundle(UserPreferences.class.getName());
-    DEFAULT_SUPPORTED_LANGUAGES = resource.getString("supportedLanguages").split("\\s");
+    Properties supportedLanguagesProperties = new Properties();
+    String [] defaultSupportedLanguages;
+    try {
+      // As of version 4.1 where Trusted-Library manifest attribute was added to applet jars, 
+      // UserPreferences.properties was renamed as SupportedLanguages.properties
+      // because strangely UserPreferences.properties resource couldn't be found in applet environment
+      InputStream in = UserPreferences.class.getResourceAsStream("SupportedLanguages.properties");
+      supportedLanguagesProperties.load(in);
+      in.close();
+      // Get property value of supportedLanguages
+      defaultSupportedLanguages = supportedLanguagesProperties.getProperty("supportedLanguages", "en").split("\\s");
+    } catch (IOException ex) {
+      defaultSupportedLanguages = new String [] {"en"};
+    }
+    DEFAULT_SUPPORTED_LANGUAGES = defaultSupportedLanguages;
   }
   
   private final PropertyChangeSupport          propertyChangeSupport;
@@ -77,6 +97,7 @@ public abstract class UserPreferences {
   private String           currency;
   private LengthUnit       unit;
   private boolean          furnitureCatalogViewedInTree = true;
+  private boolean          aerialViewCenteredOnSelectionEnabled;
   private boolean          navigationPanelVisible = true;
   private boolean          magnetismEnabled    = true;
   private boolean          rulersVisible       = true;
@@ -84,12 +105,16 @@ public abstract class UserPreferences {
   private boolean          furnitureViewedFromTop;
   private boolean          roomFloorColoredOrTextured;
   private TextureImage     wallPattern;
+  private TextureImage     newWallPattern;
   private float            newWallThickness;
   private float            newWallHeight;
   private float            newFloorThickness;
   private List<String>     recentHomes;
+  private boolean          checkUpdatesEnabled;
+  private Long             updatesMinimumDate;
   private int              autoSaveDelayForRecovery;
   private Map<String, List<String>>  autoCompletionStrings;
+  private List<Integer>    recentColors;
 
   /**
    * Creates user preferences.</br> 
@@ -99,7 +124,7 @@ public abstract class UserPreferences {
     this.propertyChangeSupport = new PropertyChangeSupport(this);
     this.classResourceBundles = new HashMap<Class<?>, ResourceBundle>();
     this.resourceBundles = new HashMap<String, ResourceBundle>();
-    this.autoCompletionStrings = new HashMap<String, List<String>>();
+    this.autoCompletionStrings = new LinkedHashMap<String, List<String>>();
 
     this.supportedLanguages = DEFAULT_SUPPORTED_LANGUAGES;
     this.defaultCountry = Locale.getDefault().getCountry();    
@@ -308,7 +333,7 @@ public abstract class UserPreferences {
    * This implementation searches first the key in a properties file named as 
    * <code>resourceClass</code>, then if this file doesn't exist, it searches 
    * the key prefixed by <code>resourceClass</code> name and a dot in a package.properties file 
-   * in the directory matching the package of <code>resourceClass</code>. 
+   * in the folder matching the package of <code>resourceClass</code>. 
    * @exception IllegalArgumentException if no string for the given key can be found
    */
   public String getLocalizedString(Class<?> resourceClass,
@@ -346,7 +371,7 @@ public abstract class UserPreferences {
    * Returns the string matching <code>resourceKey</code> in current language 
    * for the given resource family.
    * <code>resourceFamily</code> should match the absolute path of a .properties resource family,
-   * shouldn't start by a slash and may contain dots '.' or slash '/' as directory separators. 
+   * shouldn't start by a slash and may contain dots '.' or slash '/' as folder separators. 
    * If <code>resourceParameters</code> isn't empty the string is considered
    * as a format string, and the returned string will be formatted with these parameters. 
    * This implementation searches the key in a properties file named as 
@@ -493,6 +518,26 @@ public abstract class UserPreferences {
     }
   }
   
+  /**
+   * Sets whether aerial view should be centered on selection or not.
+   * @since 4.0
+   */
+  public void setAerialViewCenteredOnSelectionEnabled(boolean aerialViewCenteredOnSelectionEnabled) {
+    if (aerialViewCenteredOnSelectionEnabled != this.aerialViewCenteredOnSelectionEnabled) {
+      this.aerialViewCenteredOnSelectionEnabled = aerialViewCenteredOnSelectionEnabled;
+      this.propertyChangeSupport.firePropertyChange(Property.AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED.name(), 
+          !aerialViewCenteredOnSelectionEnabled, aerialViewCenteredOnSelectionEnabled);
+    }
+  }
+  
+  /**
+   * Returns whether aerial view should be centered on selection or not.
+   * @since 4.0
+   */
+  public boolean isAerialViewCenteredOnSelectionEnabled() {
+    return this.aerialViewCenteredOnSelectionEnabled;
+  }
+
   /**
    * Returns <code>true</code> if magnetism is enabled.
    * @return <code>true</code> by default.
@@ -629,6 +674,28 @@ public abstract class UserPreferences {
     }
   }
 
+  /**
+   * Returns the pattern used for new walls in plan or <code>null</code> if it's not set.
+   * @since 4.0
+   */
+  public TextureImage getNewWallPattern() {
+    return this.newWallPattern;
+  }
+  
+  /**
+   * Sets how new walls should be displayed in plan, and notifies
+   * listeners of this change.
+   * @since 4.0 
+   */
+  public void setNewWallPattern(TextureImage newWallPattern) {
+    if (this.newWallPattern != newWallPattern) {
+      TextureImage oldWallPattern = this.newWallPattern;
+      this.newWallPattern = newWallPattern;
+      this.propertyChangeSupport.firePropertyChange(Property.NEW_WALL_PATTERN.name(), 
+          oldWallPattern, newWallPattern);
+    }
+  }
+
   /**
    * Returns default thickness of new walls in home. 
    */
@@ -684,10 +751,54 @@ public abstract class UserPreferences {
    */
   public void setNewFloorThickness(float newFloorThickness) {
     if (this.newFloorThickness != newFloorThickness) {
-      float oldDefaultThickness = this.newFloorThickness;
+      float oldFloorThickness = this.newFloorThickness;
       this.newFloorThickness = newFloorThickness;
       this.propertyChangeSupport.firePropertyChange(Property.NEW_FLOOR_THICKNESS.name(), 
-          oldDefaultThickness, newFloorThickness);
+          oldFloorThickness, newFloorThickness);
+    }
+  }
+
+  /**
+   * Returns <code>true</code> if updates should be checked.
+   * @since 4.0
+   */
+  public boolean isCheckUpdatesEnabled() {
+    return this.checkUpdatesEnabled;
+  }
+
+  /**
+   * Sets whether updates should be checked or not.
+   * @since 4.0
+   */
+  public void setCheckUpdatesEnabled(boolean updatesChecked) {
+    if (updatesChecked != this.checkUpdatesEnabled) {
+      this.checkUpdatesEnabled = updatesChecked;
+      this.propertyChangeSupport.firePropertyChange(Property.CHECK_UPDATES_ENABLED.name(), 
+          !updatesChecked, updatesChecked);
+    }
+  }
+
+  /**
+   * Returns the minimum date of updates that may interest user.
+   * @return the date expressed in millis second since the epoch or <code>null</code> if not defined. 
+   * @since 4.0
+   */
+  public Long getUpdatesMinimumDate() {
+    return this.updatesMinimumDate;
+  }
+
+  /**
+   * Sets the minimum date of updates that may interest user, and notifies
+   * listeners of this change.  
+   * @since 4.0
+   */
+  public void setUpdatesMinimumDate(Long updatesMinimumDate) {
+    if (this.updatesMinimumDate != updatesMinimumDate
+        && (updatesMinimumDate == null || !updatesMinimumDate.equals(this.updatesMinimumDate))) {
+      Long oldUpdatesMinimumDate = this.updatesMinimumDate;
+      this.updatesMinimumDate = updatesMinimumDate;
+      this.propertyChangeSupport.firePropertyChange(Property.UPDATES_MINIMUM_DATE.name(), 
+          oldUpdatesMinimumDate, updatesMinimumDate);
     }
   }
 
@@ -797,14 +908,18 @@ public abstract class UserPreferences {
    * @since 3.4
    */
   public void addAutoCompletionString(String property, String autoCompletionString) {
-    if (autoCompletionString.length() > 0) {
+    if (autoCompletionString != null 
+        && autoCompletionString.length() > 0) {
       List<String> propertyAutoCompletionStrings = this.autoCompletionStrings.get(property);
-      if (propertyAutoCompletionStrings != null
-          && !propertyAutoCompletionStrings.contains(autoCompletionString)) {
+      if (propertyAutoCompletionStrings == null) {
+        propertyAutoCompletionStrings = new ArrayList<String>();
+      } else if (!propertyAutoCompletionStrings.contains(autoCompletionString)) {
         propertyAutoCompletionStrings = new ArrayList<String>(propertyAutoCompletionStrings);
-        propertyAutoCompletionStrings.add(0, autoCompletionString);
-        setAutoCompletionStrings(property, propertyAutoCompletionStrings);
+      } else {
+        return;
       }
+      propertyAutoCompletionStrings.add(0, autoCompletionString);
+      setAutoCompletionStrings(property, propertyAutoCompletionStrings);
     }
   }
   
@@ -834,48 +949,74 @@ public abstract class UserPreferences {
   }
   
   /**
-   * Adds <code>languageLibraryName</code> to the first language libraries folder
-   * to make the language library it contains available to supported languages.
-   * @param languageLibraryName  the name of the resource in which the library will be written. 
+   * Returns an unmodifiable list of the recent colors.
+   * @since 4.0
+   */
+  public List<Integer> getRecentColors() {
+    return Collections.unmodifiableList(this.recentColors);
+  }
+
+  /**
+   * Sets the recent colors list and notifies listeners of this change.
+   * @since 4.0
+   */
+  public void setRecentColors(List<Integer> recentColors) {
+    if (!recentColors.equals(this.recentColors)) {
+      List<Integer> oldRecentColors = this.recentColors;
+      this.recentColors = new ArrayList<Integer>(recentColors);
+      this.propertyChangeSupport.firePropertyChange(Property.RECENT_COLORS.name(), 
+          oldRecentColors, getRecentColors());
+    }
+  }
+
+  /**
+   * Adds the language library to make the languages it contains available to supported languages.
+   * @param languageLibraryLocation  the location where the library can be found. 
    * @since 2.3 
    */
-  public abstract void addLanguageLibrary(String languageLibraryName) throws RecorderException;
+  public abstract void addLanguageLibrary(String languageLibraryLocation) throws RecorderException;
   
   /**
-   * Returns <code>true</code> if the given language library exists.
-   * @param languageLibraryName the name of the resource to check
+   * Returns <code>true</code> if the language library at the given location exists.
+   * @param languageLibraryLocation the name of the resource to check
    * @since 2.3 
    */
-  public abstract boolean languageLibraryExists(String languageLibraryName) throws RecorderException;
+  public abstract boolean languageLibraryExists(String languageLibraryLocation) throws RecorderException;
 
   /**
    * Adds <code>furnitureLibraryName</code> to furniture catalog  
-   * to make the furniture library it contains available.
-   * @param furnitureLibraryName  the name of the resource in which the library will be written. 
+   * to make the furniture it contains available.
+   * @param furnitureLibraryLocation  the location where the library can be found. 
    */
-  public abstract void addFurnitureLibrary(String furnitureLibraryName) throws RecorderException;
+  public abstract void addFurnitureLibrary(String furnitureLibraryLocation) throws RecorderException;
   
   /**
-   * Returns <code>true</code> if the given furniture library exists.
-   * @param furnitureLibraryName the name of the resource to check
+   * Returns <code>true</code> if the furniture library at the given location exists.
+   * @param furnitureLibraryLocation the name of the resource to check
    */
-  public abstract boolean furnitureLibraryExists(String furnitureLibraryName) throws RecorderException;
+  public abstract boolean furnitureLibraryExists(String furnitureLibraryLocation) throws RecorderException;
 
   /**
-   * Adds <code>texturesLibraryName</code> to textures catalog  
-   * to make the textures library it contains available.
-   * @param texturesLibraryName  the name of the resource in which the library will be written.
+   * Adds the textures library at the given location to textures catalog  
+   * to make the textures it contains available.
+   * @param texturesLibraryLocation  the location where the library can be found.
    * @since 2.3 
    */
-  public abstract void addTexturesLibrary(String texturesLibraryName) throws RecorderException;
+  public abstract void addTexturesLibrary(String texturesLibraryLocation) throws RecorderException;
   
   /**
-   * Returns <code>true</code> if the given textures library exists.
-   * @param texturesLibraryName the name of the resource to check
+   * Returns <code>true</code> if the textures library at the given location exists.
+   * @param texturesLibraryLocation the name of the resource to check
    * @since 2.3 
    */
-  public abstract boolean texturesLibraryExists(String texturesLibraryName) throws RecorderException;
+  public abstract boolean texturesLibraryExists(String texturesLibraryLocation) throws RecorderException;
 
+  /**
+   * Returns the libraries available in user preferences. 
+   * @since 4.0
+   */
+  public abstract List<Library> getLibraries();
+  
   /**
    * A resource bundle with a prefix added to resource key.
    */
