@@ -45,7 +45,7 @@ public class Wall implements Serializable, Selectable, Elevatable {
                         THICKNESS, HEIGHT, HEIGHT_AT_END, 
                         LEFT_SIDE_COLOR, LEFT_SIDE_TEXTURE, LEFT_SIDE_SHININESS, 
                         RIGHT_SIDE_COLOR, RIGHT_SIDE_TEXTURE, RIGHT_SIDE_SHININESS,
-                        PATTERN, LEVEL}
+                        PATTERN, TOP_COLOR, LEVEL}
   
   private static final long serialVersionUID = 1L;
   
@@ -67,6 +67,7 @@ public class Wall implements Serializable, Selectable, Elevatable {
   private float        rightSideShininess;
   private boolean      symmetric = true;
   private TextureImage pattern;  
+  private Integer      topColor;
   private Level        level;
   
   private transient PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
@@ -80,21 +81,33 @@ public class Wall implements Serializable, Selectable, Elevatable {
    * @deprecated specify a height with the {@linkplain #Wall(float, float, float, float, float, float) other constructor}.
    */
   public Wall(float xStart, float yStart, float xEnd, float yEnd, float thickness) {
-    this.xStart = xStart;
-    this.yStart = yStart;
-    this.xEnd = xEnd;
-    this.yEnd = yEnd;
-    this.thickness = thickness;
+    this(xStart, yStart, xEnd, yEnd, thickness, 0);
   }
   
   /**
    * Creates a wall from (<code>xStart</code>,<code>yStart</code>)
    * to (<code>xEnd</code>, <code>yEnd</code>), 
-   * with given thickness and height. Left and right colors are <code>null</code>.
+   * with given thickness and height. Pattern, left and right colors are <code>null</code>.
    */
   public Wall(float xStart, float yStart, float xEnd, float yEnd, float thickness, float height) {
-    this(xStart, yStart, xEnd, yEnd, thickness);
+    this(xStart, yStart, xEnd, yEnd, thickness, height, null);
+  }
+  
+  /**
+   * Creates a wall from (<code>xStart</code>,<code>yStart</code>)
+   * to (<code>xEnd</code>, <code>yEnd</code>), 
+   * with given thickness, height and pattern. 
+   * Colors are <code>null</code>.
+   * @since 4.0
+   */
+  public Wall(float xStart, float yStart, float xEnd, float yEnd, float thickness, float height, TextureImage pattern) {
+    this.xStart = xStart;
+    this.yStart = yStart;
+    this.xEnd = xEnd;
+    this.yEnd = yEnd;
+    this.thickness = thickness;
     this.height = height;
+    this.pattern = pattern;
   }
   
   /**
@@ -250,6 +263,7 @@ public class Wall implements Serializable, Selectable, Elevatable {
   /**
    * Returns the abscissa of the arc circle center of this wall.
    * If the wall isn't round, the return abscissa is at the middle of the wall. 
+   * @since 3.0
    */
   public float getXArcCircleCenter() {
     if (this.arcExtent == null) {
@@ -262,6 +276,7 @@ public class Wall implements Serializable, Selectable, Elevatable {
   /**
    * Returns the ordinate of the arc circle center of this wall.
    * If the wall isn't round, the return ordinate is at the middle of the wall. 
+   * @since 3.0
    */
   public float getYArcCircleCenter() {
     if (this.arcExtent == null) {
@@ -434,7 +449,7 @@ public class Wall implements Serializable, Selectable, Elevatable {
    */
   public void setHeightAtEnd(Float heightAtEnd) {
     if (heightAtEnd != this.heightAtEnd
-        || (heightAtEnd != null && !heightAtEnd.equals(this.heightAtEnd))) {
+        && (heightAtEnd == null || !heightAtEnd.equals(this.heightAtEnd))) {
       Float oldHeightAtEnd = this.heightAtEnd;
       this.heightAtEnd = heightAtEnd;
       this.propertyChangeSupport.firePropertyChange(Property.HEIGHT_AT_END.name(), 
@@ -466,7 +481,7 @@ public class Wall implements Serializable, Selectable, Elevatable {
    */
   public void setLeftSideColor(Integer leftSideColor) {
     if (leftSideColor != this.leftSideColor
-        || (leftSideColor != null && !leftSideColor.equals(this.leftSideColor))) {
+        && (leftSideColor == null || !leftSideColor.equals(this.leftSideColor))) {
       Integer oldLeftSideColor = this.leftSideColor;
       this.leftSideColor = leftSideColor;
       this.propertyChangeSupport.firePropertyChange(Property.LEFT_SIDE_COLOR.name(), 
@@ -488,7 +503,7 @@ public class Wall implements Serializable, Selectable, Elevatable {
    */
   public void setRightSideColor(Integer rightSideColor) {
     if (rightSideColor != this.rightSideColor
-        || (rightSideColor != null && !rightSideColor.equals(this.rightSideColor))) {
+        && (rightSideColor == null || !rightSideColor.equals(this.rightSideColor))) {
       Integer oldLeftSideColor = this.rightSideColor;
       this.rightSideColor = rightSideColor;
       this.propertyChangeSupport.firePropertyChange(Property.RIGHT_SIDE_COLOR.name(), 
@@ -510,7 +525,7 @@ public class Wall implements Serializable, Selectable, Elevatable {
    */
   public void setLeftSideTexture(HomeTexture leftSideTexture) {
     if (leftSideTexture != this.leftSideTexture
-        || (leftSideTexture != null && !leftSideTexture.equals(this.leftSideTexture))) {
+        && (leftSideTexture == null || !leftSideTexture.equals(this.leftSideTexture))) {
       HomeTexture oldLeftSideTexture = this.leftSideTexture;
       this.leftSideTexture = leftSideTexture;
       this.propertyChangeSupport.firePropertyChange(Property.LEFT_SIDE_TEXTURE.name(), 
@@ -531,7 +546,7 @@ public class Wall implements Serializable, Selectable, Elevatable {
    */
   public void setRightSideTexture(HomeTexture rightSideTexture) {
     if (rightSideTexture != this.rightSideTexture
-        || (rightSideTexture != null && !rightSideTexture.equals(this.rightSideTexture))) {
+        && (rightSideTexture == null || !rightSideTexture.equals(this.rightSideTexture))) {
       HomeTexture oldLeftSideTexture = this.rightSideTexture;
       this.rightSideTexture = rightSideTexture;
       this.propertyChangeSupport.firePropertyChange(Property.RIGHT_SIDE_TEXTURE.name(), 
@@ -584,7 +599,7 @@ public class Wall implements Serializable, Selectable, Elevatable {
   }
 
   /**
-   * Returns the wall pattern in plan.
+   * Returns the pattern of this wall in the plan.
    * @since 3.3
    */
   public TextureImage getPattern() {
@@ -592,7 +607,7 @@ public class Wall implements Serializable, Selectable, Elevatable {
   }
   
   /**
-   * Sets how walls should be displayed in plan, and notifies
+   * Sets the pattern of this wall in the plan, and notifies
    * listeners of this change.
    * @since 3.3 
    */
@@ -605,6 +620,29 @@ public class Wall implements Serializable, Selectable, Elevatable {
     }
   }
 
+  /**
+   * Returns the color of the top of this wall in the 3D view.
+   * @since 4.0
+   */
+  public Integer getTopColor() {
+    return this.topColor;
+  }
+  
+  /**
+   * Sets the color of the top of this wall in the 3D view, and notifies
+   * listeners of this change.
+   * @since 4.0 
+   */
+  public void setTopColor(Integer topColor) {
+    if (this.topColor != topColor
+        && (topColor == null || !topColor.equals(this.topColor))) {
+      Integer oldTopColor = this.topColor;
+      this.topColor = topColor;
+      this.propertyChangeSupport.firePropertyChange(Property.TOP_COLOR.name(), 
+          oldTopColor, topColor);
+    }
+  }
+
   /**
    * Returns the level which this wall belongs to. 
    * @since 3.4
@@ -902,7 +940,7 @@ public class Wall implements Serializable, Selectable, Elevatable {
         }
       } else {
         boolean sameSignum = Math.signum(alpha1) == Math.signum(alpha2);
-        if ((sameSignum && (Math.abs(alpha1) > Math.abs(alpha2)   ? alpha1 / alpha2   : alpha2 / alpha1) > 1.0001)
+        if ((sameSignum && (Math.abs(alpha1) > Math.abs(alpha2)   ? alpha1 / alpha2   : alpha2 / alpha1) > 1.004)
             || (!sameSignum && Math.abs(alpha1 - alpha2) > 1E-5)) {
           float beta1 = point2 [1] - alpha1 * point2 [0];
           float beta2 = point4 [1] - alpha2 * point4 [0];
