@@ -22,6 +22,7 @@ package com.eteks.sweethome3d.swing;
 import java.awt.AWTException;
 import java.awt.Color;
 import java.awt.Component;
+import java.awt.Container;
 import java.awt.Cursor;
 import java.awt.Dimension;
 import java.awt.Frame;
@@ -171,8 +172,10 @@ public class ColorButton extends JButton {
                   // Move color at the beginning of the list and ensure it doesn't contain more than 20 colors
                   if (colorIndex > 0) {
                     recentColors.remove(colorIndex);
-                  } else if (recentColors.size() == RecentColorsPanel.MAX_COLORS) {
-                    recentColors.remove(RecentColorsPanel.MAX_COLORS - 1);
+                  } else {
+                    while (recentColors.size() > RecentColorsPanel.MAX_COLORS) {
+                      recentColors.remove(recentColors.size() - 1);
+                    }
                   }
                   recentColors.add(0, color);
                   preferences.setRecentColors(recentColors);     
@@ -212,8 +215,32 @@ public class ColorButton extends JButton {
             // might be ignored when the color chooser dialog is created
             List<AbstractColorChooserPanel> chooserPanels = new ArrayList<AbstractColorChooserPanel>(
                 Arrays.asList(getChooserPanels()));
-            if (chooserPanels.get(0).getClass().getName().contains("DefaultSwatchChooserPanel")) {
+            // Remove swatch chooser panel
+            if (chooserPanels.get(0).getClass().getName().equals("javax.swing.colorchooser.DefaultSwatchChooserPanel")) {
               chooserPanels.remove(0);
+            } 
+            // Remove components used to manage transparency and displayed at the fourth or fifth row 
+            // of the first panel of ColorChooserPanel instances
+            for (AbstractColorChooserPanel chooserPanel : chooserPanels) {
+              if (chooserPanel.getClass().getName().equals("javax.swing.colorchooser.ColorChooserPanel")) {
+                Component colorPanel = chooserPanel.getComponent(0);
+                if (colorPanel.getClass().getName().equals("javax.swing.colorchooser.ColorPanel")
+                    && colorPanel instanceof Container) {
+                  Container colorPanelContainer = (Container)colorPanel;
+                  if (colorPanelContainer.getComponentCount() == 15) {
+                    int transparencyComponentsRow;
+                    if (colorPanelContainer.getComponent(4) instanceof JLabel
+                        && ((JLabel)colorPanelContainer.getComponent(4)).getText().length() == 0) {
+                      transparencyComponentsRow = 3;
+                    } else {
+                      transparencyComponentsRow = 4;
+                    }
+                    colorPanelContainer.remove(transparencyComponentsRow + 10);
+                    colorPanelContainer.remove(transparencyComponentsRow + 5);
+                    colorPanelContainer.remove(transparencyComponentsRow);
+                  }
+                }
+              }
             }
             chooserPanels.add(0, palettesPanel);
             setChooserPanels(chooserPanels.toArray(new AbstractColorChooserPanel [chooserPanels.size()]));
@@ -309,6 +336,7 @@ public class ColorButton extends JButton {
     private Color                 initialColor;
       
     public PalettesColorChooserPanel(UserPreferences preferences) {
+      setOpaque(false);
       this.preferences = preferences;
       addAncestorListener(new AncestorListener() {
           private int initialDelay;
@@ -450,13 +478,15 @@ public class ColorButton extends JButton {
                 // Opacity is set to 0.05f to be almost transparent but still visible enough by the
                 // the system not to switch to another application when the user clicks
                 Window.class.getMethod("setOpacity", float.class).invoke(pipetteWindow, 0.05f);
+                // Enlarge window to reduce mouse cursor change
+                pipetteWindow.setSize(10, 10);
               } else if (OperatingSystem.isJavaVersionGreaterOrEqual("1.6")) {
                 // Call com.sun.awt.AWTUtilities.setWindowOpacity(pipetteWindow, 0.05f)
                 Class<?> awtUtilitiesClass = Class.forName("com.sun.awt.AWTUtilities");
                 awtUtilitiesClass.getMethod("setWindowOpacity", Window.class, float.class).invoke(null, pipetteWindow, 0.05f);
+                // Enlarge window to reduce mouse cursor change
+                pipetteWindow.setSize(10, 10);
               } 
-              // Enlarge window to reduce mouse cursor change
-              pipetteWindow.setSize(10, 10);
             } catch (Exception ex) {
               // For any exception, let's consider simply the method failed or doesn't exist
             }
@@ -1116,7 +1146,7 @@ public class ColorButton extends JButton {
    * Panel showing recent colors.
    */
   private static class RecentColorsPanel extends JPanel {
-    public static final int MAX_COLORS = 20;
+    public static final int MAX_COLORS = OperatingSystem.isJavaVersionGreaterOrEqual("1.7") ? 25 : 20;
     
     private Cursor pipetteCursor;
     
@@ -1136,6 +1166,7 @@ public class ColorButton extends JButton {
             }
           });
       setRecentColors(colorSelectionModel, preferences);
+      setOpaque(false);
     }
 
     private void setRecentColors(final ColorSelectionModel colorSelectionModel, 
@@ -1148,7 +1179,9 @@ public class ColorButton extends JButton {
             i++, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
             GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
       }      
-      for (final Integer color : preferences.getRecentColors()) {
+      List<Integer> recentColors = preferences.getRecentColors();
+      for (int j = 0; j < recentColors.size() && j < MAX_COLORS; j++) {
+        final Integer color = recentColors.get(j);
         Component colorComponent = new JComponent() {
             @Override
             protected void paintComponent(Graphics g) {
@@ -1192,7 +1225,11 @@ public class ColorButton extends JButton {
 
     public Insets getInsets() { 
       if (OperatingSystem.isJavaVersionGreaterOrEqual("1.7")) {
-        return super.getInsets();
+        Insets insets = super.getInsets();
+        if (OperatingSystem.isMacOSXLeopardOrSuperior()) {
+          insets.top += 10;
+        }
+        return insets;
       } else {
         if (UIManager.getLookAndFeel().getID().equals("GTK")) {
           return new Insets(5, 5, 5, 5);
