@@ -39,6 +39,7 @@ import java.awt.event.ComponentAdapter;
 import java.awt.event.ComponentEvent;
 import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
+import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
 import java.awt.event.MouseMotionAdapter;
 import java.beans.PropertyChangeEvent;
@@ -135,11 +136,33 @@ class FurnitureCatalogListPanel extends JPanel implements View {
     final CatalogListModel catalogListModel = new CatalogListModel(catalog);
     this.catalogFurnitureList = new JList(catalogListModel) {
         private FurnitureToolTip toolTip = new FurnitureToolTip(false, preferences);
+        private boolean mousePressed;
+        private boolean firstScroll;
+        
+        {
+          addMouseListener(new MouseAdapter() {
+            @Override
+            public void mousePressed(MouseEvent ev) {
+              firstScroll = true;
+              mousePressed = true;
+            }
+
+            @Override
+            public void mouseReleased(MouseEvent ev) {
+              mousePressed = false;
+            }
+          });
+        }
         
         @Override
         public JToolTip createToolTip() {
-          this.toolTip.setComponent(this);
-          return this.toolTip;
+          if (this.toolTip.isTipTextComplete()) {
+            // Use toolTip object only for its text returned in getToolTipText
+            return super.createToolTip();
+          } else {
+            this.toolTip.setComponent(this);
+            return this.toolTip;
+          }
         }
       
         @Override
@@ -148,14 +171,26 @@ class FurnitureCatalogListPanel extends JPanel implements View {
           int index = locationToIndex(ev.getPoint());
           if (index != -1) {
             this.toolTip.setPieceOfFurniture((CatalogPieceOfFurniture)getModel().getElementAt(index));
-            return this.toolTip.getToolTipText();
+            return this.toolTip.getTipText();
           } else {
             return null;
           }
         }
+        
+        @Override
+        public void scrollRectToVisible(Rectangle rectangle) {
+          if (!this.mousePressed
+              || this.firstScroll) {
+            // During a drag and drop, let's accept only the first viewport adjustment provoked by UI
+            this.firstScroll = false;
+            super.scrollRectToVisible(rectangle);
+          }
+        }
       };
     this.catalogFurnitureList.setLayoutOrientation(JList.HORIZONTAL_WRAP);
     this.catalogFurnitureList.setCellRenderer(new CatalogCellRenderer());
+    this.catalogFurnitureList.setAutoscrolls(false);
+    this.catalogFurnitureList.setTransferHandler(null);
     // Remove Select all action
     this.catalogFurnitureList.getActionMap().getParent().remove("selectAll");
     addDragListener(this.catalogFurnitureList);
@@ -196,7 +231,7 @@ class FurnitureCatalogListPanel extends JPanel implements View {
     updateListSelectedFurniture(catalog, controller);
     addSelectionListeners(catalog, controller);
     
-    this.categoryFilterLabel = new JLabel(preferences.getLocalizedString(
+    this.categoryFilterLabel = new JLabel(SwingTools.getLocalizedLabelText(preferences,
         FurnitureCatalogListPanel.class, "categoryFilterLabel.text"));    
     List<FurnitureCategory> categories = new ArrayList<FurnitureCategory>();
     categories.add(null);
@@ -228,7 +263,7 @@ class FurnitureCatalogListPanel extends JPanel implements View {
         }
       });
     
-    this.searchLabel = new JLabel(preferences.getLocalizedString(
+    this.searchLabel = new JLabel(SwingTools.getLocalizedLabelText(preferences,
         FurnitureCatalogListPanel.class, "searchLabel.text"));
     this.searchTextField = new JTextField(5);
     this.searchTextField.getDocument().addDocumentListener(new DocumentListener() {  
@@ -342,12 +377,6 @@ class FurnitureCatalogListPanel extends JPanel implements View {
                                  final FurnitureCatalogController controller) {
     final Cursor handCursor = new Cursor(Cursor.HAND_CURSOR);
     MouseInputAdapter mouseListener = new MouseInputAdapter () {
-        // Track first drag and drop under Mac OS X / Java 7 where the selection in the catalog
-        // visually changes the first time the cursor enters in the plan component   
-        private boolean                 firstMouseDrag = OperatingSystem.isMacOSXLionOrSuperior()
-                                                         && OperatingSystem.isJavaVersionGreaterOrEqual("1.7");;
-        private CatalogPieceOfFurniture firstSelectedPiece;
-        
         @Override
         public void mouseClicked(MouseEvent ev) {
           if (SwingUtilities.isLeftMouseButton(ev)) {
@@ -379,35 +408,6 @@ class FurnitureCatalogListPanel extends JPanel implements View {
           }
           setCursor(Cursor.getDefaultCursor());
         }
-        
-        @Override
-        public void mouseDragged(MouseEvent ev) {
-          if (this.firstMouseDrag
-              && this.firstSelectedPiece != null) {
-            List<CatalogPieceOfFurniture> selectedFurniture = controller.getSelectedFurniture();
-            if (!selectedFurniture.isEmpty()
-                && this.firstSelectedPiece != selectedFurniture.get(0)) {
-              controller.setSelectedFurniture(Arrays.asList(this.firstSelectedPiece));
-              this.firstMouseDrag = false;
-              this.firstSelectedPiece = null;
-            }
-          }
-        }
-        
-        @Override
-        public void mousePressed(MouseEvent ev) {
-          if (this.firstMouseDrag) {
-            EventQueue.invokeLater(new Runnable() {
-                public void run() {
-                  List<CatalogPieceOfFurniture> selectedFurniture = controller.getSelectedFurniture();
-                  if (!selectedFurniture.isEmpty()) {
-                    // Store the selected piece at the first drag and drop
-                    firstSelectedPiece = selectedFurniture.get(0);
-                  }
-                }
-              });
-          }
-        }
       };
     catalogFurnitureList.addMouseListener(mouseListener);
     catalogFurnitureList.addMouseMotionListener(mouseListener);
