@@ -250,8 +250,13 @@ public class FurnitureCatalogTree extends JTree implements View {
    */
   @Override
   public JToolTip createToolTip() {    
-    this.toolTip.setComponent(this);
-    return this.toolTip;
+    if (this.toolTip.isTipTextComplete()) {
+      // Use toolTip object only for its text returned in getToolTipText
+      return super.createToolTip();
+    } else {
+      this.toolTip.setComponent(this);
+      return this.toolTip;
+    }
   }
 
   /**
@@ -264,7 +269,7 @@ public class FurnitureCatalogTree extends JTree implements View {
         && path != null
         && path.getPathCount() == 3) {
       this.toolTip.setPieceOfFurniture((CatalogPieceOfFurniture)path.getLastPathComponent());
-      return this.toolTip.getToolTipText();
+      return this.toolTip.getTipText();
     } else {
       return null;
     }
