@@ -43,6 +43,8 @@ import java.beans.PropertyChangeListener;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
+import java.net.MalformedURLException;
+import java.net.URL;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
@@ -60,6 +62,7 @@ import javax.swing.JList;
 import javax.swing.JOptionPane;
 import javax.swing.JPanel;
 import javax.swing.JSpinner;
+import javax.swing.JTextArea;
 import javax.swing.JTextField;
 import javax.swing.KeyStroke;
 import javax.swing.SwingUtilities;
@@ -90,6 +93,7 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
   private CardLayout                      cardLayout;
   private JLabel                          imageChoiceOrChangeLabel;
   private JButton                         imageChoiceOrChangeButton;
+  private JButton                         findImagesButton;
   private JLabel                          imageChoiceErrorLabel;
   private ScaledImageComponent            imageChoicePreviewComponent;
   private JLabel                          attributesLabel;
@@ -149,6 +153,38 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
           }
         }
       });
+    try {
+      this.findImagesButton = new JButton(SwingTools.getLocalizedLabelText(preferences, 
+          ImportedTextureWizardStepsPanel.class, "findImagesButton.text"));
+      final String findImagesUrl = preferences.getLocalizedString(
+          ImportedTextureWizardStepsPanel.class, "findImagesButton.url");
+      this.findImagesButton.addActionListener(new ActionListener() {
+          public void actionPerformed(ActionEvent ev) {
+            boolean documentShown = false;
+            try { 
+              // Display Find models page in browser
+              documentShown = SwingTools.showDocumentInBrowser(new URL(findImagesUrl)); 
+            } catch (MalformedURLException ex) {
+              // Document isn't shown
+            }
+            if (!documentShown) {
+              // If the document wasn't shown, display a message 
+              // with a copiable URL in a message box 
+              JTextArea findImagesMessageTextArea = new JTextArea(preferences.getLocalizedString(
+                  ImportedTextureWizardStepsPanel.class, "findImagesMessage.text"));
+              String findImagesTitle = preferences.getLocalizedString(
+                  ImportedTextureWizardStepsPanel.class, "findImagesMessage.title");
+              findImagesMessageTextArea.setEditable(false);
+              findImagesMessageTextArea.setOpaque(false);
+              JOptionPane.showMessageDialog(SwingUtilities.getRootPane(ImportedTextureWizardStepsPanel.this), 
+                  findImagesMessageTextArea, findImagesTitle, 
+                  JOptionPane.INFORMATION_MESSAGE);
+            }
+          }
+        });
+    } catch (IllegalArgumentException ex) {
+      // Don't create findImagesButton if its text or url isn't defined
+    }
     this.imageChoiceErrorLabel = new JLabel(preferences.getLocalizedString(
         ImportedTextureWizardStepsPanel.class, "imageChoiceErrorLabel.text"));
     // Make imageChoiceErrorLabel visible only if an error occurred during image content loading
@@ -163,25 +199,32 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
         
         @Override
         public boolean importData(JComponent comp, Transferable transferedFiles) {
-          boolean success = true;
+          boolean success = false;
           try {
             List<File> files = (List<File>)transferedFiles.getTransferData(DataFlavor.javaFileListFlavor);
-            final String textureName = files.get(0).getAbsolutePath();
-            EventQueue.invokeLater(new Runnable() {
-                public void run() {
-                  updateController(textureName, controller.getContentManager(), preferences, false);
-                }
-              });
+            for (File file : files) {
+              final String textureName = file.getAbsolutePath();
+              // Try to import the first file that would be accepted by content manager
+              if (controller.getContentManager().isAcceptable(textureName, ContentManager.ContentType.IMAGE)) {
+                EventQueue.invokeLater(new Runnable() {
+                    public void run() {
+                      updateController(textureName, controller.getContentManager(), preferences, false);
+                    }
+                  });
+                success = true;
+                break;
+              }
+            }
           } catch (UnsupportedFlavorException ex) {
-            success = false;
+            // No success
           } catch (IOException ex) {
-            success = false;
+            // No success
           }
           if (!success) {
             EventQueue.invokeLater(new Runnable() {
                 public void run() {
                   JOptionPane.showMessageDialog(SwingUtilities.getRootPane(ImportedTextureWizardStepsPanel.this), 
-                      preferences.getLocalizedString(ImportedTextureWizardStepsPanel.class, "imageChoiceError"));
+                      preferences.getLocalizedString(ImportedTextureWizardStepsPanel.class, "imageChoiceErrorLabel.text"));
                 }
               });
           }
@@ -237,17 +280,20 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
           name = name.trim();
           // If category is empty, replace it by the last selected item
           if (name.length() == 0) {
-            setItem(categoryComboBox.getSelectedItem());
+            Object selectedItem = categoryComboBox.getSelectedItem();
+            setItem(selectedItem);
+            return selectedItem;
+          } else {
+            TexturesCategory category = new TexturesCategory(name);
+            // Search an existing category
+            List<TexturesCategory> categories = preferences.getTexturesCatalog().getCategories();
+            int categoryIndex = Collections.binarySearch(categories, category);
+            if (categoryIndex >= 0) {
+              return categories.get(categoryIndex);
+            }
+            // If no existing category was found, return a new one          
+            return category;
           }
-          TexturesCategory category = new TexturesCategory(name);
-          // Search an existing category
-          List<TexturesCategory> categories = preferences.getTexturesCatalog().getCategories();
-          int categoryIndex = Collections.binarySearch(categories, category);
-          if (categoryIndex >= 0) {
-            return categories.get(categoryIndex);
-          }
-          // If no existing category was found, return a new one          
-          return category;
         }
       
         public void setItem(Object value) {
@@ -357,6 +403,10 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
    */
   private void setMnemonics(UserPreferences preferences) {
     if (!OperatingSystem.isMacOSX()) {
+      if (this.findImagesButton != null) {
+        this.findImagesButton.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
+            ImportedTextureWizardStepsPanel.class, "findImagesButton.mnemonic")).getKeyCode());
+      }
       this.nameLabel.setDisplayedMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
           ImportedTextureWizardStepsPanel.class, "nameLabel.mnemonic")).getKeyCode());
       this.nameLabel.setLabelFor(this.nameTextField);
@@ -381,14 +431,23 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
     
     JPanel imageChoiceTopPanel = new JPanel(new GridBagLayout());
     imageChoiceTopPanel.add(this.imageChoiceOrChangeLabel, new GridBagConstraints(
-        0, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+        0, 0, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
         GridBagConstraints.HORIZONTAL, new Insets(5, 0, 5, 0), 0, 0));
     this.imageChoicePreviewComponent.setPreferredSize(new Dimension(150, 150));
-    imageChoiceTopPanel.add(this.imageChoiceOrChangeButton, new GridBagConstraints(
-        0, 1, 1, 1, 0, 0, GridBagConstraints.CENTER, 
-        GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
+    if (this.findImagesButton != null) {
+      imageChoiceTopPanel.add(this.imageChoiceOrChangeButton, new GridBagConstraints(
+          0, 1, 1, 1, 1, 0, GridBagConstraints.LINE_END, 
+          GridBagConstraints.NONE, new Insets(0, 0, 0, 10), 0, 0));
+      imageChoiceTopPanel.add(this.findImagesButton, new GridBagConstraints(
+          1, 1, 1, 1, 1, 0, GridBagConstraints.LINE_START, 
+          GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
+    } else {
+      imageChoiceTopPanel.add(this.imageChoiceOrChangeButton, new GridBagConstraints(
+          0, 1, 2, 1, 1, 0, GridBagConstraints.CENTER, 
+          GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
+    }
     imageChoiceTopPanel.add(this.imageChoiceErrorLabel, new GridBagConstraints(
-        0, 2, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+        0, 2, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
         GridBagConstraints.NONE, new Insets(5, 0, 0, 0), 0, 0));
     
     JPanel imageChoicePanel = new JPanel(new ProportionalLayout());
@@ -558,7 +617,9 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
                   }
                   controller.setCategory(userCategory);
                   float defaultWidth = 20;
-                  if (preferences.getLengthUnit() == LengthUnit.INCH) {
+                  LengthUnit lengthUnit = preferences.getLengthUnit();
+                  if (lengthUnit == LengthUnit.INCH
+                      || lengthUnit == LengthUnit.INCH_DECIMALS) {
                     defaultWidth = LengthUnit.inchToCentimeter(8);
                   }
                   controller.setWidth(defaultWidth);
