@@ -22,6 +22,7 @@ package com.eteks.sweethome3d.swing;
 import java.awt.BorderLayout;
 import java.awt.CardLayout;
 import java.awt.Component;
+import java.awt.Cursor;
 import java.awt.Dimension;
 import java.awt.EventQueue;
 import java.awt.Graphics;
@@ -461,12 +462,28 @@ public class MultipleLevelsPlanPanel extends JPanel implements PlanView, Printab
   }
 
   /**
-   * Sets the cursor of this component as rotation cursor. 
+   * Sets the cursor of this component. 
    */
   public void setCursor(CursorType cursorType) {
     this.planComponent.setCursor(cursorType);
   }
 
+  /**
+   * Sets the cursor of this component.
+   */
+  @Override
+  public void setCursor(Cursor cursor) {
+    this.planComponent.setCursor(cursor);
+  }
+  
+  /**
+   * Returns the cursor of this component.
+   */
+  @Override
+  public Cursor getCursor() {
+    return this.planComponent.getCursor();
+  }
+  
   /**
    * Sets tool tip text displayed as feedback. 
    */
