@@ -21,6 +21,7 @@ package com.eteks.sweethome3d.swing;
 
 import java.awt.Color;
 import java.awt.Component;
+import java.awt.Cursor;
 import java.awt.Dimension;
 import java.awt.EventQueue;
 import java.awt.Frame;
@@ -30,9 +31,12 @@ import java.awt.GraphicsDevice;
 import java.awt.GraphicsEnvironment;
 import java.awt.GridBagLayout;
 import java.awt.Insets;
+import java.awt.Point;
 import java.awt.Rectangle;
 import java.awt.Toolkit;
 import java.awt.Window;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
 import java.awt.event.ComponentAdapter;
 import java.awt.event.ComponentEvent;
 import java.awt.event.FocusEvent;
@@ -76,6 +80,7 @@ import javax.swing.JSeparator;
 import javax.swing.JToggleButton;
 import javax.swing.JViewport;
 import javax.swing.SwingUtilities;
+import javax.swing.Timer;
 import javax.swing.UIManager;
 import javax.swing.border.AbstractBorder;
 import javax.swing.border.Border;
@@ -227,41 +232,52 @@ public class SwingTools {
   }
 
   /**
-   * Updates the Swing resource bundles in use from the current Locale and class loader. 
+   * Updates the Swing resource bundles in use from the default Locale and class loader. 
    */
   public static void updateSwingResourceLanguage() {
-    updateSwingResourceLanguage(Arrays.asList(new ClassLoader [] {SwingTools.class.getClassLoader()}));
+    updateSwingResourceLanguage(Arrays.asList(new ClassLoader [] {SwingTools.class.getClassLoader()}), null);
   }
 
   /**
-   * Updates the Swing resource bundles in use from the current Locale and the class loaders of preferences.
+   * Updates the Swing resource bundles in use from the preferences Locale and the class loaders of preferences.
    */
   public static void updateSwingResourceLanguage(UserPreferences preferences) {
-    updateSwingResourceLanguage(preferences.getResourceClassLoaders());
+    updateSwingResourceLanguage(preferences.getResourceClassLoaders(), preferences.getLanguage());
   }
 
   /**
-   * Updates the Swing resource bundles in use from the current Locale and class loaders.
+   * Updates the Swing resource bundles in use from the preferences Locale and class loaders.
    */
-  private static void updateSwingResourceLanguage(List<ClassLoader> classLoaders) {
+  private static void updateSwingResourceLanguage(List<ClassLoader> classLoaders,
+                                                  String language) {
+    // Clear resource cache
+    UIManager.getDefaults().removeResourceBundle(null);
+    UIManager.getDefaults().setDefaultLocale(Locale.getDefault());
     // Read Swing localized properties because Swing doesn't update its internal strings automatically
     // when default Locale is updated (see bug http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4884480)
-    updateSwingResourceBundle("com.sun.swing.internal.plaf.metal.resources.metal", classLoaders);
-    updateSwingResourceBundle("com.sun.swing.internal.plaf.basic.resources.basic", classLoaders);
+    updateSwingResourceBundle("com.sun.swing.internal.plaf.metal.resources.metal", classLoaders, language);
+    updateSwingResourceBundle("com.sun.swing.internal.plaf.basic.resources.basic", classLoaders, language);
     if (UIManager.getLookAndFeel().getClass().getName().equals("com.sun.java.swing.plaf.gtk.GTKLookAndFeel")) {
-      updateSwingResourceBundle("com.sun.java.swing.plaf.gtk.resources.gtk", classLoaders);
+      updateSwingResourceBundle("com.sun.java.swing.plaf.gtk.resources.gtk", classLoaders, language);
     } else if (UIManager.getLookAndFeel().getClass().getName().equals("com.sun.java.swing.plaf.motif.MotifLookAndFeel")) {
-      updateSwingResourceBundle("com.sun.java.swing.plaf.motif.resources.motif", classLoaders);
+      updateSwingResourceBundle("com.sun.java.swing.plaf.motif.resources.motif", classLoaders, language);
     } 
   }
 
   /**
    * Updates a Swing resource bundle in use from the current Locale. 
    */
-  private static void updateSwingResourceBundle(String swingResource, List<ClassLoader> classLoaders) {
-    ResourceBundle resource = ResourceBundle.getBundle(swingResource, Locale.ENGLISH);
+  private static void updateSwingResourceBundle(String swingResource, 
+                                                List<ClassLoader> classLoaders,
+                                                String language) {
+    ResourceBundle resource;
     try {
-      Locale defaultLocale = Locale.getDefault();
+      Locale defaultLocale = language == null 
+          ? Locale.getDefault()
+              : (language.indexOf('_') == -1 
+              ? new Locale(language)
+              : new Locale(language.substring(0, 2), language.substring(3, 5)));
+      resource = ResourceBundle.getBundle(swingResource, defaultLocale);
       for (ClassLoader classLoader : classLoaders) {
         ResourceBundle bundle = ResourceBundle.getBundle(swingResource, defaultLocale, classLoader);
         if (defaultLocale.equals(bundle.getLocale())) {
@@ -274,11 +290,36 @@ public class SwingTools {
         }
       }
     } catch (MissingResourceException ex) {
+      resource = ResourceBundle.getBundle(swingResource, Locale.ENGLISH);
     }
-    // Update UIManager properties
+    
+    // Update UIManager properties    
+    final String textAndMnemonicSuffix = ".textAndMnemonic";
     for (Enumeration<?> it = resource.getKeys(); it.hasMoreElements(); ) {
-      String property = (String)it.nextElement();
-      UIManager.put(property, resource.getString(property));
+      String key = (String)it.nextElement();
+      if (key.endsWith(textAndMnemonicSuffix)) {
+        String value = resource.getString(key);
+        UIManager.put(key, value);
+        // Decompose property value like in javax.swing.UIDefaults.TextAndMnemonicHashMap because  
+        // UIDefaults#getResourceCache(Locale) doesn't store the correct localized value for non English resources 
+        // (.textAndMnemonic suffix appeared with Java 1.7) 
+        String text = value.replace("&", "");
+        String keyPrefix = key.substring(0, key.length() - textAndMnemonicSuffix.length());
+        UIManager.put(keyPrefix + "NameText", text);
+        UIManager.put(keyPrefix + "Text", text); 
+        int index = value.indexOf('&');
+        if (index >= 0 && index < value.length() - 1) {
+          UIManager.put(key.replace(textAndMnemonicSuffix, "Mnemonic"), 
+              String.valueOf(Character.toUpperCase(value.charAt(index + 1))));
+        }
+      }
+    }
+    // Store other properties coming from read resource and give them a higher priority if already set in previous loop 
+    for (Enumeration<?> it = resource.getKeys(); it.hasMoreElements(); ) {
+      String key = (String)it.nextElement();
+      if (!key.endsWith(textAndMnemonicSuffix)) {
+        UIManager.put(key, resource.getString(key));
+      }
     }
   }
   
@@ -376,10 +417,12 @@ public class SwingTools {
    */
   public static void deselectAllRadioButtons(JRadioButton ... radioButtons) {
     for (JRadioButton radioButton : radioButtons) {
-      ButtonGroup group = ((JToggleButton.ToggleButtonModel)radioButton.getModel()).getGroup();
-      group.remove(radioButton);
-      radioButton.setSelected(false);
-      group.add(radioButton);
+      if (radioButton != null) {
+        ButtonGroup group = ((JToggleButton.ToggleButtonModel)radioButton.getModel()).getGroup();
+        group.remove(radioButton);
+        radioButton.setSelected(false);
+        group.add(radioButton);
+      }
     }    
   }
   
@@ -402,7 +445,7 @@ public class SwingTools {
       dialog.addComponentListener(new ComponentAdapter() {
           @Override
           public void componentShown(ComponentEvent ev) {
-            focusedComponent.requestFocusInWindow();
+            requestFocus(focusedComponent);
             dialog.removeComponentListener(this);
           }
         });
@@ -418,6 +461,22 @@ public class SwingTools {
     }
   }
 
+  /**
+   * Requests the focus for the given component.
+   */
+  private static void requestFocus(final JComponent focusedComponent) {
+    if (!focusedComponent.requestFocusInWindow()) {
+      // Prefer to call requestFocusInWindow in a timer with a small delay   
+      // than calling it with EnventQueue#invokeLater to ensure it always works
+      new Timer(50, new ActionListener() {
+          public void actionPerformed(ActionEvent ev) {
+            focusedComponent.requestFocusInWindow();
+            ((Timer)ev.getSource()).stop();
+          }
+        }).start();
+    }
+  }
+  
   /**
    * Displays <code>messageComponent</code> in a modal dialog box, giving focus to one of its components. 
    */
@@ -437,7 +496,7 @@ public class SwingTools {
       dialog.addComponentListener(new ComponentAdapter() {
           @Override
           public void componentShown(ComponentEvent ev) {
-            focusedComponent.requestFocusInWindow();
+            requestFocus(focusedComponent);
             dialog.removeComponentListener(this);
           }
         });
@@ -484,9 +543,9 @@ public class SwingTools {
           }
 
           @Override
-          public int filterRGB(int x, int y, int rgba) {
+          public int filterRGB(int x, int y, int argb) {
             // Always use foreground color and alpha
-            return (rgba & 0xFF000000) | foregroundColorRgb;
+            return (argb & 0xFF000000) | foregroundColorRgb;
           }
         })), 0, 0, null);
     imageGraphics.dispose();
@@ -567,7 +626,11 @@ public class SwingTools {
     if (!OperatingSystem.isMacOSXLeopardOrSuperior()) {
       panelBorder = BorderFactory.createCompoundBorder(
           panelBorder, BorderFactory.createEmptyBorder(0, 2, 2, 2));
-    }    
+    } else if (OperatingSystem.isJavaVersionGreaterOrEqual("1.7")) {
+      // Enlarge space at the top of the border
+      panelBorder = BorderFactory.createCompoundBorder(
+          panelBorder, BorderFactory.createEmptyBorder(10, 0, 0, 0));
+    }
     titledPanel.setBorder(panelBorder);    
     return titledPanel;
   }
@@ -760,4 +823,43 @@ public class SwingTools {
     }
     return devicesArea.contains(rectangle);
   }
+  
+  /**
+   * Returns a new custom cursor.
+   */
+  public static Cursor createCustomCursor(URL smallCursorImageUrl, 
+                                          URL largeCursorImageUrl,
+                                          float xCursorHotSpot,
+                                          float yCursorHotSpot,
+                                          String cursorName,
+                                          Cursor defaultCursor) {
+    if (GraphicsEnvironment.isHeadless()) {
+      return defaultCursor;
+    }
+    // Retrieve system cursor size
+    Dimension cursorSize = Toolkit.getDefaultToolkit().getBestCursorSize(16, 16);
+    URL cursorImageResource;
+    // If returned cursor size is 0, system doesn't support custom cursor  
+    if (cursorSize.width == 0) {      
+      return defaultCursor;      
+    } else {
+      // Use a different cursor image depending on system cursor size 
+      if (cursorSize.width > 16) {
+        cursorImageResource = largeCursorImageUrl;
+      } else {
+        cursorImageResource = smallCursorImageUrl;
+      }
+      try {
+        // Read cursor image
+        BufferedImage cursorImage = ImageIO.read(cursorImageResource);
+        // Create custom cursor from image
+        return Toolkit.getDefaultToolkit().createCustomCursor(cursorImage, 
+            new Point(Math.min(cursorSize.width - 1, Math.round(cursorSize.width * xCursorHotSpot)), 
+                      Math.min(cursorSize.height - 1, Math.round(cursorSize.height * yCursorHotSpot))),
+            cursorName);
+      } catch (IOException ex) {
+        throw new IllegalArgumentException("Unknown resource " + cursorImageResource);
+      }
+    }
+  }
 }
