@@ -34,8 +34,10 @@ import java.beans.PropertyChangeListener;
 import java.lang.ref.WeakReference;
 import java.security.AccessControlException;
 import java.util.Arrays;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
+import java.util.Map;
 
 import javax.swing.ButtonGroup;
 import javax.swing.DefaultComboBoxModel;
@@ -75,15 +77,14 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
   private JComboBox      languageComboBox;
   private JButton        languageLibraryImportButton;
   private JLabel         unitLabel;
-  private JRadioButton   centimeterRadioButton;
-  private JRadioButton   inchRadioButton;
-  private JRadioButton   millimeterRadioButton;
-  private JRadioButton   meterRadioButton;
+  private JComboBox      unitComboBox;
   private JLabel         furnitureCatalogViewLabel;
   private JRadioButton   treeRadioButton;
   private JRadioButton   listRadioButton;
   private JLabel         navigationPanelLabel;
   private JCheckBox      navigationPanelCheckBox;
+  private JLabel         aerialViewCenteredOnSelectionLabel;
+  private JCheckBox      aerialViewCenteredOnSelectionCheckBox;
   private JLabel         magnetismLabel;
   private JCheckBox      magnetismCheckBox;
   private JLabel         rulersLabel;
@@ -98,12 +99,16 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
   private JRadioButton   floorColorOrTextureRadioButton;
   private JLabel         wallPatternLabel;
   private JComboBox      wallPatternComboBox;
+  private JLabel         newWallPatternLabel;
+  private JComboBox      newWallPatternComboBox;
   private JLabel         newWallThicknessLabel;
   private JSpinner       newWallThicknessSpinner;
   private JLabel         newWallHeightLabel;
   private JSpinner       newWallHeightSpinner;
   private JLabel         newFloorThicknessLabel;
   private JSpinner       newFloorThicknessSpinner;
+  private JCheckBox      checkUpdatesCheckBox;
+  private JButton        checkUpdatesNowButton;
   private JCheckBox      autoSaveDelayForRecoveryCheckBox;
   private JSpinner       autoSaveDelayForRecoverySpinner;
   private JLabel         autoSaveDelayForRecoveryUnitLabel;
@@ -185,57 +190,38 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
     }
     
     if (controller.isPropertyEditable(UserPreferencesController.Property.UNIT)) {
-      // Create unit label and radio buttons bound to controller UNIT property
-      this.unitLabel = new JLabel(preferences.getLocalizedString(
+      // Create unit label and combo box bound to controller UNIT property
+      this.unitLabel = new JLabel(SwingTools.getLocalizedLabelText(preferences,
           UserPreferencesPanel.class, "unitLabel.text"));
-      this.centimeterRadioButton = new JRadioButton(SwingTools.getLocalizedLabelText(preferences, 
-          UserPreferencesPanel.class, "centimeterRadioButton.text"), 
-          controller.getUnit() == LengthUnit.CENTIMETER);
-      this.centimeterRadioButton.setActionCommand(LengthUnit.CENTIMETER.name());
-      this.inchRadioButton = new JRadioButton(SwingTools.getLocalizedLabelText(preferences, 
-          UserPreferencesPanel.class, "inchRadioButton.text"), 
-          controller.getUnit() == LengthUnit.INCH);
-      this.inchRadioButton.setActionCommand(LengthUnit.INCH.name());
-      this.millimeterRadioButton = new JRadioButton(SwingTools.getLocalizedLabelText(preferences, 
-          UserPreferencesPanel.class, "millimeterRadioButton.text"), 
-          controller.getUnit() == LengthUnit.MILLIMETER);
-      this.millimeterRadioButton.setActionCommand(LengthUnit.MILLIMETER.name());
-      this.meterRadioButton = new JRadioButton(SwingTools.getLocalizedLabelText(preferences, 
-          UserPreferencesPanel.class, "meterRadioButton.text"), 
-          controller.getUnit() == LengthUnit.METER);
-      this.meterRadioButton.setActionCommand(LengthUnit.METER.name());
-      final ButtonGroup unitButtonGroup = new ButtonGroup();
-      unitButtonGroup.add(this.centimeterRadioButton);
-      unitButtonGroup.add(this.inchRadioButton);
-      unitButtonGroup.add(this.millimeterRadioButton);
-      unitButtonGroup.add(this.meterRadioButton);
-
-      ItemListener unitChangeListener = new ItemListener() {
-          public void itemStateChanged(ItemEvent ev) {
-            controller.setUnit(LengthUnit.valueOf(unitButtonGroup.getSelection().getActionCommand())); 
+      this.unitComboBox = new JComboBox(LengthUnit.values());
+      final Map<LengthUnit, String> comboBoxTexts = new HashMap<LengthUnit, String>();
+      comboBoxTexts.put(LengthUnit.MILLIMETER, preferences.getLocalizedString(
+          UserPreferencesPanel.class, "unitComboBox.millimeter.text"));
+      comboBoxTexts.put(LengthUnit.CENTIMETER, preferences.getLocalizedString(
+          UserPreferencesPanel.class, "unitComboBox.centimeter.text"));
+      comboBoxTexts.put(LengthUnit.METER, preferences.getLocalizedString(
+          UserPreferencesPanel.class, "unitComboBox.meter.text"));
+      comboBoxTexts.put(LengthUnit.INCH, preferences.getLocalizedString(
+          UserPreferencesPanel.class, "unitComboBox.inch.text"));
+      comboBoxTexts.put(LengthUnit.INCH_DECIMALS, preferences.getLocalizedString(
+          UserPreferencesPanel.class, "unitComboBox.inchDecimals.text"));
+      this.unitComboBox.setRenderer(new DefaultListCellRenderer() {
+          @Override
+          public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected,
+                                                        boolean cellHasFocus) {
+            return super.getListCellRendererComponent(list, comboBoxTexts.get(value), index, isSelected, cellHasFocus);
           }
-        };
-      this.centimeterRadioButton.addItemListener(unitChangeListener);
-      this.inchRadioButton.addItemListener(unitChangeListener);
-      this.millimeterRadioButton.addItemListener(unitChangeListener);
-      this.meterRadioButton.addItemListener(unitChangeListener);
+        });
+      this.unitComboBox.setSelectedItem(controller.getUnit());
+      this.unitComboBox.addItemListener(new ItemListener() {
+          public void itemStateChanged(ItemEvent ev) {
+            controller.setUnit((LengthUnit)unitComboBox.getSelectedItem());
+          }
+        });
       controller.addPropertyChangeListener(UserPreferencesController.Property.UNIT, 
           new PropertyChangeListener() {
             public void propertyChange(PropertyChangeEvent ev) {
-              switch (controller.getUnit()) {
-                case CENTIMETER :
-                  centimeterRadioButton.setSelected(true);
-                  break;
-                case INCH :
-                  inchRadioButton.setSelected(true);
-                  break;
-                case MILLIMETER :
-                  millimeterRadioButton.setSelected(true);
-                  break;
-                case METER :
-                  meterRadioButton.setSelected(true);
-                  break;
-              }
+              unitComboBox.setSelectedItem(controller.getUnit());
             }
           });
     }
@@ -304,6 +290,25 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
       }
     }
 
+    if (controller.isPropertyEditable(UserPreferencesController.Property.AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED)) {
+      // Create aerialViewCenteredOnSelection label and check box bound to controller AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED property
+      this.aerialViewCenteredOnSelectionLabel = new JLabel(preferences.getLocalizedString(
+          UserPreferencesPanel.class, "aerialViewCenteredOnSelectionLabel.text"));
+      this.aerialViewCenteredOnSelectionCheckBox = new JCheckBox(SwingTools.getLocalizedLabelText(preferences, 
+          UserPreferencesPanel.class, "aerialViewCenteredOnSelectionCheckBox.text"), controller.isAerialViewCenteredOnSelectionEnabled());
+      this.aerialViewCenteredOnSelectionCheckBox.addItemListener(new ItemListener() {
+          public void itemStateChanged(ItemEvent ev) {
+            controller.setAerialViewCenteredOnSelectionEnabled(aerialViewCenteredOnSelectionCheckBox.isSelected());
+          }
+        });
+      controller.addPropertyChangeListener(UserPreferencesController.Property.AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED, 
+          new PropertyChangeListener() {
+            public void propertyChange(PropertyChangeEvent ev) {
+              aerialViewCenteredOnSelectionCheckBox.setSelected(controller.isAerialViewCenteredOnSelectionEnabled());
+            }
+          });
+    }
+
     if (controller.isPropertyEditable(UserPreferencesController.Property.MAGNETISM_ENABLED)) {
       // Create magnetism label and check box bound to controller MAGNETISM_ENABLED property
       this.magnetismLabel = new JLabel(preferences.getLocalizedString(
@@ -428,42 +433,35 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
           });
     }
 
-    if (controller.isPropertyEditable(UserPreferencesController.Property.WALL_PATTERN)) {
+    if (controller.isPropertyEditable(UserPreferencesController.Property.NEW_WALL_PATTERN)) {
+      // Create new wall pattern label and combo box bound to controller NEW_WALL_PATTERN property
+      this.newWallPatternLabel = new JLabel(SwingTools.getLocalizedLabelText(preferences, 
+          UserPreferencesPanel.class, "newWallPatternLabel.text"));    
+      List<TextureImage> patterns = preferences.getPatternsCatalog().getPatterns();
+      this.newWallPatternComboBox = new JComboBox(new DefaultComboBoxModel(patterns.toArray()));
+      this.newWallPatternComboBox.setRenderer(getPatternRenderer());
+      TextureImage newWallPattern = controller.getNewWallPattern();
+      this.newWallPatternComboBox.setSelectedItem(newWallPattern != null 
+          ? newWallPattern  
+          : controller.getWallPattern());
+      this.newWallPatternComboBox.addItemListener(new ItemListener() {
+          public void itemStateChanged(ItemEvent ev) {
+            controller.setNewWallPattern((TextureImage)newWallPatternComboBox.getSelectedItem());
+          }
+        });
+      controller.addPropertyChangeListener(UserPreferencesController.Property.NEW_WALL_PATTERN, 
+          new PropertyChangeListener() {
+            public void propertyChange(PropertyChangeEvent ev) {
+              newWallPatternComboBox.setSelectedItem(controller.getNewWallPattern());
+            }
+          });
+    } else if (controller.isPropertyEditable(UserPreferencesController.Property.WALL_PATTERN)) {
       // Create wall pattern label and combo box bound to controller WALL_PATTERN property
       this.wallPatternLabel = new JLabel(SwingTools.getLocalizedLabelText(preferences, 
           UserPreferencesPanel.class, "wallPatternLabel.text"));    
       List<TextureImage> patterns = preferences.getPatternsCatalog().getPatterns();
       this.wallPatternComboBox = new JComboBox(new DefaultComboBoxModel(patterns.toArray()));
-      this.wallPatternComboBox.setRenderer(new DefaultListCellRenderer() {
-          @Override
-          public Component getListCellRendererComponent(final JList list, 
-              Object value, int index, boolean isSelected, boolean cellHasFocus) {
-            TextureImage wallPattern = (TextureImage)value;
-            final Component component = super.getListCellRendererComponent(
-                list, "", index, isSelected, cellHasFocus);
-            final BufferedImage patternImage = SwingTools.getPatternImage(
-                wallPattern, list.getBackground(), list.getForeground());
-            setIcon(new Icon() {
-                public int getIconWidth() {
-                  return patternImage.getWidth() * 4 + 1;
-                }
-          
-                public int getIconHeight() {
-                  return patternImage.getHeight() + 2;
-                }
-          
-                public void paintIcon(Component c, Graphics g, int x, int y) {
-                  Graphics2D g2D = (Graphics2D)g;
-                  for (int i = 0; i < 4; i++) {
-                    g2D.drawImage(patternImage, x + i * patternImage.getWidth(), y + 1, list);
-                  }
-                  g2D.setColor(list.getForeground());
-                  g2D.drawRect(x, y, getIconWidth() - 2, getIconHeight() - 1);
-                }
-              });
-            return component;
-          }
-        });
+      this.wallPatternComboBox.setRenderer(getPatternRenderer());
       this.wallPatternComboBox.setSelectedItem(controller.getWallPattern());
       this.wallPatternComboBox.addItemListener(new ItemListener() {
           public void itemStateChanged(ItemEvent ev) {
@@ -541,6 +539,32 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
           });
     }
     
+    if (controller.isPropertyEditable(UserPreferencesController.Property.CHECK_UPDATES_ENABLED)) {
+      // Create check box bound to controller CHECK_UPDATES_ENABLED property
+      this.checkUpdatesCheckBox = new JCheckBox(SwingTools.getLocalizedLabelText(preferences, 
+          UserPreferencesPanel.class, "checkUpdatesCheckBox.text"), controller.isCheckUpdatesEnabled());
+      this.checkUpdatesCheckBox.addItemListener(new ItemListener() {
+          public void itemStateChanged(ItemEvent ev) {
+            controller.setCheckUpdatesEnabled(checkUpdatesCheckBox.isSelected());
+          }
+        });
+      controller.addPropertyChangeListener(UserPreferencesController.Property.CHECK_UPDATES_ENABLED, 
+          new PropertyChangeListener() {
+            public void propertyChange(PropertyChangeEvent ev) {
+              checkUpdatesCheckBox.setSelected(controller.isCheckUpdatesEnabled());
+            }
+          });
+      
+      this.checkUpdatesNowButton = new JButton(new ResourceAction.ButtonAction(
+          new ResourceAction(preferences, UserPreferencesPanel.class, "CHECK_UPDATES_NOW", true) {
+            @Override
+            public void actionPerformed(ActionEvent ev) {
+              controller.checkUpdates();
+            }
+          }));
+    }
+    
+
     if (controller.isPropertyEditable(UserPreferencesController.Property.AUTO_SAVE_DELAY_FOR_RECOVERY)) {
       this.autoSaveDelayForRecoveryCheckBox = new JCheckBox(SwingTools.getLocalizedLabelText(preferences, 
           UserPreferencesPanel.class, "autoSaveDelayForRecoveryCheckBox.text"));
@@ -597,6 +621,42 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
     this.dialogTitle = preferences.getLocalizedString(UserPreferencesPanel.class, "preferences.title");
   }
 
+  /**
+   * Returns a renderer for patterns combo box.
+   */
+  private DefaultListCellRenderer getPatternRenderer() {
+    return new DefaultListCellRenderer() {
+        @Override
+        public Component getListCellRendererComponent(final JList list, 
+            Object value, int index, boolean isSelected, boolean cellHasFocus) {
+          TextureImage wallPattern = (TextureImage)value;
+          final Component component = super.getListCellRendererComponent(
+              list, "", index, isSelected, cellHasFocus);
+          final BufferedImage patternImage = SwingTools.getPatternImage(
+              wallPattern, list.getBackground(), list.getForeground());
+          setIcon(new Icon() {
+              public int getIconWidth() {
+                return patternImage.getWidth() * 4 + 1;
+              }
+        
+              public int getIconHeight() {
+                return patternImage.getHeight() + 2;
+              }
+        
+              public void paintIcon(Component c, Graphics g, int x, int y) {
+                Graphics2D g2D = (Graphics2D)g;
+                for (int i = 0; i < 4; i++) {
+                  g2D.drawImage(patternImage, x + i * patternImage.getWidth(), y + 1, list);
+                }
+                g2D.setColor(list.getForeground());
+                g2D.drawRect(x, y, getIconWidth() - 2, getIconHeight() - 1);
+              }
+            });
+          return component;
+        }
+      };
+  }
+
   /**
    * Preferences property listener bound to this component with a weak reference to avoid
    * strong link between preferences and this component.  
@@ -652,14 +712,9 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
         this.languageLabel.setLabelFor(this.languageComboBox);
       }
       if (this.unitLabel != null) {
-        this.centimeterRadioButton.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
-            UserPreferencesPanel.class, "centimeterRadioButton.mnemonic")).getKeyCode());
-        this.inchRadioButton.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
-            UserPreferencesPanel.class, "inchRadioButton.mnemonic")).getKeyCode());
-        this.millimeterRadioButton.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
-            UserPreferencesPanel.class, "millimeterRadioButton.mnemonic")).getKeyCode());
-        this.meterRadioButton.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
-            UserPreferencesPanel.class, "meterRadioButton.mnemonic")).getKeyCode());
+        this.unitLabel.setDisplayedMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
+            UserPreferencesPanel.class, "unitLabel.mnemonic")).getKeyCode());
+        this.unitLabel.setLabelFor(this.unitComboBox);
       }
       if (this.furnitureCatalogViewLabel != null) {
         this.treeRadioButton.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
@@ -675,6 +730,10 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
         this.magnetismCheckBox.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
             UserPreferencesPanel.class, "magnetismCheckBox.mnemonic")).getKeyCode());
       }
+      if (this.aerialViewCenteredOnSelectionLabel != null) {
+        this.aerialViewCenteredOnSelectionCheckBox.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
+            UserPreferencesPanel.class, "aerialViewCenteredOnSelectionCheckBox.mnemonic")).getKeyCode());
+      }
       if (this.rulersLabel != null) {
         this.rulersCheckBox.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
             UserPreferencesPanel.class, "rulersCheckBox.mnemonic")).getKeyCode());
@@ -695,11 +754,15 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
         this.floorColorOrTextureRadioButton.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
             UserPreferencesPanel.class, "floorColorOrTextureRadioButton.mnemonic")).getKeyCode());
       }
-      if (this.wallPatternLabel != null) {
+      if (this.newWallPatternLabel != null) {
+        this.newWallPatternLabel.setDisplayedMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
+            UserPreferencesPanel.class, "newWallPatternLabel.mnemonic")).getKeyCode());
+        this.newWallPatternLabel.setLabelFor(this.newWallPatternComboBox);
+      } else if (this.wallPatternLabel != null) {
         this.wallPatternLabel.setDisplayedMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
             UserPreferencesPanel.class, "wallPatternLabel.mnemonic")).getKeyCode());
         this.wallPatternLabel.setLabelFor(this.wallPatternComboBox);
-      }
+      } 
       if (this.newWallThicknessLabel != null) {
         this.newWallThicknessLabel.setDisplayedMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
             UserPreferencesPanel.class, "newWallThicknessLabel.mnemonic")).getKeyCode());
@@ -715,6 +778,10 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
             UserPreferencesPanel.class, "newFloorThicknessLabel.mnemonic")).getKeyCode());
         this.newFloorThicknessLabel.setLabelFor(this.newFloorThicknessSpinner);
       }
+      if (this.checkUpdatesCheckBox != null) {
+        this.checkUpdatesCheckBox.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
+            UserPreferencesPanel.class, "checkUpdatesCheckBox.mnemonic")).getKeyCode());
+      }      
       if (this.autoSaveDelayForRecoveryCheckBox != null) {
         this.autoSaveDelayForRecoveryCheckBox.setMnemonic(KeyStroke.getKeyStroke(preferences.getLocalizedString(
             UserPreferencesPanel.class, "autoSaveDelayForRecoveryCheckBox.mnemonic")).getKeyCode());
@@ -740,11 +807,11 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
           GridBagConstraints.NONE, labelInsets, 0, 0));
       add(this.languageComboBox, new GridBagConstraints(
           1, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.NONE, new Insets(OperatingSystem.isMacOSX() ? 1 : 0, 0, 5, 5), 0, 0));
+          GridBagConstraints.HORIZONTAL, new Insets(OperatingSystem.isMacOSX() ? 1 : 0, 0, 5, 0), 0, 0));
       if (this.languageLibraryImportButton != null) {
         add(this.languageLibraryImportButton, new GridBagConstraints(
             2, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-            GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));
+            GridBagConstraints.NONE, new Insets(0, 5, 5, 0), 0, 0));
       }
     }
     if (this.unitLabel != null) {
@@ -752,19 +819,10 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
       add(this.unitLabel, new GridBagConstraints(
           0, 1, 1, 1, 0, 0, labelAlignment, 
           GridBagConstraints.NONE, labelInsets, 0, 0));
-      add(this.centimeterRadioButton, new GridBagConstraints(
+      add(this.unitComboBox, new GridBagConstraints(
           1, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.NONE, new Insets(0, 0, 2, 5), 0, 0));
-      add(this.inchRadioButton, new GridBagConstraints(
-          2, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
-      // Third row
-      add(this.millimeterRadioButton, new GridBagConstraints(
-          1, 2, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.NONE, labelInsetsWithSpace, 0, 0));
-      add(this.meterRadioButton, new GridBagConstraints(
-          2, 2, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.NONE, rightComponentInsetsWithSpace, 0, 0));
+          GridBagConstraints.HORIZONTAL, rightComponentInsets, 0, 0));
+      // Keep third row empty (used to contain unit radio buttons)
     }
     if (this.furnitureCatalogViewLabel != null) {
       // Fourth row
@@ -782,106 +840,146 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
       // Fifth row
       add(this.navigationPanelLabel, new GridBagConstraints(
           0, 4, 1, 1, 0, 0, labelAlignment, 
-          GridBagConstraints.NONE, labelInsetsWithSpace, 0, 0));
+          GridBagConstraints.NONE, labelInsets, 0, 0));
       add(this.navigationPanelCheckBox, new GridBagConstraints(
           1, 4, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
+          GridBagConstraints.NONE, rightComponentInsets, 0, 0));
+    }
+    if (this.aerialViewCenteredOnSelectionLabel != null) {
+      // Sixth row
+      add(this.aerialViewCenteredOnSelectionLabel, new GridBagConstraints(
+          0, 5, 1, 1, 0, 0, labelAlignment, 
+          GridBagConstraints.NONE, labelInsetsWithSpace, 0, 0));
+      add(this.aerialViewCenteredOnSelectionCheckBox, new GridBagConstraints(
+          1, 5, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
           GridBagConstraints.NONE, rightComponentInsetsWithSpace, 0, 0));
     }
     if (this.magnetismLabel != null) {
-      // Sixth row
-      add(this.magnetismLabel, new GridBagConstraints(
-          0, 5, 1, 1, 0, 0, labelAlignment, 
-          GridBagConstraints.NONE, labelInsets, 0, 0));
-      add(this.magnetismCheckBox, new GridBagConstraints(
-          1, 5, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.NONE, rightComponentInsets, 0, 0));
-    }
-    if (this.rulersLabel != null) {
       // Seventh row
-      add(this.rulersLabel, new GridBagConstraints(
+      add(this.magnetismLabel, new GridBagConstraints(
           0, 6, 1, 1, 0, 0, labelAlignment, 
           GridBagConstraints.NONE, labelInsets, 0, 0));
-      add(this.rulersCheckBox, new GridBagConstraints(
+      add(this.magnetismCheckBox, new GridBagConstraints(
           1, 6, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
           GridBagConstraints.NONE, rightComponentInsets, 0, 0));
     }
-    if (this.gridLabel != null) {
+    if (this.rulersLabel != null) {
       // Eighth row
-      add(this.gridLabel, new GridBagConstraints(
+      add(this.rulersLabel, new GridBagConstraints(
           0, 7, 1, 1, 0, 0, labelAlignment, 
           GridBagConstraints.NONE, labelInsets, 0, 0));
-      add(this.gridCheckBox, new GridBagConstraints(
+      add(this.rulersCheckBox, new GridBagConstraints(
           1, 7, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
           GridBagConstraints.NONE, rightComponentInsets, 0, 0));
     }
-    if (this.furnitureIconLabel != null) {
+    if (this.gridLabel != null) {
       // Ninth row
-      add(this.furnitureIconLabel, new GridBagConstraints(
+      add(this.gridLabel, new GridBagConstraints(
           0, 8, 1, 1, 0, 0, labelAlignment, 
           GridBagConstraints.NONE, labelInsets, 0, 0));
-      add(this.catalogIconRadioButton, new GridBagConstraints(
-          1, 8, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.NONE, labelInsets, 0, 0));
-      add(this.topViewRadioButton, new GridBagConstraints(
-          2, 8, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.NONE, rightComponentInsets , 0, 0));
+      add(this.gridCheckBox, new GridBagConstraints(
+          1, 8, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
+          GridBagConstraints.NONE, rightComponentInsets, 0, 0));
     }
-    if (this.roomRenderingLabel != null) {
+    if (this.furnitureIconLabel != null) {
       // Tenth row
-      add(this.roomRenderingLabel, new GridBagConstraints(
+      add(this.furnitureIconLabel, new GridBagConstraints(
           0, 9, 1, 1, 0, 0, labelAlignment, 
           GridBagConstraints.NONE, labelInsets, 0, 0));
-      add(this.monochromeRadioButton, new GridBagConstraints(
+      add(this.catalogIconRadioButton, new GridBagConstraints(
           1, 9, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
           GridBagConstraints.NONE, labelInsets, 0, 0));
-      add(this.floorColorOrTextureRadioButton, new GridBagConstraints(
+      add(this.topViewRadioButton, new GridBagConstraints(
           2, 9, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
           GridBagConstraints.NONE, rightComponentInsets , 0, 0));
     }
-    if (this.wallPatternLabel != null) {
+    if (this.roomRenderingLabel != null) {
       // Eleventh row
-      add(this.wallPatternLabel, new GridBagConstraints(
+      add(this.roomRenderingLabel, new GridBagConstraints(
           0, 10, 1, 1, 0, 0, labelAlignment, 
           GridBagConstraints.NONE, labelInsets, 0, 0));
-      add(this.wallPatternComboBox, new GridBagConstraints(
-          1, 10, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.NONE, rightComponentInsets, 0, 0));
+      add(this.monochromeRadioButton, new GridBagConstraints(
+          1, 10, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+          GridBagConstraints.NONE, labelInsets, 0, 0));
+      add(this.floorColorOrTextureRadioButton, new GridBagConstraints(
+          2, 10, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+          GridBagConstraints.NONE, rightComponentInsets , 0, 0));
     }
-    if (this.newWallThicknessLabel != null) {
-      // Twelfth  row
-      add(this.newWallThicknessLabel, new GridBagConstraints(
+    if (this.newWallPatternLabel != null) {
+      // Twelfth row
+      add(this.newWallPatternLabel, new GridBagConstraints(
           0, 11, 1, 1, 0, 0, labelAlignment, 
           GridBagConstraints.NONE, labelInsets, 0, 0));
-      add(this.newWallThicknessSpinner, new GridBagConstraints(
-          1, 11, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.HORIZONTAL, rightComponentInsets, 0, 0));
-    }
-    if (this.newWallHeightLabel != null) {
-      // Thirteenth  row
-      add(this.newWallHeightLabel, new GridBagConstraints(
+      add(this.newWallPatternComboBox, new GridBagConstraints(
+          1, 11, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
+          GridBagConstraints.NONE, rightComponentInsets, 0, 0));
+    } else if (this.wallPatternLabel != null) {
+      // Twelfth row
+      add(this.wallPatternLabel, new GridBagConstraints(
+          0, 11, 1, 1, 0, 0, labelAlignment, 
+          GridBagConstraints.NONE, labelInsets, 0, 0));
+      add(this.wallPatternComboBox, new GridBagConstraints(
+          1, 11, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
+          GridBagConstraints.NONE, rightComponentInsets, 0, 0));
+    } 
+    if (this.newWallThicknessLabel != null) {
+      // Thirteenth row
+      add(this.newWallThicknessLabel, new GridBagConstraints(
           0, 12, 1, 1, 0, 0, labelAlignment, 
           GridBagConstraints.NONE, labelInsets, 0, 0));
-      add(this.newWallHeightSpinner, new GridBagConstraints(
+      add(this.newWallThicknessSpinner, new GridBagConstraints(
           1, 12, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
           GridBagConstraints.HORIZONTAL, rightComponentInsets, 0, 0));
     }
-    if (this.newFloorThicknessLabel != null) {
-      // Fourteenth  row
-      add(this.newFloorThicknessLabel, new GridBagConstraints(
+    if (this.newWallHeightLabel != null) {
+      // Fourteenth row
+      add(this.newWallHeightLabel, new GridBagConstraints(
           0, 13, 1, 1, 0, 0, labelAlignment, 
           GridBagConstraints.NONE, labelInsets, 0, 0));
-      add(this.newFloorThicknessSpinner, new GridBagConstraints(
+      add(this.newWallHeightSpinner, new GridBagConstraints(
           1, 13, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
           GridBagConstraints.HORIZONTAL, rightComponentInsets, 0, 0));
     }
-    if (this.autoSaveDelayForRecoveryCheckBox != null) {
-      JPanel autoSaveDelayForRecoveryPanel = new JPanel();
-      // Fifteenth  row
-      autoSaveDelayForRecoveryPanel.add(this.autoSaveDelayForRecoveryCheckBox);
-      autoSaveDelayForRecoveryPanel.add(this.autoSaveDelayForRecoverySpinner);
-      autoSaveDelayForRecoveryPanel.add(this.autoSaveDelayForRecoveryUnitLabel);
-      add(autoSaveDelayForRecoveryPanel, new GridBagConstraints(
-          0, 14, 3, 1, 0, 0, GridBagConstraints.LINE_START, 
+    if (this.newFloorThicknessLabel != null) {
+      // Fifteenth row
+      add(this.newFloorThicknessLabel, new GridBagConstraints(
+          0, 14, 1, 1, 0, 0, labelAlignment, 
+          GridBagConstraints.NONE, labelInsets, 0, 0));
+      add(this.newFloorThicknessSpinner, new GridBagConstraints(
+          1, 14, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+          GridBagConstraints.HORIZONTAL, rightComponentInsets, 0, 0));
+    }
+    if (this.checkUpdatesCheckBox != null
+        || this.autoSaveDelayForRecoveryCheckBox != null) {
+      JPanel updatesAndAutoSaveDelayForRecoveryPanel = new JPanel(new GridBagLayout());
+      // Sixteenth row
+      if (this.checkUpdatesCheckBox != null) {
+        updatesAndAutoSaveDelayForRecoveryPanel.add(this.checkUpdatesCheckBox,
+            new GridBagConstraints(
+                0, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+                GridBagConstraints.NONE, labelInsets, 0, 0));
+        updatesAndAutoSaveDelayForRecoveryPanel.add(this.checkUpdatesNowButton,
+            new GridBagConstraints(
+                1, 0, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
+                GridBagConstraints.NONE, rightComponentInsets, 0, 0));
+      }
+      // Seventeenth row
+      if (this.autoSaveDelayForRecoveryCheckBox != null) {
+        updatesAndAutoSaveDelayForRecoveryPanel.add(this.autoSaveDelayForRecoveryCheckBox,
+            new GridBagConstraints(
+                0, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+                GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
+        updatesAndAutoSaveDelayForRecoveryPanel.add(this.autoSaveDelayForRecoverySpinner,
+            new GridBagConstraints(
+                1, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+                GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
+        updatesAndAutoSaveDelayForRecoveryPanel.add(this.autoSaveDelayForRecoveryUnitLabel,
+            new GridBagConstraints(
+                2, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
+                GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
+      }
+      add(updatesAndAutoSaveDelayForRecoveryPanel, new GridBagConstraints(
+          0, 15, 3, 1, 0, 0, GridBagConstraints.LINE_START, 
           GridBagConstraints.HORIZONTAL, rightComponentInsets, 0, 0));
     }
     // Last row
@@ -889,7 +987,7 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
         && this.resetDisplayedActionTipsButton.getText().length() > 0) {
       // Display reset button only if its text isn't empty 
       add(this.resetDisplayedActionTipsButton, new GridBagConstraints(
-          0, 15, 3, 1, 0, 0, GridBagConstraints.CENTER, 
+          0, 16, 3, 1, 0, 0, GridBagConstraints.CENTER, 
           GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
     }
   }
@@ -933,18 +1031,19 @@ public class UserPreferencesPanel extends JPanel implements DialogView {
       float lengthInCentimeter = getLength();
       unit = controller.getUnit();
       switch (controller.getUnit()) {
-        case CENTIMETER :
-          setStepSize(centimeterStepSize);
-          break;
-        case INCH :
-          setStepSize(inchStepSize);
-          break;
         case MILLIMETER :
           setStepSize(millimeterStepSize);
           break;
+        case CENTIMETER :
+          setStepSize(centimeterStepSize);
+          break;
         case METER :
           setStepSize(meterStepSize);
           break;
+        case INCH :
+        case INCH_DECIMALS :
+          setStepSize(inchStepSize);
+          break;
       }
       setLength(lengthInCentimeter);
     }
