@@ -67,6 +67,7 @@ import java.io.InputStream;
 import java.io.InterruptedIOException;
 import java.io.OutputStream;
 import java.lang.ref.WeakReference;
+import java.lang.reflect.Field;
 import java.text.DateFormat;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
@@ -80,6 +81,7 @@ import java.util.Hashtable;
 import java.util.List;
 import java.util.ListIterator;
 import java.util.Locale;
+import java.util.Map;
 import java.util.TimeZone;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
@@ -899,14 +901,12 @@ public class VideoPanel extends JPanel implements DialogView {
     add(new JLabel(), new GridBagConstraints(
         3, 3, 1, 6, 0.5f, 0, GridBagConstraints.CENTER, 
         GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
-    Insets labelInsets = new Insets(0, 0, 0, 5);
     add(this.videoFormatLabel, new GridBagConstraints(
         1, 3, 1, 1, 0, 0, labelAlignment, 
-        GridBagConstraints.NONE, labelInsets, 0, 0));
-    Insets componentInsets = new Insets(0, 0, 0, 10);
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
     add(this.videoFormatComboBox, new GridBagConstraints(
         2, 3, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.HORIZONTAL, componentInsets, -50, 0));
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 10), -50, 0));
     // Fifth row
     add(this.qualityLabel, new GridBagConstraints(
         1, 4, 1, 1, 0, 0, labelAlignment, 
@@ -1081,7 +1081,7 @@ public class VideoPanel extends JPanel implements DialogView {
       // Record only new locations
       cameraPath = new ArrayList<Camera>(cameraPath);
       Camera recordedCamera = camera.clone();
-      recordedCamera.setLens(Camera.Lens.NORMAL);
+      recordedCamera.setLens(Camera.Lens.PINHOLE);
       recordedCamera.setTime(this.controller.getTime());
       cameraPath.add(recordedCamera);
       this.controller.setCameraPath(cameraPath);
@@ -1183,7 +1183,7 @@ public class VideoPanel extends JPanel implements DialogView {
    */
   private void deleteCameraPath() {
     List<Camera> cameraPath = Collections.emptyList();
-    this.controller.setCameraPath(cameraPath );
+    this.controller.setCameraPath(cameraPath);
   }
 
   /**
@@ -1301,7 +1301,7 @@ public class VideoPanel extends JPanel implements DialogView {
             yaw + deltaYawAngle * j, pitch + deltaPitchAngle * j, 
             fieldOfView + deltaFieldOfViewAngle * j,
             time + deltaTime * j,
-            Camera.Lens.NORMAL));
+            Camera.Lens.PINHOLE));
       }
       
       x = newX;
@@ -1652,6 +1652,7 @@ public class VideoPanel extends JPanel implements DialogView {
     private final javax.media.format.VideoFormat format;
     private int                                  imageIndex;
     private boolean                              stopped;
+    private Object                               mainThreadContext;
 
     public ImageSourceStream(VideoFormat format, 
                              FrameGenerator frameGenerator,
@@ -1661,6 +1662,12 @@ public class VideoPanel extends JPanel implements DialogView {
       this.framesPath = framesPath;
       this.progressModel = progressModel;
       this.format = format;      
+      try {
+        // Retrieve main thread AppContext instance by reflection         
+        this.mainThreadContext = Class.forName("sun.awt.AppContext").getMethod("getAppContext").invoke(null);
+      } catch (Throwable ex) {
+        // Let's consider app context is not necessary for the program
+      }
     }
 
     /**
@@ -1680,10 +1687,11 @@ public class VideoPanel extends JPanel implements DialogView {
       if (endOfStream()) {
         buffer.setEOM(true);
         buffer.setLength(0);
-      } else {  
+      } else {
+        checkAppContext();
         ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
         BufferedImage frame = this.frameGenerator.renderImageAt(this.framesPath [this.imageIndex],
-            this.imageIndex == this.framesPath.length - 1);           
+            this.imageIndex == this.framesPath.length - 1);
         ImageIO.write(frame, "JPEG", outputStream);
         byte [] data = outputStream.toByteArray();
         buffer.setData(data);
@@ -1700,6 +1708,31 @@ public class VideoPanel extends JPanel implements DialogView {
       }
     }
 
+    /**
+     * Checks that app context is correct.
+     */
+    private void checkAppContext() {
+      if (this.mainThreadContext != null) {
+        try {
+          // Check by reflection that sun.awt.AppContext.getAppContext() doesn't return null 
+          // (required by ImageIO.write() and other JMF internal calls) to apply workaround proposed at 
+          // http://stackoverflow.com/questions/17223304/appcontext-is-null-from-rmi-thread-with-java-7-update-25
+          Class<?> appContextClass = Class.forName("sun.awt.AppContext");
+          if (appContextClass.getMethod("getAppContext").invoke(null) == null) {
+            Field field = appContextClass.getDeclaredField("threadGroup2appContext");
+            field.setAccessible(true);
+            Map threadGroup2appContext = (Map)field.get(null);
+            ThreadGroup currentThreadGroup = Thread.currentThread().getThreadGroup();
+            threadGroup2appContext.put(currentThreadGroup, this.mainThreadContext);
+          }
+        } catch (Throwable ex) {
+          // Let's consider app context is not necessary for the program
+        }
+        // Don't need mainThreadContext anymore
+        this.mainThreadContext = null;
+      }
+    }
+
     /**
      * Return the format of each video frame. That will be JPEG.
      */
