@@ -22,6 +22,7 @@ package com.eteks.sweethome3d.tools;
 import java.io.File;
 import java.io.FileFilter;
 import java.io.IOException;
+import java.math.BigInteger;
 import java.security.AccessControlException;
 import java.util.ArrayList;
 import java.util.Comparator;
@@ -104,8 +105,7 @@ public class OperatingSystem {
   }
 
   /**
-   * Returns <code>true</code> if current operating is Mac OS X 10.5 
-   * or superior.
+   * Returns <code>true</code> if current operating is Mac OS X 10.5 or superior.
    */
   public static boolean isMacOSXLeopardOrSuperior() {
     // Just need to test is OS version is different of 10.4 because Sweet Home 3D
@@ -114,6 +114,15 @@ public class OperatingSystem {
         && !System.getProperty("os.version").startsWith("10.4");
   }
   
+  /**
+   * Returns <code>true</code> if current operating is Mac OS X 10.7 or superior.
+   * @since 4.1
+   */
+  public static boolean isMacOSXLionOrSuperior() {
+    return isMacOSX()
+        && compareVersions(System.getProperty("os.version"), "10.7") >= 0;
+  }
+  
   /**
    * Returns <code>true</code> if the given version is greater than or equal to the version 
    * of the current JVM. 
@@ -123,6 +132,17 @@ public class OperatingSystem {
     return compareVersions(javaMinimumVersion, System.getProperty("java.version")) <= 0;
   }
 
+  /**
+   * Returns <code>true</code> if the version of the current JVM is greater or equal to the 
+   * <code>javaMinimumVersion</code> and smaller than <code>javaMaximumVersion</code>. 
+   * @since 4.2
+   */
+  public static boolean isJavaVersionBetween(String javaMinimumVersion, String javaMaximumVersion) {
+    String javaVersion = System.getProperty("java.version");
+    return compareVersions(javaMinimumVersion, javaVersion) <= 0 
+        && compareVersions(javaVersion, javaMaximumVersion) < 0;
+  }
+
   /**
    * Returns a negative number if <code>version1</code> &lt; <code>version2</code>,
    * 0 if <code>version1</code> = <code>version2</code>
@@ -165,10 +185,10 @@ public class OperatingSystem {
     for ( ; i < version1Parts.size() || i < version2Parts.size(); i++) {
       Object version1Part = i < version1Parts.size() 
           ? convertPreReleaseVersion(version1Parts.get(i))
-          : 0; // Missing part is considered as 0
+          : BigInteger.ZERO; // Missing part is considered as 0
       Object version2Part = i < version2Parts.size() 
           ? convertPreReleaseVersion(version2Parts.get(i))
-          : 0;
+          : BigInteger.ZERO;
       if (version1Part.getClass() == version2Part.getClass()) {
         @SuppressWarnings({"unchecked", "rawtypes"})
         int comparison = ((Comparable)version1Part).compareTo(version2Part);
@@ -201,7 +221,7 @@ public class OperatingSystem {
           for ( ; i < part.length() && Character.isDigit(c = part.charAt(i)); i++) {
             subPart.append(c);
           }
-          versionParts.add(new Integer(subPart.toString()));
+          versionParts.add(new BigInteger(subPart.toString()));
         } else {
           for ( ; i < part.length() && !Character.isDigit(c = part.charAt(i)); i++) {
             subPart.append(c);
@@ -221,11 +241,11 @@ public class OperatingSystem {
     if (versionPart instanceof String) {
       String versionPartString = (String)versionPart;
       if ("alpha".equalsIgnoreCase(versionPartString)) {
-        return -3;
+        return new BigInteger("-3");
       } else if ("beta".equalsIgnoreCase(versionPartString)) {
-        return -2;
+        return new BigInteger("-2");
       } else if ("rc".equalsIgnoreCase(versionPartString)) {
-        return -1;
+        return new BigInteger("-1");
       }
     }
     return versionPart;
@@ -249,12 +269,23 @@ public class OperatingSystem {
   }
   
   /**
-   * Returns a file comparator that sorts file names according to their version number. 
+   * Returns a file comparator that sorts file names according to their version number (excluding their extension when they are the same). 
    */
   public static Comparator<File> getFileVersionComparator() {
     return new Comparator<File>() {
         public int compare(File file1, File file2) {
-          return OperatingSystem.compareVersions(file1.getName(), file2.getName());
+          String fileName1 = file1.getName();
+          String fileName2 = file2.getName();
+          int extension1Index = fileName1.lastIndexOf('.');
+          String extension1 = extension1Index != -1  ? fileName1.substring(extension1Index)  : null;
+          int extension2Index = fileName2.lastIndexOf('.');
+          String extension2 = extension2Index != -1  ? fileName2.substring(extension2Index)  : null;
+          // If the files have the same extension, remove it 
+          if (extension1 != null && extension1.equals(extension2)) {
+            fileName1 = fileName1.substring(0, extension1Index);
+            fileName2 = fileName2.substring(0, extension2Index);
+          }
+          return OperatingSystem.compareVersions(fileName1, fileName2);
         }
       };
   }
@@ -282,7 +313,12 @@ public class OperatingSystem {
    */
   private synchronized static File getDefaultTemporaryFolder(boolean create) throws IOException {
     if (TEMPORARY_SUB_FOLDER != null) {
-      File temporaryFolder = new File(getDefaultApplicationFolder(), TEMPORARY_SUB_FOLDER);
+      File temporaryFolder;
+      if (new File(TEMPORARY_SUB_FOLDER).isAbsolute()) {
+        temporaryFolder = new File(TEMPORARY_SUB_FOLDER);
+      } else {
+        temporaryFolder = new File(getDefaultApplicationFolder(), TEMPORARY_SUB_FOLDER);
+      }
       final String versionPrefix = Home.CURRENT_VERSION + "-";
       final File sessionTemporaryFolder = new File(temporaryFolder, 
           versionPrefix + TEMPORARY_SESSION_SUB_FOLDER);      
@@ -296,10 +332,8 @@ public class OperatingSystem {
           });
         
         // Create temporary folder  
-        if (!sessionTemporaryFolder.mkdirs()) {
+        if (!createTemporaryFolders(sessionTemporaryFolder)) {
           throw new IOException("Can't create temporary folder " + sessionTemporaryFolder);
-        } else {
-          sessionTemporaryFolder.deleteOnExit();
         }
         
         // Launch a timer that updates modification date of the temporary folder each minute
@@ -342,6 +376,35 @@ public class OperatingSystem {
     }
   }
   
+  /**
+   * Creates the temporary folders in parameters and returns <code>true</code> if it was successful.
+   */
+  private static boolean createTemporaryFolders(File temporaryFolder) {
+    // Inspired from java.io.File#mkdirs
+    if (temporaryFolder.exists()) {
+      return false;
+    }
+    if (temporaryFolder.mkdir()) {
+      temporaryFolder.deleteOnExit();
+      return true;
+    }
+    File canonicalFile = null;
+    try {
+      canonicalFile = temporaryFolder.getCanonicalFile();
+    } catch (IOException e) {
+      return false;
+    }
+    File parent = canonicalFile.getParentFile();
+    if (parent != null 
+        && (createTemporaryFolders(parent) || parent.exists()) 
+        && canonicalFile.mkdir()) {
+      temporaryFolder.deleteOnExit();
+      return true;
+    } else {
+      return false;
+    }
+  }
+  
   /**
    * Returns default application folder. 
    */
