@@ -430,8 +430,7 @@ public class ImportedFurnitureWizardController extends WizardController
    * Sets the name of the imported piece.
    */
   public void setName(String name) {
-    if (name != this.name
-        || (name != null && !name.equals(this.name))) {
+    if (name != this.name) {
       String oldName = this.name;
       this.name = name;
       if (this.propertyChangeSupport != null) {
@@ -594,8 +593,7 @@ public class ImportedFurnitureWizardController extends WizardController
    * Sets the shape used to cut out upper levels at its intersection with a staircase.
    */
   public void setStaircaseCutOutShape(String staircaseCutOutShape) {
-    if (staircaseCutOutShape != this.staircaseCutOutShape
-        || (staircaseCutOutShape != null && !staircaseCutOutShape.equals(this.staircaseCutOutShape))) {
+    if (staircaseCutOutShape != this.staircaseCutOutShape) {
       String oldStaircaseCutOutShape = this.staircaseCutOutShape;
       this.staircaseCutOutShape = staircaseCutOutShape;
       if (this.propertyChangeSupport != null) {
@@ -723,21 +721,19 @@ public class ImportedFurnitureWizardController extends WizardController
    * Furniture model step state (first step).
    */
   private class FurnitureModelStepState extends ImportedFurnitureWizardStepState {
-    public FurnitureModelStepState() {
-      ImportedFurnitureWizardController.this.addPropertyChangeListener(Property.MODEL, 
-          new PropertyChangeListener() {
-              public void propertyChange(PropertyChangeEvent evt) {
-                setNextStepEnabled(getModel() != null);
-              }
-            });
-    }
-
+    private PropertyChangeListener modelChangeListener = new PropertyChangeListener() {
+        public void propertyChange(PropertyChangeEvent ev) {
+          setNextStepEnabled(getModel() != null);
+        }
+      };
+      
     @Override
     public void enter() {
       super.enter();
       setFirstStep(true);
       // First step is valid once a model is available
       setNextStepEnabled(getModel() != null);
+      addPropertyChangeListener(Property.MODEL, this.modelChangeListener);
     }
 
     @Override
@@ -749,6 +745,11 @@ public class ImportedFurnitureWizardController extends WizardController
     public void goToNextStep() {
       setStepState(getFurnitureOrientationStepState());
     }
+    
+    @Override
+    public void exit() {
+      removePropertyChangeListener(Property.MODEL, this.modelChangeListener);
+    }
   }
 
   /**
