@@ -25,6 +25,7 @@ import java.util.Locale;
 import java.util.concurrent.Callable;
 
 import javax.swing.JCheckBox;
+import javax.swing.JComboBox;
 import javax.swing.JRadioButton;
 import javax.swing.JSpinner;
 import javax.swing.undo.UndoableEditSupport;
@@ -42,6 +43,7 @@ import com.eteks.sweethome3d.model.UserPreferences;
 import com.eteks.sweethome3d.swing.FileContentManager;
 import com.eteks.sweethome3d.swing.SwingViewFactory;
 import com.eteks.sweethome3d.swing.UserPreferencesPanel;
+import com.eteks.sweethome3d.tools.OperatingSystem;
 import com.eteks.sweethome3d.tools.URLContent;
 import com.eteks.sweethome3d.viewcontroller.BackgroundImageWizardController;
 import com.eteks.sweethome3d.viewcontroller.CompassController;
@@ -56,6 +58,7 @@ import com.eteks.sweethome3d.viewcontroller.ImportedFurnitureWizardController;
 import com.eteks.sweethome3d.viewcontroller.ImportedTextureWizardController;
 import com.eteks.sweethome3d.viewcontroller.LabelController;
 import com.eteks.sweethome3d.viewcontroller.LevelController;
+import com.eteks.sweethome3d.viewcontroller.ModelMaterialsController;
 import com.eteks.sweethome3d.viewcontroller.ObserverCameraController;
 import com.eteks.sweethome3d.viewcontroller.PageSetupController;
 import com.eteks.sweethome3d.viewcontroller.PhotoController;
@@ -96,7 +99,7 @@ public class UserPreferencesPanelTest extends TestCase {
     preferences.setFurnitureViewedFromTop(
         defaultPreferences.isFurnitureViewedFromTop());
     preferences.setFloorColoredOrTextured(
-        defaultPreferences.isFurnitureViewedFromTop());
+        defaultPreferences.isRoomFloorColoredOrTextured());
     preferences.setNewWallThickness(
         defaultPreferences.getNewWallThickness());
     preferences.setNewWallHeight(
@@ -106,14 +109,8 @@ public class UserPreferencesPanelTest extends TestCase {
     UserPreferencesController controller = 
         new UserPreferencesController(preferences, new SwingViewFactory(), null);
     UserPreferencesPanel panel = (UserPreferencesPanel)controller.getView();
-    JRadioButton centimeterRadioButton = 
-        (JRadioButton)TestUtilities.getField(panel, "centimeterRadioButton");
-    JRadioButton inchRadioButton = 
-        (JRadioButton)TestUtilities.getField(panel, "inchRadioButton");
-    JRadioButton meterRadioButton = 
-        (JRadioButton)TestUtilities.getField(panel, "meterRadioButton");
-    JRadioButton millimeterRadioButton = 
-        (JRadioButton)TestUtilities.getField(panel, "millimeterRadioButton");
+    JComboBox unitComboBox = 
+        (JComboBox)TestUtilities.getField(panel, "unitComboBox");
     JCheckBox    magnetismCheckBox = 
         (JCheckBox)TestUtilities.getField(panel, "magnetismCheckBox");
     JCheckBox    rulersCheckBox = 
@@ -133,29 +130,27 @@ public class UserPreferencesPanelTest extends TestCase {
     JSpinner newHomeWallHeightSpinner = 
         (JSpinner)TestUtilities.getField(panel, "newWallHeightSpinner");
     // Check panel components value
-    assertTrue("Centimeter radio button isn't selected", centimeterRadioButton.isSelected());
-    assertFalse("Inch radio button is selected",  inchRadioButton.isSelected());
-    assertFalse("Meter radio button is selected", meterRadioButton.isSelected());
-    assertFalse("Millimeter radio button is selected", millimeterRadioButton.isSelected());
+    assertEquals("Centimeter isn't the current unit", LengthUnit.CENTIMETER, unitComboBox.getSelectedItem());
     assertTrue("Magnestism isn't selected", magnetismCheckBox.isSelected());
     assertTrue("Rulers isn't selected", rulersCheckBox.isSelected());
     assertTrue("Grid isn't selected", gridCheckBox.isSelected());
-    assertTrue("Catalog icon radio button isn't selected", catalogIconRadioButton.isSelected());
-    assertFalse("Top view button is selected",  topViewRadioButton.isSelected());
-    assertTrue("Monochrome radio button isn't selected", monochromeRadioButton.isSelected());
-    assertFalse("Floor color radio button is selected",  floorColorOrTextureRadioButton.isSelected());
+    boolean macOSXOrWindows = OperatingSystem.isMacOSX() || OperatingSystem.isWindows();
+    assertTrue("Catalog icon radio button isn't selected", catalogIconRadioButton.isSelected() != macOSXOrWindows);
+    assertFalse("Top view button is selected",  topViewRadioButton.isSelected() != macOSXOrWindows);
+    assertTrue("Monochrome radio button isn't selected", monochromeRadioButton.isSelected() != macOSXOrWindows);
+    assertFalse("Floor color radio button is selected",  floorColorOrTextureRadioButton.isSelected() != macOSXOrWindows);
     assertEquals("Wrong default thickness", 
         newWallThicknessSpinner.getValue(), defaultPreferences.getNewWallThickness());
     assertEquals("Wrong default wall height", 
         newHomeWallHeightSpinner.getValue(), defaultPreferences.getNewWallHeight());
     
     // 3. Change panel values
-    inchRadioButton.setSelected(true);
+    unitComboBox.setSelectedItem(LengthUnit.INCH);
     magnetismCheckBox.setSelected(false);
     rulersCheckBox.setSelected(false);
     gridCheckBox.setSelected(false);
-    topViewRadioButton.setSelected(true);
-    floorColorOrTextureRadioButton.setSelected(true);
+    catalogIconRadioButton.setSelected(true);
+    monochromeRadioButton.setSelected(true);
     newWallThicknessSpinner.setValue(1);
     newHomeWallHeightSpinner.setValue(100);
     
@@ -163,7 +158,7 @@ public class UserPreferencesPanelTest extends TestCase {
     controller.modifyUserPreferences();
     // Check preferences value
     assertPreferencesEqual(LengthUnit.INCH, false, false, false,
-        true, true,
+        false, false,
         LengthUnit.inchToCentimeter(1), 
         LengthUnit.inchToCentimeter(100), 
         preferences);
@@ -343,6 +338,7 @@ public class UserPreferencesPanelTest extends TestCase {
       new VideoController(home, preferences, viewFactory, contentManager).getView();
       
       new TextureChoiceController("", preferences, viewFactory, contentManager).getView();
+      new ModelMaterialsController("", preferences, viewFactory, contentManager).getView();
       new ThreadedTaskController(new Callable<Void>() { 
           public Void call() throws Exception {
             return null;
